@@ -344,7 +344,13 @@ not supported."
        (let ()
          #+sb-thread
          (sb-impl::finalizer-thread-stop)
-         (sb-thread::with-all-threads-lock
+         ;; Acquiring sb-thread::*make-thread-lock* prevents creation
+         ;; of new threads.
+         (sb-int:with-system-mutex (sb-thread::*make-thread-lock*)
+           ;; Dead threads aren't pruned from *ALL-THREADS* until the Pthread join.
+           ;; Do that now so that the forked process has only the main thread
+           ;; in *ALL-THREADS* and nothing in *JOINABLE-THREADS*.
+           #+pauseless-threadstart (sb-thread::join-pthread-joinables #'identity)
            (when (let ((avltree sb-thread::*all-threads*))
                    (or (sb-thread::avlnode-left avltree)
                        (sb-thread::avlnode-right avltree)))
@@ -504,11 +510,9 @@ not supported."
 (define-call "mlockall" int minusp (flags int))
 (define-call "munlockall" int minusp)
 
-#-win32
-(define-call "getpagesize" int minusp)
-#+win32
-;;; KLUDGE: This could be taken from GetSystemInfo
-(export (defun getpagesize () 4096))
+(export 'getpagesize)
+(declaim (inline getpagesize))
+(defun getpagesize () (extern-alien "os_reported_page_size" (unsigned 32)))
 
 ;;; passwd database
 ;; The docstrings are copied from the descriptions in SUSv3,
