@@ -11,7 +11,8 @@
   (:use #:common-lisp)
   (:import-from #:sb-kernel #:ansi-stream #:charpos #:line-length)
   (:import-from #:sb-gray #:fundamental-stream)
-  #+sb-package-locks
+  (:import-from #:sb-impl
+                #:in-stream-from-designator #:out-stream-from-designator)
   ;; FIXME: Using deffoo! or equivalent might be nicer.
   (:implement #:common-lisp #:sb-kernel #:sb-int)
   (:export ;; Stream classes
@@ -69,6 +70,8 @@
    #:PRIN1-TO-STRING #:PRINC #:PRINC-TO-STRING #:PRINT #:READ
    #:READ-DELIMITED-LIST #:READ-FROM-STRING #:WRITE #:WRITE-LINE
    #:WRITE-TO-STRING #:READ-PRESERVING-WHITESPACE))
+(eval-when (:compile-toplevel :load-toplevel :execute)
+  (setf (sb-int:system-package-p (find-package "SB-SIMPLE-STREAMS")) t))
 
 #||
 (in-package "SB-EXT")
