@@ -16,7 +16,7 @@
 ;;;   absolute address
 (defconstant +elements-per-sample+ 2)
 
-;;; 0 the symbol TRACE-START
+;;; 0 the trace start marker (trace-start . END-INDEX)
 ;;; 1 the current thread, an SB-THREAD:THREAD instance
 ;;; 2 the current internal real time
 (defconstant +elements-per-trace-start+ 3)
@@ -43,6 +43,18 @@
   ;; the old contents.
   (vector          nil                  :type simple-vector)
   (index           0                    :type sb-int:index)
+  ;; Only used while recording a trace. Stores a cell that is
+  ;;
+  ;;   (start-trace . nil)   (start-trace is the marker symbol
+  ;;                          SB-SPROF:START-TRACE)
+  ;;
+  ;; when starting to record a trace and
+  ;;
+  ;;   (start-trace . END-INDEX-OF-THE-TRACE)
+  ;;
+  ;; after finishing recording the trace. This allows MAP-TRACES to
+  ;; jump from one trace to the next in O(1) instead of O(N) time.
+  (trace-start     '(nil . nil)         :type cons)
   (trace-count     0                    :type sb-int:index)
 
   (sampled-threads nil                  :type list)
@@ -89,10 +101,15 @@
 (defun record-trace-start (samples)
   ;; Mark the start of the trace.
   (multiple-value-bind (vector index) (ensure-samples-vector samples)
-    (setf (aref vector index) 'trace-start
-          (aref vector (+ index 1)) sb-thread:*current-thread*
-          (aref vector (+ index 2)) (get-internal-real-time))
-    (setf (samples-index samples) (+ index +elements-per-trace-start+))))
+    (let ((trace-start (cons 'trace-start nil)))
+      (setf (aref vector index)       trace-start
+            (aref vector (+ index 1)) sb-thread:*current-thread*
+            (aref vector (+ index 2)) (get-internal-real-time))
+      (setf (samples-index samples)       (+ index +elements-per-trace-start+)
+            (samples-trace-start samples) trace-start))))
+
+(defun record-trace-end (samples)
+  (setf (cdr (samples-trace-start samples)) (samples-index samples)))
 
 (declaim (inline record-sample))
 (defun record-sample (samples info pc-or-offset)
@@ -124,15 +141,13 @@ EXPERIMENTAL: Interface subject to change."
         (index (samples-index samples))
         (start-time (samples-start-time samples)))
     (when (plusp index)
-      (sb-int:aver (eq (aref vector 0) 'trace-start))
+      (sb-int:aver (typep (aref vector 0) '(cons (eql trace-start) index)))
       (loop for start = 0 then end
             while (< start index)
+            for end = (cdr (aref vector start))
             for thread = (aref vector (+ start 1))
             for time = (/ (- (aref vector (+ start 2)) start-time)
                           internal-time-units-per-second)
-            for end = (or (position 'trace-start vector
-                                    :start (+ start +elements-per-trace-start+))
-                          index)
             do (let ((trace (list vector start end)))
                  (funcall function thread time trace))))))
 
@@ -185,7 +200,7 @@ EXPERIMENTAL: Interface subject to change."
   (etypecase info
     ;; Assembly routines or foreign functions don't move around, so
     ;; we've stored a raw PC
-    ((or null sb-kernel:code-component string)
+    ((or null sb-kernel:code-component string symbol)
      pc-or-offset)
     ;; Lisp functions might move, so we've stored a offset from the
     ;; start of the code component.
@@ -242,14 +257,17 @@ EXPERIMENTAL: Interface subject to change."
 (defun debug-info (pc)
   (declare (type system-area-pointer pc)
            (muffle-conditions compiler-note))
-  (let ((code (sb-di::code-header-from-pc pc)))
+  (let ((code (sb-di::code-header-from-pc pc))
+        (pc-int (sap-int pc)))
     (cond ((not code)
            (let ((name (sap-foreign-symbol pc)))
              (if name
                  (values (format nil "foreign function ~a" name)
-                         (sap-int pc)
-                         :foreign)
-                 (values nil (sap-int pc) :foreign))))
+                         pc-int :foreign)
+                 (values nil pc-int :foreign))))
+          ((eq code sb-fasl:*assembler-routines*)
+           (values (sb-disassem::find-assembler-routine pc-int)
+                   pc-int :asm-routine))
           (t
            (let* ((code-header-len (* (sb-kernel:code-header-words code)
                                       sb-vm:n-word-bytes))
@@ -262,15 +280,15 @@ EXPERIMENTAL: Interface subject to change."
                   (di (unless (typep (sb-kernel:%code-debug-info code)
                                      'sb-c::compiled-debug-info)
                         (return-from debug-info
-                          (values code (sap-int pc)))))
-                  (pc-offset (- (sap-int pc)
+                          (values code pc-int nil))))
+                  (pc-offset (- pc-int
                                 (- (sb-kernel:get-lisp-obj-address code)
                                    sb-vm:other-pointer-lowtag)
                                 code-header-len))
                   (df (sb-di::debug-fun-from-pc code pc-offset)))
              #+immobile-code (declare (ignorable di))
              (cond ((typep df 'sb-di::bogus-debug-fun)
-                    (values code (sap-int pc) nil))
+                    (values code pc-int nil))
                    (df
                     ;; The code component might be moved by the GC. Store
                     ;; a PC offset, and reconstruct the data in
@@ -297,6 +315,8 @@ EXPERIMENTAL: Interface subject to change."
   (let ((profiled-threads *profiled-threads*))
     (remove *timer-thread*
             (if (eq :all profiled-threads)
+                ;; FIXME: inefficient, probably memoize on a best-effort basis:
+                ;;  detect whether the tree has changed, and recompute only if it did.
                 (sb-thread:list-all-threads)
                 profiled-threads))))
 
@@ -312,11 +332,6 @@ EXPERIMENTAL: Interface subject to change."
   (defvar *profiler-lock* (sb-thread:make-mutex :name "Statistical Profiler"))
   (defvar *distribution-lock* (sb-thread:make-mutex :name "Wallclock profiling lock"))
 
-  #+sb-thread
-  (declaim (inline pthread-kill))
-  #+sb-thread
-  (define-alien-routine pthread-kill int (os-thread unsigned-long) (signal int))
-
   ;;; A random thread will call this in response to either a timer firing,
   ;;; This in turn will distribute the notice to those threads we are
   ;;; interested using SIGPROF.
@@ -326,15 +341,13 @@ EXPERIMENTAL: Interface subject to change."
     (let ((lock *distribution-lock*))
       ;; Don't flood the system with more interrupts if the last
       ;; set is still being delivered.
-      (unless (sb-thread:mutex-value lock)
-        (sb-thread::with-system-mutex (lock)
+      (unless (sb-thread:mutex-owner lock)
+        (with-system-mutex (lock)
           (dolist (thread (profiled-threads))
-            ;; This may occasionally fail to deliver the signal, but that
-            ;; seems better then using kill_thread_safely with it's 1
-            ;; second backoff.
-            (let ((os-thread (sb-thread::thread-os-thread thread)))
-              (when os-thread
-                (pthread-kill os-thread sb-unix:sigprof)))))))
+            (sb-thread:with-deathlok (thread c-thread)
+              (unless (= c-thread 0)
+                (sb-thread:pthread-kill (sb-thread::thread-os-thread thread)
+                                        sb-unix:sigprof)))))))
     #-sb-thread
     (unix-kill 0 sb-unix:sigprof))
 
@@ -360,14 +373,19 @@ EXPERIMENTAL: Interface subject to change."
                  ;; pointless there -- though it may be that our mach magic is
                  ;; partially to blame?
                  (or (not (eq :cpu profiling)) (profiled-thread-p self)))
-        (sb-thread::with-system-mutex (*profiler-lock* :without-gcing t)
-          (let ((samples *samples*))
+        (with-code-pages-pinned (:dynamic)
+         (with-system-mutex (*profiler-lock*)
+          (let ((samples *samples*)
+                ;; Don't touch the circularity hash-table
+                *print-circle*)
             (when (and samples
                        (< (samples-trace-count samples)
                           (samples-max-samples samples)))
               (with-alien ((scp (* os-context-t) :local scp))
                 (let* ((pc-ptr (sb-vm:context-pc scp))
-                       (fp (sb-vm::context-register scp #.sb-vm::ebp-offset)))
+                       (fp (sb-vm::context-register scp
+                                                    #+x86-64 #.sb-vm::rbp-offset
+                                                    #+x86 #.sb-vm::ebp-offset)))
                   ;; foreign code might not have a useful frame
                   ;; pointer in ebp/rbp, so make sure it looks
                   ;; reasonable before walking the stack
@@ -406,11 +424,12 @@ EXPERIMENTAL: Interface subject to change."
                           (decf (samples-index samples)
                                 (+ +elements-per-trace-start+
                                    (* +elements-per-sample+ (1+ i)))))
-                        (return))))))
+                        (return)))
+                    (record-trace-end samples))))
               ;; Reset thread-local allocation counter before interrupts
               ;; are enabled.
               (when (eq t sb-vm::*alloc-signal*)
-                (setf sb-vm:*alloc-signal* (1- (samples-alloc-interval samples)))))))))
+                (setf sb-vm:*alloc-signal* (1- (samples-alloc-interval samples))))))))))
     nil))
 
 ;; FIXME: On non-x86 platforms we don't yet walk the call stack deeper
@@ -434,7 +453,8 @@ EXPERIMENTAL: Interface subject to change."
                 (unless (eq (record samples pc-ptr) :foreign)
                   (record samples (sap-ref-sap
                                    (int-sap fp)
-                                   (* sb-vm::lra-save-offset sb-vm::n-word-bytes))))))))))))
+                                   (* sb-vm::lra-save-offset sb-vm::n-word-bytes)))))
+              (record-trace-end samples))))))))
 
 ;;; Return the start address of CODE.
 (defun code-start (code)
@@ -445,5 +465,5 @@ EXPERIMENTAL: Interface subject to change."
 (defun code-bounds (code)
   (declare (type sb-kernel:code-component code))
   (let* ((start (code-start code))
-         (end (+ start (sb-kernel:%code-code-size code))))
+         (end (+ start (sb-kernel:%code-text-size code))))
     (values start end)))
