@@ -16,14 +16,17 @@
         ;; discard and reasons they're not needed. The default assumption should be
         ;; to drop any build-time feature that lacks a rationale to preserve it.
         ;;
-        '(;; :SB-AFTER-XC-CORE is essentially an option flag to make-host-2
+        ;; READ-FROM-STRING prevents making references to
+        ;; all these keywords from the source form itself.
+        (read-from-string "
+         (;; :SB-AFTER-XC-CORE is essentially an option flag to make-host-2
           :SB-AFTER-XC-CORE
           ;; CONS-PROFILING sets the initial compiler policy which persists
           ;; into the default baseline policy. It has no relevance post-build
           ;; in as much as policy can be changed later arbitrarily.
           :CONS-PROFILING
           ;; Used by nothing after compiling 'target-thread.lisp' or 'thread.c'
-          :SB-FUTEX :FUTEX-USE-TID :OS-THREAD-STACK
+          :SB-FUTEX :OS-THREAD-STACK
           ;; These affect the BREAK instruction emitter, but the C code is able
           ;; to handle anything, and post-build we don't care which it is.
           :UD2-BREAKPOINTS :INT4-BREAKPOINTS
@@ -32,15 +35,16 @@
           ;; more-or-less confined to serve-event, except for a test which now
           ;; detects whether COMPUTE-POLLFDS is defined and therefore testable.
           :OS-PROVIDES-POLL
-          ;; The final batch of symbols is strictly for C. The prefix of
-          ;; "LISP_FEATURE_" on the corresponding #define is unfortunate.
+          ;; The final batch of symbols is strictly for C. The LISP_FEATURE_
+          ;; prefix on the corresponding #define is unfortunate.
           :GCC-TLS :USE-SYS-MMAP
           :RESTORE-FS-SEGMENT-REGISTER-FROM-TLS ; only for 'src/runtime/thread.h'
-          :OS-PROVIDES-BLKSIZE-T)) ; only for 'src/runtime/wrap.h'
+          :OS-PROVIDES-BLKSIZE-T)")) ; only for 'src/runtime/wrap.h'
        (public-features
         (cons
          sb-impl::!sbcl-architecture
-         '(:COMMON-LISP :SBCL :ANSI-CL :IEEE-FLOATING-POINT
+         (read-from-string "
+          (:COMMON-LISP :SBCL :ANSI-CL :IEEE-FLOATING-POINT
            :64-BIT ; choice of word size. 32-bit if absent
            :BIG-ENDIAN :LITTLE-ENDIAN ; endianness: pick one and only one
            :BSD :UNIX :LINUX :WIN32 :DARWIN :SUNOS :ANDROID ; OS: pick one or more
@@ -67,7 +71,7 @@
            :PACKAGE-LOCAL-NICKNAMES
            ;; Developer mode features. A release build will never have them,
            ;; hence it makes no difference whether they're public or not.
-           :SB-FLUID :SB-DEVEL)))
+           :SB-FLUID :SB-DEVEL)")))
        (removable-features
         (append non-target-features public-features)))
   (defconstant sb-impl:+internal-features+
