@@ -12,8 +12,7 @@
 
 (in-package "SB-VM")
 
-(!define-load-time-global *allocator-mutex*
-    #.(sb-thread:make-mutex :name "Allocator"))
+(!define-load-time-global *allocator-mutex* (sb-thread:make-mutex :name "Allocator"))
 
 (defun allocate-static-vector (widetag length words)
   (declare (type (unsigned-byte #.n-widetag-bits) widetag)
@@ -25,7 +24,7 @@
   ;; To think about why it is OK to leave GC enabled, consider that
   ;; neither GC nor another thread will examine static space above the
   ;; current value of *STATIC-SPACE-FREE-POINTER*.
-  (or (sb-thread::with-system-mutex (*allocator-mutex*)
+  (or (with-system-mutex (*allocator-mutex*)
         (let* ((pointer *static-space-free-pointer*)
                (nbytes (pad-data-block (+ words vector-data-offset)))
                (new-pointer (sap+ pointer nbytes)))
@@ -279,7 +278,7 @@
   (setq n-bytes (align-up n-bytes (* 2 n-word-bytes)))
   ;; Can't allocate fewer than 4 words due to min hole size.
   (aver (>= n-bytes (* 4 n-word-bytes)))
-  (sb-thread::with-system-mutex (*allocator-mutex* :without-gcing t)
+  (with-system-mutex (*allocator-mutex* :without-gcing t)
    (unless (zerop varyobj-holes)
      ;; If deferred sweep needs to happen, do so now.
      ;; Concurrency could potentially be improved here: at most one thread
@@ -425,15 +424,15 @@
 ;;; 25 bits is the maximum unboxed size expressed in bytes,
 ;;; if n-word-bytes = 8 and almost the entire code object is unboxed
 ;;; which is, practically speaking, not really possible.
-(declaim (ftype (sfunction (t (unsigned-byte 22) (unsigned-byte 25))
+(declaim (ftype (sfunction (t (unsigned-byte 16) (unsigned-byte 22) (unsigned-byte 25))
                            code-component)
                 allocate-code-object))
 ;;; Allocate a code component with BOXED words in the header
 ;;; followed by UNBOXED bytes of raw data.
 ;;; BOXED must be the exact count of boxed words desired. No adjustments
 ;;; are made for alignment considerations or the fixed slots.
-(defun allocate-code-object (space boxed unboxed)
-  (declare (ignorable space))
+(defun allocate-code-object (space n-named-calls boxed unboxed)
+  (declare (ignorable space n-named-calls))
   (let* ((total-words
            (the (unsigned-byte 22) ; Enforce limit on total words as well
                 (align-up (+ boxed (ceiling unboxed n-word-bytes)) 2)))
@@ -467,19 +466,33 @@
            (%primitive var-alloc total-words 'alloc-code
                        ;; subtract 1 because var-alloc always adds 1 word
                        ;; for the header, which is not right for code objects.
-                       -1 code-header-widetag other-pointer-lowtag)))
-    ;; The 1st slot beyond the header stores the boxed header size in bytes
-    ;; as an untagged number, which has the same representation as a tagged
-    ;; value denoting a word count if WORD-SHIFT = N-FIXNUM-TAG-BITS.
-    ;; This slot is allowed to be 0 prior to writing any pointer descriptors
-    ;; into the object.
-    ;;
-    ;; If 64-bit words, assign a serial number unless the space is NIL.
-    ;; Use ATOMIC-INCF on the serialno to get automatic wraparound,
-    ;; and not because atomicity makes things deterministic, which it doesn't
-    ;; if there are several threads allocating code.
-    ;; TODO: this unnecessarily calls CODE-HEADER-SET if code cards use soft
-    ;; marking. Maybe pin the object and use (SETF SAP-REF-WORD) instead.
+                       -1 code-header-widetag other-pointer-lowtag nil)))
+
+    (with-pinned-objects (code)
+      (let ((sap (sap+ (int-sap (get-lisp-obj-address code))
+                       (- other-pointer-lowtag))))
+        ;; The immobile space allocator pre-zeroes, and also it needs a nonzero
+        ;; value in the boxed word count because otherwise it looks like
+        ;; an immobile space page filler. So don't do any more zeroing there.
+        ;; (Could dead immobile objects be converted to use FILLER-WIDETAG instead?)
+        (unless (immobile-space-obj-p code)
+          ;; Before writing the boxed word count, zeroize up to and including 1 word
+          ;; after the boxed header so that all point words can be safely read
+          ;; by GC and so the jump table count word is 0.
+          (loop for byte-index from (ash boxed word-shift) downto (ash 2 word-shift)
+                by n-word-bytes
+                do (setf (sap-ref-word sap byte-index) 0)))
+        ;; The 1st slot beyond the header stores the boxed header size in bytes
+        ;; as an untagged number, which has the same representation as a tagged
+        ;; value denoting a word count if WORD-SHIFT = N-FIXNUM-TAG-BITS.
+        ;; This boxed-size MUST be 0 prior to writing any pointers into the object
+        ;; because the boxed words will not necessarily have been pre-zeroed;
+        ;; scavenging them prior to zeroing them out would see wild pointers.
+        (setf (sap-ref-word sap (ash code-boxed-size-slot word-shift))
+              ;; For 32-bit words, we'll have to add another primitive-object slot.
+              ;; But so far nothing makes use of the n-named-calls value.
+              (logior #+64-bit (ash n-named-calls 32)
+                      (ash boxed word-shift)))))
 
     ;; FIXME: Sort out 64-bit and cheneygc.
     #+(and 64-bit cheneygc)
@@ -487,10 +500,4 @@
           (%make-lisp-obj
            (logior (ash total-words 32)
                    sb-vm:code-header-widetag)))
-    (setf (code-header-ref code code-boxed-size-slot)
-          (%make-lisp-obj
-           (logior (ash boxed word-shift)
-                   #+64-bit
-                   (logand (ash (atomic-incf sb-fasl::*code-serialno*) 32)
-                           most-positive-word))))
     code))
