@@ -192,10 +192,12 @@
                              (* code-slots-per-simple-fun (code-n-entries code1)))
                below (code-header-words code1)
                always (eq (code-header-ref code1 i) (code-header-ref code2 i)))
-         ;; Compare unboxed constants
-         (let ((nwords (ceiling (code-n-unboxed-data-bytes code1) n-word-bytes))
-               (sap1 (code-instructions code1))
-               (sap2 (code-instructions code2)))
+         ;; jump table word contains serial# which is arbitrary; don't compare it
+         (= (code-jump-table-words code1) (code-jump-table-words code2))
+         ;; Compare unboxed constants less 1 word which was already compared
+         (let ((nwords (1- (ceiling (code-n-unboxed-data-bytes code1) n-word-bytes)))
+               (sap1 (sap+ (code-instructions code1) n-word-bytes))
+               (sap2 (sap+ (code-instructions code2) n-word-bytes)))
            (dotimes (i nwords t)
              (unless (= (sap-ref-word sap1 (ash i word-shift))
                         (sap-ref-word sap2 (ash i word-shift)))
@@ -227,19 +229,35 @@
              (return nil))))))
 
 ;;; Compute a key for binning purposes.
-(defun compute-code-hash-key (code)
-  (with-pinned-objects (code)
-    (list* (code-header-words code)
-           (collect ((offs))
-             (dotimes (i (code-n-entries code) (offs))
-               (offs (%code-fun-offset code i)
-                     (%simple-fun-text-len (%code-entry-point code i) i))))
-           ;; Ignore the debug-info, fixups, and simple-fun metadata.
-           ;; (Same things that are ignored by the CODE-EQUIVALENT-P predicate)
-           (loop for i from (+ code-constants-offset
-                               (* code-slots-per-simple-fun (code-n-entries code)))
-                   below (code-header-words code)
-                   collect (code-header-ref code i)))))
+(defun compute-code-hash-key (code constants)
+  (flet ((constant-to-moniker (object)
+           ;; Prevent EQUAL from descending into certain objects by assigning
+           ;; a sequential integer as its moniker.
+           ;; - CONS is obvious: just don't do it.
+           ;; - Do not collapse STRING, because doing so would break the concept
+           ;;   of similarity as applied to strings of differing element types
+           ;;   which are EQUAL but dissimilar.
+           ;; - Do not collapse PATHNAME because those contain strings.
+           ;; - Additionally, as I intend to implement lazy stable hash values on
+           ;;   all INSTANCE types stored in EQUAL tables, assign them a moniker
+           ;;   so that extra GC work is avoided. (This subsumes PATHNAME too)
+           ;; BIT-VECTOR is the only remaining nontrivial type for which EQUAL
+           ;; does anything other than EQL. That's fine.
+           (if (typep object '(or cons instance string pathname))
+               (ensure-gethash object constants (hash-table-count constants))
+               object)))
+    (with-pinned-objects (code)
+      (list* (code-header-words code)
+             (collect ((offs))
+               (dotimes (i (code-n-entries code) (offs))
+                 (offs (%code-fun-offset code i)
+                       (%simple-fun-text-len (%code-entry-point code i) i))))
+             ;; Ignore the debug-info, fixups, and simple-fun metadata.
+             ;; (Same things that are ignored by the CODE-EQUIVALENT-P predicate)
+             (loop for i from (+ code-constants-offset
+                                 (* code-slots-per-simple-fun (code-n-entries code)))
+                     below (code-header-words code)
+                     collect (constant-to-moniker (code-header-ref code i)))))))
 
 (declaim (inline default-allow-icf-p))
 (defun default-allow-icf-p (code)
@@ -262,7 +280,7 @@
                        sb-c::deftransform
                        :source-transform))))))
 
-(defun fold-identical-code (&key aggressive preserve-docstrings print)
+(defun fold-identical-code (&key aggressive preserve-docstrings (print nil))
   (loop
     #+gencgc (gc :gen 7)
     ;; Pass 1: count code objects.  I'd like to enhance MAP-ALLOCATED-OBJECTS
@@ -289,7 +307,12 @@
                       (plusp (code-n-entries obj)))
              (setf (aref code-objects i) obj)
              (incf i)))
-         :all))
+         :all)
+        ;; GC in between the first and second heap walk might somehow free
+        ;; a code object. It's possible apparently, because a user encountered
+        ;; an error at (COMPUTE-CODE-HASH-KEY 0) in save-lisp-and-die.
+        (unless (= i (length code-objects))
+          (%shrink-vector code-objects i)))
       (unless aggressive
         ;; Figure out which of those are referenced by any object
         ;; except for an fdefn.
@@ -319,12 +342,16 @@
            :all)))
       ;; Now place objects that possibly match into the same bin.
       (let ((bins (make-hash-table :test 'equal))
+            ;; Constants all need to be treated as though they are atoms.
+            ;; Map each one to a fixnum so that the EQUAL table doesn't
+            ;; recurse into constants during the binning step.
+            (constants (make-hash-table :test 'eq))
             (n 0))
         (dovector (x code-objects)
           (when (or (not (gethash x referenced-objects))
                     (default-allow-icf-p x))
             (incf n)
-            (push x (gethash (compute-code-hash-key x) bins))))
+            (push x (gethash (compute-code-hash-key x constants) bins))))
         (when print
           (format t "ICF: ~d objects, ~d candidates, ~d bins~%"
                   (length code-objects) n (hash-table-count bins)))
@@ -358,15 +385,12 @@
                                               (name (and entry
                                                          (sb-kernel:%fun-name entry))))
                                          (and (symbolp name)
+                                              (symbol-package name)
                                               (eq (nth-value 1 (find-symbol (symbol-name name)
                                                                             (symbol-package name)))
                                                   :external))))
                                      (keyfn (x)
-                                       #+64-bit
-                                       (%code-serialno x)
-                                       ;; 32-bit code objects will need another slot
-                                       #-64-bit
-                                       (bug "Need to pick a deterministic order key for ~A" x))
+                                       (%code-serialno x))
                                      (compare (a b)
                                        (let ((exported-a (exported-p a))
                                              (exported-b (exported-p b) ))
