@@ -43,7 +43,7 @@
 
 ;;;; utilities for reading from fasl files
 
-#-sb-fluid (declaim (inline read-byte))
+(declaim (inline read-byte))
 
 ;;; This expands into code to read an N-byte unsigned integer using
 ;;; FAST-READ-BYTE.
@@ -349,7 +349,7 @@
                (sbcl-version (if (<= fasl-version 76)
                                  "1.0.11.18"
                                  (string-from-stream)))
-               (expected-version (sb-xc:lisp-implementation-version)))
+               (expected-version (lisp-implementation-version)))
           (push fasl-version results)
           (unless (string= expected-version sbcl-version)
             (restart-case
