@@ -173,12 +173,10 @@
 
 ;;; predicates for testing character attributes
 
-#-sb-fluid
-(progn
-  (declaim (inline whitespace[1]p whitespace[2]p))
-  (declaim (inline constituentp terminating-macrop))
-  (declaim (inline single-escape-p multiple-escape-p))
-  (declaim (inline token-delimiterp)))
+(declaim (inline whitespace[1]p whitespace[2]p))
+(declaim (inline constituentp terminating-macrop))
+(declaim (inline single-escape-p multiple-escape-p))
+(declaim (inline token-delimiterp))
 
 ;;; the [1] and [2] here refer to ANSI glossary entries for
 ;;; "whitespace".
@@ -1683,8 +1681,8 @@ extended <package-name>::<form-in-package> syntax."
            (inline token-buf-getchar)) ; makes for smaller code
   (let* ((fixnum-max-digits
           (macrolet ((maxdigits ()
-                       (sb-xc:coerce (integer-reader-safe-digits)
-                                     '(vector (unsigned-byte 8)))))
+                       (coerce (integer-reader-safe-digits)
+                               '(vector (unsigned-byte 8)))))
             (aref (maxdigits) (- base 2))))
          (base-power
           (macrolet ((base-powers ()
