@@ -11,6 +11,16 @@
 
 (in-package "SB-THREAD")
 
+;;; It's possible to make futex/non-futex switchable at runtime by ensuring that
+;;; these synchronization primitive structs contain all the slots for the union
+;;; of any kind of backing object.  Some of the #+sb-futex/#-sb-futex cases in
+;;; target-thread also have to be changed to a COND rather than a compile-time test.
+;;; Among the uses of this would be to test real posix mutexes (with a finalizer on
+;;; each lisp-side proxy) or support older C APIs. Posix mutexes work nicely now
+;;; because finalizers are more efficient than they were, and when many threads
+;;; compete for a mutex, the pthread code seems to do a better job at reducing
+;;; cycles spent in the OS.
+
 ;;; N.B.: If you alter this definition, then you need to verify that FOP-FUNCALL
 ;;; in genesis can properly emulate MAKE-MUTEX for the altered structure,
 ;;; or even better, make sure that genesis can emulate any constructor,
@@ -18,15 +28,41 @@
 (sb-xc:defstruct (mutex (:constructor make-mutex (&key name))
                         (:copier nil))
   "Mutex type."
-  ;; C code could (but doesn't currently) access the name
-  #+(and sb-thread sb-futex)
-  (state    0 :type fixnum)
+  #+sb-futex (state 0 :type sb-vm:word)
   ;; If adding slots between STATE and NAME, please see futex_name() in linux_os.c
   ;; which attempts to divine a string from a futex word address.
   (name   nil :type (or null simple-string))
   (%owner nil :type (or null thread)))
 
-(sb-xc:defstruct (thread (:constructor %make-thread (name %ephemeral-p))
+(sb-xc:defstruct (waitqueue (:copier nil) (:constructor make-waitqueue (&key name)))
+  "Waitqueue type."
+  ;; futex words are actually 32-bits, but it needs to be a raw slot and we don't have
+  ;; 32-bit raw slots on 64-bit machines.
+  #+sb-futex (token 0 :type sb-ext:word)
+  ;; If adding slots between TOKEN and NAME, please see futex_name() in linux_os.c
+  ;; which attempts to divine a string from a futex word address.
+  (name nil :type (or null string))
+  ;; For WITH-CAS-LOCK: because CONDITION-WAIT must be able to call
+  ;; %WAITQUEUE-WAKEUP without re-aquiring the mutex, we need a separate
+  ;; lock. In most cases this should be uncontested thanks to the mutex --
+  ;; the only case where that might not be true is when CONDITION-WAIT
+  ;; unwinds and %WAITQUEUE-DROP is called.
+  . #+sb-futex nil
+    #-sb-futex (%owner %head %tail))
+
+(sb-xc:defstruct (semaphore (:copier nil)
+                            (:constructor %make-semaphore (%count mutex queue)))
+  "Semaphore type. The fact that a SEMAPHORE is a STRUCTURE-OBJECT
+should be considered an implementation detail, and may change in the
+future."
+  (%count    0 :type (integer 0))
+  (waitcount 0 :type sb-vm:word)
+  (mutex nil :read-only t :type mutex)
+  (queue nil :read-only t :type waitqueue))
+
+(declaim (sb-ext:freeze-type waitqueue semaphore))
+
+(sb-xc:defstruct (thread (:constructor %make-thread (name %ephemeral-p semaphore))
                          (:copier nil))
   "Thread type. Do not rely on threads being structs as it may change
 in future versions."
@@ -65,13 +101,15 @@ in future versions."
   ;; user-visible threads that do not map directly to OSs threads (or LWPs).
   ;; Any use of THREAD-OS-THREAD from lisp should take care to ensure validity of
   ;; the thread id by holding the INTERRUPTIONS-LOCK.
-  (os-thread 0 :type sb-vm:word)
-  ;; Keep a copy of CONTROL-STACK-END from the "primitive" thread.
-  ;; Reading that memory for any thread except *CURRENT-THREAD* is not safe
-  ;; due to possible unmapping on thread death.
+  ;; Not needed for win32 threads.
+  #-win32 (os-thread 0 :type sb-vm:word)
+  ;; Keep a copy of the stack range for use in SB-EXT:STACK-ALLOCATED-P so that
+  ;; we don't have to read it from the primitive thread which is unsafe for any
+  ;; thread other than the current thread.
   ;; Usually this is a fixed amount below PRIMITIVE-THREAD, but the exact offset
   ;; varies by build configuration, and if #+win32 it is not related in any way.
-  (stack-end 0 :type sb-vm:word)
+  (control-stack-start 0 :type sb-vm:word)
+  (control-stack-end 0 :type sb-vm:word)
   ;; At the beginning of the thread's life, this is a vector of data required
   ;; to start the user code. At the end, is it pointer to the 'struct thread'
   ;; so that it can be either freed or reused.
@@ -81,19 +119,24 @@ in future versions."
   ;; might have been known as the %ALIVE-P flag.
   (%visible 1 :type fixnum)
   (interruptions nil :type list)
-  ;; On succesful execution of the thread's lambda a list of values.
-  (result 0)
   (interruptions-lock
    (make-mutex :name "thread interruptions lock")
    :type mutex :read-only t)
-  (result-lock
-   (make-mutex :name "thread result lock")
-   :type (or null mutex) :read-only t)
+  ;; On succesful execution of the thread's lambda, a list of values.
+  (result 0)
+  ;; The completion condition _could_ be manifested as a condition var, but a difficulty
+  ;; in doing so is that condition vars can always experience a spurious wakeup.
+  ;; Dealing with timeouts becomes troublesome then. But we can utilize the fact that
+  ;; WAIT-ON-SEMPAHORE implements a timeout, though as its comment says, the timeout
+  ;; doesn't account for re-acquiring the internal mutex if that takes nontrivial time,
+  ;; which it shouldn't since it guards very little work.
+  ;; At any rate, the semaphore abstraction is never subject to spurious wakeup.
+  (semaphore nil :type (or null semaphore))
   waiting-for)
 
 (sb-xc:defstruct (foreign-thread
                   (:copier nil)
-                  (:include thread (result-lock nil) (name "callback"))
+                  (:include thread (name "callback"))
                   (:constructor make-foreign-thread ())
                   (:conc-name "THREAD-"))
   "Type of native threads which are attached to the runtime as Lisp threads
@@ -107,4 +150,4 @@ temporarily.")
   "Asynchronous signal handling thread."
   (signal-number nil :type integer))
 
-#-sb-xc-host (declaim (sb-ext:freeze-type mutex thread))
+(declaim (sb-ext:freeze-type mutex thread))
