@@ -173,6 +173,7 @@
     #+sb-thread *stop-for-gc-pending*
     ;; non-x86oid gencgc object pinning
     #+(and gencgc (not (or x86 x86-64))) *pinned-objects*
+    #+gencgc (*gc-pin-code-pages* 0)
     ;; things needed for non-local-exit
     (*current-catch-block* 0)
     (*current-unwind-protect-block* 0)
@@ -198,8 +199,6 @@
 
     #+immobile-space *immobile-freelist* ; not per-thread (yet...)
 
-    #+hpux *c-lra*
-
     ;; stack pointers
     #-sb-thread *binding-stack-start* ; a thread slot if #+sb-thread
     #-sb-thread *control-stack-start* ; ditto
@@ -230,12 +229,27 @@
 ;;; Refer to the lengthy comment in 'src/runtime/interrupt.h' about
 ;;; the choice of this number. Rather than have to two copies
 ;;; of the comment, please see that file before adjusting this.
-(defconstant max-interrupts 1024)
+;;; I think most of the need for a ridiculously large value stemmed from
+;;; receive-pending-interrupt after a pseudo-atomic code section.
+;;; If that trapped into GC, and then ran finalizers in POST-GC (while still
+;;; in a signal handler), which consed, which caused the need for another GC,
+;;; you'd receive a nested interrupt, as the GC trap was still on the stack
+;;; not having returned to "user" code yet. [See example in src/code/final]
+;;;
+;;; But now that #+sb-thread creates a dedicated finalizer thread, nesting
+;;; seems unlikely to occur, because "your" code doesn't get a chance to run again
+;;; until after the interrupt returns. And the finalizer thread won't invoke
+;;; run-pending-finalizers in post-GC, it will just pick up the next finalizer
+;;; in due turn. You could potentially force a nested GC by consing a lot in
+;;; a post-GC hook, but if you do that, your hook function is badly behaved
+;;; and you should fix it.
+(defconstant max-interrupts
+  #+sb-thread    8  ; reasonable value
+  #-sb-thread 1024) ; crazy value
+
 ;;; Thread slots accessed at negative indices relative to struct thread.
-;;; These slots encroach on the interrupt contexts- the maximum that
-;;; can actually be stored is decreased by this amount.
-;;; sb-safepoint puts the safepoint page immediately preceding the
-;;; thread structure, so this trick doesn't work.
+;;; sb-safepoint needs to access one word below thread-base-tn for the
+;;; foreign call safepoint trap word, so this trick doesn't work.
 (defconstant thread-header-slots
   (+ #+(and x86-64 (not sb-safepoint)) 16))
 
