@@ -88,20 +88,7 @@
                        pointer
                        &allow-other-keys)
             (if (atom spec) (list spec) spec)
-          #-alpha
           (declare (ignorable pointer))
-          #+alpha
-          (when pointer
-            ;; Pointer values on ALPHA are 64 bits wide, and
-            ;; double-word aligned.  We may also wish to have such a
-            ;; mode for other 64-bit hardware outside of any defined
-            ;; 32-on-64 ABI (which would presumably have 32-bit
-            ;; pointers in the first place, obviating the alignment
-            ;; and size requirements).
-            (unless rest-p
-              (setf length 2))
-            (when (oddp offset)
-              (incf offset)))
           (slots `(make-slot ',slot-name ,rest-p ,offset ',special
                              ',(remove-keywords options '(:rest-p :length))))
           (let ((offset-sym (symbolicate name "-" slot-name
@@ -187,15 +174,15 @@
 ;;; storage class. Applies to all backends.
 (defconstant sc-number-limit 62)
 (defconstant sc-number-bits (integer-length (1- sc-number-limit)))
-(def!type sb-c:sc-number () `(integer 0 (,sc-number-limit)))
+(deftype sb-c:sc-number () `(integer 0 (,sc-number-limit)))
 
 (defconstant sc-offset-limit (ash 1 21))
 (defconstant sc-offset-bits (integer-length (1- sc-offset-limit)))
 (deftype sc-offset () `(integer 0 (,sc-offset-limit)))
 
 (defconstant finite-sc-offset-limit
-  #-(or sparc alpha hppa) 32
-  #+(or sparc alpha hppa) 64)
+  #-(or sparc) 32
+  #+(or sparc) 64)
 (defconstant finite-sc-offset-bits
   (integer-length (1- finite-sc-offset-limit)))
 (deftype finite-sc-offset () `(integer 0 (,finite-sc-offset-limit)))
