@@ -117,7 +117,6 @@
     ;; There are some benefits to using the simple case for a known exact type:
     ;; - the layout can be wired in which might or might not save 1 instruction
     ;;   depending on whether layouts are in immobile space.
-    ;; - all backends can do this (subject to stack-allocatable-fixed-objects)
     ;; - for a small number of slots, copying them is inlined
     (cond ((not dd) ; it's going to be some subtype of NAME
            `(%copy-instance (%make-instance (%instance-length instance)) instance))
@@ -145,13 +144,17 @@
 ;;; The layout is stored in slot 0.
 ;;; *** These next two transforms should be the only code, aside from
 ;;;     some parts of the C runtime, with knowledge of the layout index.
+#+compact-instance-header
+(define-source-transform function-with-layout-p (x) `(functionp ,x))
 #-compact-instance-header
 (progn
   (define-source-transform %instance-layout (x)
     `(truly-the layout (%instance-ref ,x 0)))
   (define-source-transform %set-instance-layout (x val)
     `(%instance-set ,x 0 (the layout ,val)))
-  (define-source-transform %funcallable-instance-layout (x)
+  (define-source-transform function-with-layout-p (x)
+    `(funcallable-instance-p ,x))
+  (define-source-transform %fun-layout (x)
     `(truly-the layout (%funcallable-instance-info ,x 0)))
   (define-source-transform %set-funcallable-instance-layout (x val)
     `(setf (%funcallable-instance-info ,x 0) (the layout ,val))))
@@ -381,7 +384,7 @@
                 ,(if (and (integer-type-p index-type)
                           (numeric-type-low index-type))
                      `(integer ,(numeric-type-low index-type)
-                               (,sb-xc:array-dimension-limit))
+                               (,array-dimension-limit))
                      `index))))))
 
 (deftransform %data-vector-and-index ((%array %index)
@@ -501,6 +504,7 @@
                   (word-logical-not (%vector-raw-bits bit-array index))))))))
 
 (deftransform bit-vector-= ((x y) (simple-bit-vector simple-bit-vector))
+  ;; TODO: unroll if length is known and not more than a few words
   `(and (= (length x) (length y))
         (let ((length (length x)))
           (or (= length 0)
@@ -541,6 +545,7 @@
 
 (deftransform count ((item sequence) (bit simple-bit-vector) *
                      :policy (>= speed space))
+  ;; TODO: unroll if length is known and not more than a few words
   `(let ((length (length sequence)))
     (if (zerop length)
         0
