@@ -536,8 +536,8 @@ Return VALUE without evaluating it."
                             (lexenv-blocks *lexenv*) :from-end t))
               *source-namestring*
               (awhen (case *name-context-file-path-selector*
-                       (pathname (or sb-xc:*compile-file-pathname* *load-pathname*))
-                       (truename (or sb-xc:*compile-file-truename* *load-truename*)))
+                       (pathname (or *compile-file-pathname* *load-pathname*))
+                       (truename (or *compile-file-truename* *load-truename*)))
                 (namestring it)))))
     (when context
       (list :in context))))
@@ -571,6 +571,10 @@ Return VALUE without evaluating it."
     (t
      (compiler-error "~S is not a legal function name." thing))))
 
+(def-ir1-translator %fun-name-leaf ((thing) start next result)
+  (fun-name-leaf thing)
+  (ir1-convert start next result nil))
+
 (def-ir1-translator %%allocate-closures ((&rest leaves) start next result)
   (aver (eq result 'nil))
   (let ((lambdas leaves))
@@ -610,7 +614,7 @@ be a lambda expression."
     (reference-leaf start next result leaf)))
 
 (defun constant-global-fun-name (thing)
-  (let ((constantp (sb-xc:constantp thing)))
+  (let ((constantp (constantp thing)))
     (when constantp
       (let ((name (constant-form-value thing)))
         (when (legal-fun-name-p name)
@@ -970,7 +974,7 @@ other."
                     (values-subtypep (make-single-value-type (leaf-type value))
                                      type))
                (and (not (fun-designator-type-p type))
-                    (sb-xc:constantp value)
+                    (constantp value)
                     (or (not (values-type-p type))
                         (values-type-may-be-single-value-p type))
                     (ctypep (constant-form-value value)
@@ -1322,12 +1326,14 @@ due to normal completion or a non-local exit such as THROW)."
   ;; an XEP.
   (ir1-convert
    start next result
-   (with-unique-names (cleanup-fun drop-thru-tag exit-tag next start count)
+   (with-unique-names (cleanup-fun drop-thru-tag exit-tag . #-no-continue-unwind (next start count)
+                                                            #+no-continue-unwind ())
      `(flet ((,cleanup-fun ()
                ,@cleanup
                (values)))
         ;; FIXME: Maybe %ESCAPE-FUN could dynamic extent too.
         (declare (dynamic-extent #',cleanup-fun))
+        #-no-continue-unwind
         (block ,drop-thru-tag
           (multiple-value-bind (,next ,start ,count)
               (block ,exit-tag
@@ -1338,14 +1344,28 @@ due to normal completion or a non-local exit such as THROW)."
                  (return-from ,drop-thru-tag ,protected)))
             (declare (optimize (insert-debug-catch 0)))
             (,cleanup-fun)
-            (%continue-unwind ,next ,start ,count)))))))
+            (%unwind ,next ,start ,count)))
+        #+no-continue-unwind
+        (block ,drop-thru-tag
+          (block ,exit-tag
+            (%within-cleanup
+             :unwind-protect
+             (%unwind-protect (%escape-fun ,exit-tag)
+                              (%cleanup-fun ,cleanup-fun))
+             (return-from ,drop-thru-tag ,protected)))
+          (locally
+              (declare (optimize (insert-debug-catch 0)))
+            (,cleanup-fun)
+            (%continue-unwind)))))))
 
 (def-ir1-translator inspect-unwinding
     ((protected inspect-fun) start next result)
   (ir1-convert
    start next result
-   (with-unique-names (drop-thru-tag exit-tag next start count)
+   (with-unique-names (drop-thru-tag exit-tag next . #-no-continue-unwind (start count)
+                                                     #+no-continue-unwind ())
      `(block ,drop-thru-tag
+        #-no-continue-unwind
         (multiple-value-bind (,next ,start ,count)
             (block ,exit-tag
               (%within-cleanup
@@ -1355,15 +1375,40 @@ due to normal completion or a non-local exit such as THROW)."
                (return-from ,drop-thru-tag ,protected)))
           (declare (optimize (insert-debug-catch 0)))
           (funcall ,inspect-fun ,next)
-          (%continue-unwind ,next ,start ,count))))))
+          (%unwind ,next ,start ,count))
+        #+no-continue-unwind
+        (let ((,next
+                (block ,exit-tag
+                  (%within-cleanup
+                   :unwind-protect
+                   (%unwind-protect (%escape-fun ,exit-tag)
+                                    nil)
+                   (return-from ,drop-thru-tag ,protected)))))
+          (declare (optimize (insert-debug-catch 0)))
+          (funcall ,inspect-fun ,next)
+          (%continue-unwind))))))
 
 ;;; Evaluate CLEANUP iff PROTECTED does a non-local exit.
 (def-ir1-translator nlx-protect
     ((protected &body cleanup) start next result)
   (ir1-convert
    start next result
-   (with-unique-names (drop-thru-tag exit-tag next start count)
+   (with-unique-names (drop-thru-tag exit-tag . #-no-continue-unwind (next start count)
+                                                #+no-continue-unwind ())
      `(block ,drop-thru-tag
+        #+no-continue-unwind
+        (progn
+          (block ,exit-tag
+            (%within-cleanup
+             :unwind-protect
+             (%unwind-protect (%escape-fun ,exit-tag)
+                              nil)
+             (return-from ,drop-thru-tag ,protected)))
+          (locally
+              (declare (optimize (insert-debug-catch 0)))
+            ,@cleanup
+            (%continue-unwind)))
+        #-no-continue-unwind
         (multiple-value-bind (,next ,start ,count)
             (block ,exit-tag
               (%within-cleanup
@@ -1373,7 +1418,7 @@ due to normal completion or a non-local exit such as THROW)."
                (return-from ,drop-thru-tag ,protected)))
           (declare (optimize (insert-debug-catch 0)))
           ,@cleanup
-          (%continue-unwind ,next ,start ,count))))))
+          (%unwind ,next ,start ,count))))))
 
 ;;;; multiple-value stuff
 
