@@ -381,9 +381,9 @@
            (length :scs (any-reg immediate))
            (words :scs (any-reg immediate)))
     (:results (result :scs (descriptor-reg) :from :load))
-    (:temporary (:sc any-reg :offset ecx-offset :from :eval) rcx)
-    (:temporary (:sc any-reg :offset eax-offset :from :eval) rax)
-    (:temporary (:sc any-reg :offset edi-offset :from :eval) rdi)
+    (:temporary (:sc any-reg :offset rcx-offset :from :eval) rcx)
+    (:temporary (:sc any-reg :offset rax-offset :from :eval) rax)
+    (:temporary (:sc any-reg :offset rdi-offset :from :eval) rdi)
     (:temporary (:sc complex-double-reg) zero)
     (:arg-types positive-fixnum positive-fixnum positive-fixnum)
     (:translate allocate-vector)
