@@ -25,6 +25,34 @@
 
 ;;;; DEFCLASS macro and close personal friends
 
+(declaim (global  *the-class-t*
+                  *the-class-slot-object*
+                  *the-class-structure-object*
+                  *the-class-standard-object*
+                  *the-class-function*
+                  *the-class-funcallable-standard-object*
+                  *the-class-system-class*
+                  *the-class-slot-class*
+                  *the-class-condition-class*
+                  *the-class-structure-class*
+                  *the-class-standard-class*
+                  *the-class-funcallable-standard-class*
+                  *the-class-forward-referenced-class*
+                  *the-class-method*
+                  *the-class-standard-method*
+                  *the-class-standard-reader-method*
+                  *the-class-standard-writer-method*
+                  *the-class-standard-boundp-method*
+                  *the-class-global-reader-method*
+                  *the-class-global-writer-method*
+                  *the-class-global-boundp-method*
+                  *the-class-standard-generic-function*
+                  *the-class-standard-direct-slot-definition*
+                  *the-class-standard-effective-slot-definition*
+
+                  *the-eslotd-standard-class-slots*
+                  *the-eslotd-funcallable-standard-class-slots*))
+
 ;;; state for the current DEFCLASS expansion
 (defvar *initfunctions-for-this-defclass*)
 (defvar *readers-for-this-defclass*)
@@ -54,7 +82,7 @@
       ;; metaclass.
       (mapc #'sb-int:check-deprecated-type direct-superclasses)
       (sb-int:check-deprecated-type metaclass)
-      (let ((canonical-slots (canonize-defclass-slots name direct-slots env))
+      (let ((canonical-slots (canonize-defclass-slots name metaclass direct-slots env))
             ;; DEFSTRUCT-P should be true if the class is defined
             ;; with a metaclass STRUCTURE-CLASS, so that a DEFSTRUCT
             ;; is compiled for the class.
@@ -128,9 +156,8 @@
   (maplist (lambda (sublist)
              (let ((option-name (first (pop sublist))))
                (when (member option-name sublist :key #'first :test #'eq)
-                 (error 'simple-program-error
-                        :format-control "Multiple ~S options in DEFCLASS ~S."
-                        :format-arguments (list option-name class-name)))))
+                 (%program-error "Multiple ~S options in DEFCLASS ~S."
+                                 option-name class-name))))
            options)
   (let (metaclass
         default-initargs
@@ -143,20 +170,19 @@
           (:metaclass
            (let ((maybe-metaclass (second option)))
              (unless (and maybe-metaclass (legal-class-name-p maybe-metaclass))
-               (error 'simple-program-error
-                      :format-control "~@<The value of the :metaclass option (~S) ~
-                         is not a legal class name.~:@>"
-                      :format-arguments (list maybe-metaclass)))
+               (%program-error "~@<The value of the :metaclass ~
+                                option (~S) is not a legal class ~
+                                name.~:@>"
+                               maybe-metaclass))
              (setf metaclass maybe-metaclass)))
           (:default-initargs
            (let (initargs arg-names)
              (doplist (key val) (cdr option)
                (when (member key arg-names :test #'eq)
-                 (error 'simple-program-error
-                        :format-control "~@<Duplicate initialization argument ~
-                                           name ~S in :DEFAULT-INITARGS of ~
-                                           DEFCLASS ~S.~:>"
-                        :format-arguments (list key class-name)))
+                 (%program-error "~@<Duplicate initialization argument ~
+                                  name ~S in :DEFAULT-INITARGS of ~
+                                  DEFCLASS ~S.~:>"
+                                 key class-name))
                (push key arg-names)
                (push ``(,',key ,',val ,,(make-initfunction val)) initargs))
              (setf default-initargs t)
@@ -173,93 +199,86 @@
         (push '(:direct-default-initargs nil) canonized-options))
       (values (or metaclass 'standard-class) (nreverse canonized-options))))
 
-(defun canonize-defclass-slots (class-name slots env)
-  (let (canonized-specs)
-    (dolist (spec slots)
-      (let ((location (or (and (boundp 'sb-c::*current-path*)
-                               (boundp 'sb-c::*source-paths*)
-                               (let ((sb-c::*current-path*
-                                       (or (sb-c::get-source-path spec)
-                                           sb-c::*current-path*)))
-                                 (sb-c::make-definition-source-location)))
-                          (sb-c::make-definition-source-location))))
-        (when (atom spec)
-          (setf spec (list spec)))
-       (when (and (cdr spec) (null (cddr spec)))
-         (error 'simple-program-error
-                :format-control "~@<in DEFCLASS ~S, the slot specification ~S ~
-                                is invalid; the probable intended meaning may ~
-                                be achieved by specifiying ~S instead.~:>"
-                :format-arguments (list class-name spec
-                                        `(,(car spec) :initform ,(cadr spec)))))
-       (let* ((name (car spec))
-              (plist (cdr spec))
-              (readers ())
-              (writers ())
-              (initargs ())
-              (others ())
-              (unsupplied (list nil))
-              (type t)
-              (initform unsupplied))
-         (check-slot-name-for-defclass name class-name env)
-         (push name *slot-names-for-this-defclass*)
-         (flet ((note-reader (x)
-                  (unless (symbolp x)
-                    (error 'simple-program-error
-                           :format-control "Slot reader name ~S for slot ~S in ~
-                                           DEFCLASS ~S is not a symbol."
-                           :format-arguments (list x name class-name)))
-                  (push x readers)
-                  (push x *readers-for-this-defclass*))
-                (note-writer (x)
-                  (push x writers)
-                  (push x *writers-for-this-defclass*)))
-           (doplist (key val) plist
+(defun canonize-defclass-slot (class-name metaclass spec env)
+  (let ((location (sb-c::make-definition-source-location))
+        (spec (sb-int:ensure-list spec)))
+    (when (and (cdr spec) (null (cddr spec)))
+      (%program-error "~@<in DEFCLASS ~S, the slot specification ~S ~
+                       is invalid; the probable intended meaning may ~
+                       be achieved by specifiying ~S instead.~:>"
+                      class-name spec `(,(car spec) :initform ,(cadr spec))))
+    (let* ((name (car spec))
+           (plist (cdr spec))
+           (readers ())
+           (writers ())
+           (initargs ())
+           (others ())
+           (unsupplied (list nil))
+           (type t)
+           (initform unsupplied))
+      (check-slot-name-for-defclass name class-name env)
+      (push name *slot-names-for-this-defclass*)
+      (flet ((note-reader (x)
+               (unless (symbolp x)
+                 (%program-error "Slot reader name ~S for slot ~S in ~
+                                  DEFCLASS ~S is not a symbol."
+                                 x name class-name))
+               (push x readers)
+               (push x *readers-for-this-defclass*))
+             (note-writer (x)
+               (push x writers)
+               (push x *writers-for-this-defclass*)))
+        (doplist (key val) plist
+          (case key
+            (:accessor (note-reader val) (note-writer `(setf ,val)))
+            (:reader   (note-reader val))
+            (:writer   (note-writer val))
+            (:initarg
+             (unless (symbolp val)
+               (%program-error "Slot initarg name ~S for slot ~S in ~
+                                DEFCLASS ~S is not a symbol."
+                               val name class-name))
+             (push val initargs))
+            ((:initform :type :allocation :documentation)
              (case key
-               (:accessor (note-reader val) (note-writer `(setf ,val)))
-               (:reader   (note-reader val))
-               (:writer   (note-writer val))
-               (:initarg
-                (unless (symbolp val)
-                  (error 'simple-program-error
-                         :format-control "Slot initarg name ~S for slot ~S in ~
-                                         DEFCLASS ~S is not a symbol."
-                         :format-arguments (list val name class-name)))
-                (push val initargs))
-               (otherwise
-                (when (member key '(:initform :allocation :type :documentation))
-                  (when (eq key :initform)
-                    (setf initform val))
-                  (when (eq key :type)
-                    (setf type val))
-                  (when (get-properties others (list key))
-                    (error 'simple-program-error
-                           :format-control "Duplicate slot option ~S for slot ~
-                                           ~S in DEFCLASS ~S."
-                           :format-arguments (list key name class-name))))
-                ;; For non-standard options multiple entries go in a list
-                (push val (getf others key))))))
-         ;; Unwrap singleton lists (AMOP 5.4.2)
-         (do ((head others (cddr head)))
-             ((null head))
-           (unless (cdr (second head))
-             (setf (second head) (car (second head)))))
-         (let ((canon `(:name ',name :readers ',readers :writers ',writers
-                        :initargs ',initargs  'source ,location ',others)))
-           (push (if (eq initform unsupplied)
-                     `(list* ,@canon)
-                     `(list* :initfunction ,(make-initfunction initform)
-                             ,@canon))
-                 canonized-specs)))))
-    (nreverse canonized-specs)))
+               (:initform
+                (setf initform val))
+               (:type
+                (when (eq metaclass 'standard-class)
+                  (sb-kernel::check-slot-type-specifier
+                   val name (cons 'defclass class-name))
+                  (setf type val))))
+             (when (get-properties others (list key))
+               (%program-error "Duplicate slot option ~S for slot ~
+                                ~S in DEFCLASS ~S."
+                               key name class-name))
+             (push val (getf others key)))
+            (otherwise
+             ;; For non-standard options multiple entries go in a list
+             (push val (getf others key))))))
+      ;; Unwrap singleton lists (AMOP 5.4.2)
+      (do ((head others (cddr head)))
+          ((null head))
+        (unless (cdr (second head))
+          (setf (second head) (car (second head)))))
+      (let ((canon `(:name ',name :readers ',readers :writers ',writers
+                     :initargs ',initargs  'source ,location ',others)))
+        (if (eq initform unsupplied)
+            `(list* ,@canon)
+            `(list* :initfunction ,(make-initfunction initform type spec)
+                    ,@canon))))))
 
+(defun canonize-defclass-slots (class-name metaclass slots env)
+  (map 'list (lambda (spec)
+               (with-current-source-form (spec)
+                 (canonize-defclass-slot class-name metaclass spec env)))
+       slots))
 
 (defun check-slot-name-for-defclass (name class-name env)
   (flet ((slot-name-illegal (reason)
-           (error 'simple-program-error
-                  :format-control "~@<In DEFCLASS ~S, the slot name ~S ~
-                                   is ~A.~@:>"
-                  :format-arguments (list class-name name reason))))
+           (%program-error "~@<In DEFCLASS ~S, the slot name ~S is ~
+                            ~A.~@:>"
+                           class-name name reason)))
     (cond ((not (symbolp name))
            (slot-name-illegal "not a symbol"))
           ((keywordp name)
@@ -267,23 +286,35 @@
           ((constantp name env)
            (slot-name-illegal "a constant"))
           ((member name *slot-names-for-this-defclass* :test #'eq)
-           (error 'simple-program-error
-                  :format-control "Multiple slots named ~S in DEFCLASS ~S."
-                  :format-arguments (list name class-name))))))
+           (%program-error "Multiple slots named ~S in DEFCLASS ~S."
+                           name class-name)))))
 
-(defun make-initfunction (initform)
-  (cond ((or (eq initform t)
-             (equal initform ''t))
+(defun make-initfunction (initform &optional (type t)
+                                             source-form)
+  (cond ((and (or (eq initform t)
+                  (equal initform ''t))
+              (eq type t))
          '(function constantly-t))
-        ((or (eq initform nil)
-             (equal initform ''nil))
+        ((and (or (eq initform nil)
+                  (equal initform ''nil))
+              (eq type t))
          '(function constantly-nil))
-        ((or (eql initform 0)
-             (equal initform ''0))
+        ((and (or (eql initform 0)
+                  (equal initform ''0))
+              (eq type t))
          '(function constantly-0))
         (t
-         (let ((entry (assoc initform *initfunctions-for-this-defclass*
-                             :test #'equal)))
+         (let* ((initform (if (eq type t)
+                              initform
+                              `(sb-c::with-source-form ,source-form
+                                (the* (,type :source-form ,initform
+                                       ;; Don't want to insert a cast,
+                                       ;; as a subclass may change the type,
+                                       ;; just report this at compile-time.
+                                             :use-annotations t)
+                                      ,initform))))
+                (entry (assoc initform *initfunctions-for-this-defclass*
+                              :test #'equal)))
            (unless entry
              (setq entry (list initform
                                (gensym)
@@ -411,15 +442,9 @@
 (defun !bootstrap-set-slot (type object slot-name new-value)
   (setf (!bootstrap-get-slot type object slot-name) new-value))
 
-(defun early-class-name (class)
-  (!bootstrap-get-slot 'class class 'name))
-
 (defun early-class-precedence-list (class)
   (!bootstrap-get-slot 'pcl-class class '%class-precedence-list))
 
-(defun early-class-name-of (instance)
-  (early-class-name (class-of instance)))
-
 (defun early-class-slotds (class)
   (!bootstrap-get-slot 'slot-class class 'slots))
 
@@ -432,14 +457,6 @@
 (defun early-slot-definition-info (slotd)
   (!bootstrap-get-slot 'standard-effective-slot-definition slotd 'info))
 
-(defun early-accessor-method-slot-name (method)
-  (!bootstrap-get-slot 'standard-accessor-method method 'slot-name))
-
-(unless (fboundp 'class-name-of)
-  (setf (symbol-function 'class-name-of)
-        (symbol-function 'early-class-name-of)))
-(unintern 'early-class-name-of)
-
 (defun early-class-direct-subclasses (class)
   (!bootstrap-get-slot 'class class 'direct-subclasses))
 
