@@ -31,21 +31,18 @@
 
 ;;; This is called in BRAID when we are making wrappers for classes
 ;;; whose slots are not initialized yet, and which may be built-in
-;;; classes. We pass in the class name in addition to the class.
-(defun !boot-make-wrapper (length name &optional class (bitmap -1))
+;;; classes.
+(defun !boot-make-wrapper (length name &optional (bitmap -1))
   (let ((found (find-classoid name nil)))
     (cond
      (found
-      (unless (classoid-pcl-class found)
-        (setf (classoid-pcl-class found) class))
-      (aver (eq (classoid-pcl-class found) class))
       (let ((layout (classoid-layout found)))
         (aver layout)
         layout))
      (t
       (set-layout-valid
-       (make-layout (randomish-layout-clos-hash name)
-                    (make-standard-classoid :name name :pcl-class class)
+       (make-layout (hash-layout-name name)
+                    (make-standard-classoid :name name)
                     :length length
                     :flags +pcl-object-layout-flag+
                     :bitmap bitmap))))))
@@ -75,7 +72,7 @@
                      (t
                       (bug "Got to T branch in ~S" 'make-wrapper))))))
        (set-layout-valid
-        (make-layout (randomish-layout-clos-hash name)
+        (make-layout (hash-layout-name name)
                      classoid :length length :flags +pcl-object-layout-flag+))))
     (t
      (let* ((found (find-classoid (slot-value class 'name)))
@@ -86,23 +83,6 @@
        (aver layout)
        layout))))
 
-(declaim (inline wrapper-class*))
-(defun wrapper-class* (wrapper)
-  (or (wrapper-class wrapper)
-      ;; FIXME: this branch seems unreachable.
-      ;; It would be nice to eliminate WRAPPER-CLASS* if we can show that it
-      ;; is only a holdover from an earlier way of bootstrapping that resulted
-      ;; in the temporary absence of a PCL-CLASS for some non-standard-class.
-      ;; Certainly no test gets here [changing it to (BUG "got here") worked].
-      ;; Note however that
-      ;;  (CLASSOID-PCL-CLASS (FIND-CLASSOID 'STANDARD-INSTANCE)) => NIL
-      ;; which can be resolved by just ensuring one time that it has a CLASS.
-      ;; And nothing else seems to be problematic.
-      (let ((classoid (layout-classoid wrapper)))
-        (ensure-non-standard-class
-         (classoid-name classoid)
-         classoid))))
-
 ;;; The wrapper cache machinery provides general mechanism for
 ;;; trapping on the next access to any instance of a given class. This
 ;;; mechanism is used to implement the updating of instances when the
@@ -116,6 +96,15 @@
 ;;; SLOT-VALUE-USING-CLASS check the wrapper validity as well. This is
 ;;; done by calling CHECK-WRAPPER-VALIDITY.
 
+;;; This "simple" function hides a horrible inconsistency: we don't have a single
+;;; canonical atomically checkable validity indicator. We carry around invalid layouts
+;;; with nonzero hashes. Why else would there be so may places that can do:
+;;;   (setf (layout-invalid layout) nil
+;;; whilst the layout in question already has a "valid" hash?
+;;; It's hard to know what the right thing is, but since internal code uses
+;;; this test to decide how to handle layouts that have been invalidated,
+;;; I don't think we can just change it to examine the hash value
+;;; even though logically that *should* be the canonical test.
 (declaim (inline invalid-wrapper-p))
 (defun invalid-wrapper-p (wrapper)
   (not (null (layout-invalid wrapper))))
@@ -167,8 +156,7 @@
 ;;; but also pretty much not correct.
 (defun %invalidate-wrapper (owrapper state nwrapper)
   (aver (member state '(:flush :obsolete)))
-  #+sb-thread (aver (eq (sb-thread:mutex-owner sb-c::**world-lock**)
-                        sb-thread:*current-thread*))
+  #+sb-thread (aver (sb-thread:holding-mutex-p sb-c::**world-lock**))
   (let ((classoid (layout-classoid nwrapper))
         (new-previous ())
         (new-state (cons state nwrapper)))
@@ -197,10 +185,13 @@
 
     ;; FIXME: We are here inside PCL lock, but might someone be
     ;; accessing the wrapper at the same time from outside the lock?
-    (setf (layout-clos-hash owrapper) 0)
-    ;; And shouldn't these assignments be flipped, so that if an observer
-    ;; sees a 0 hash in owrapper, it can find the new layout?
+    ;; Inform readers of the reason for wrapper invalidity before marking
+    ;; the wrapper as invalid.
     (setf (layout-invalid owrapper) new-state)
+    ;; Ensure that the INVALID slot conveying ancillary data describing the
+    ;; invalidity reason is published before causing the invalid layout trap.
+    (sb-thread:barrier (:write))
+    (setf (layout-clos-hash owrapper) 0)
     (push (make-weak-pointer owrapper) new-previous)
     ;; This function is called for effect; return value is arbitrary.
     (setf  (sb-kernel::standard-classoid-old-layouts classoid)
@@ -216,7 +207,7 @@
       (aver (not (eq state :uninitialized)))
       (cond ((not state)
              owrapper)
-            ((not (layout-for-std-class-p owrapper))
+            ((not (layout-for-pcl-obj-p owrapper))
              ;; Obsolete structure trap.
              (%obsolete-instance-trap owrapper nil instance))
             ((eq t state)
@@ -230,7 +221,7 @@
              ;;    previous call to REGISTER-LAYOUT for a superclass of
              ;;    INSTANCE's class.  See also the comment above
              ;;    FORCE-CACHE-FLUSHES.  Paul Dietz has test cases for this.
-             (let ((class (wrapper-class* owrapper)))
+             (let ((class (wrapper-class owrapper)))
                (%force-cache-flushes class)
                ;; KLUDGE: avoid an infinite recursion, it's still better to
                ;; bail out with an error for server softwares. see FIXME above.
@@ -260,7 +251,7 @@
                   (cond ((std-instance-p instance)
                          (setf (%instance-layout instance) new))
                         ((fsc-instance-p instance)
-                         (setf (%funcallable-instance-layout instance) new))
+                         (setf (%fun-layout instance) new))
                         (t
                          (bug "unrecognized instance type")))))
                (:obsolete
@@ -376,7 +367,7 @@
                     (setf (cdr dfun-wrappers-tail) new-dfun-wrappers-tail)
                     (setf dfun-wrappers-tail new-dfun-wrappers-tail))))
            ,@(when wrappers
-               `((setq class (wrapper-class* wrapper))
+               `((setq class (wrapper-class wrapper))
                  (setq type `(class-eq ,class)))))
          ,@(when wrappers
              `((push wrapper wrappers-rev)
