@@ -35,6 +35,7 @@
 #include "gc.h"
 #include "code.h"
 #include "var-io.h"
+#include "gc-internal.h"
 #include "forwarding-ptr.h"
 #include "lispstring.h"
 
@@ -207,28 +208,16 @@ print_entry_points (struct code *code, FILE *f)
  * better not change. */
 
 struct call_frame {
-#ifndef LISP_FEATURE_ALPHA
         struct call_frame *old_cont;
-#else
-        u32 old_cont;
-#endif
         lispobj saved_lra;
         lispobj code;
         lispobj other_state[5];
 };
 
 struct call_info {
-#ifndef LISP_FEATURE_ALPHA
     struct call_frame *frame;
-#else
-    u32 frame;
-#endif
     int interrupted;
-#ifndef LISP_FEATURE_ALPHA
     struct code *code;
-#else
-    u32 code;
-#endif
     lispobj lra;
     int pc; /* Note: this is the trace file offset, not the actual pc. */
 };
@@ -311,11 +300,7 @@ call_info_from_context(struct call_info *info, os_context_t *context)
     }
     if (info->code != NULL)
         info->pc = pc - (uword_t) info->code -
-#ifndef LISP_FEATURE_ALPHA
             (HEADER_LENGTH(info->code->header) * sizeof(lispobj));
-#else
-            (HEADER_LENGTH(((struct code *)info->code)->header) * sizeof(lispobj));
-#endif
     else
         info->pc = 0;
 }
@@ -485,7 +470,7 @@ void
 describe_thread_state(void)
 {
     struct thread *thread = arch_os_get_current_thread();
-    struct interrupt_data *data = thread->interrupt_data;
+    struct interrupt_data *data = &thread_interrupt_data(thread);
 #ifndef LISP_FEATURE_WIN32
     sigset_t mask;
     get_current_sigmask(&mask);
@@ -605,16 +590,15 @@ int simple_fun_index_from_pc(struct code* code, char *pc)
 static boolean __attribute__((unused)) print_lisp_fun_name(char* pc)
 {
   struct code* code;
-  int fi;
-  lispobj fn;
   if (gc_managed_addr_p((uword_t)pc) &&
-      (code = (void*)component_ptr_from_pc(pc)) != 0 &&
-      (fi = simple_fun_index_from_pc(code, pc)) >= 0 &&
-      is_lisp_pointer(fn = code->constants[fi*CODE_SLOTS_PER_SIMPLE_FUN])) {
-      fprintf(stderr, " %p [", pc);
-      print_entry_name(fn, stderr);
-      fprintf(stderr, "]\n");
-      return 1;
+      (code = (void*)component_ptr_from_pc(pc)) != 0) {
+      struct compiled_debug_fun* df = debug_function_from_pc(code, pc);
+      if (df) {
+          fprintf(stderr, " %p [", pc);
+          print_entry_name(df->name, stderr);
+          fprintf(stderr, "]\n");
+          return 1;
+      }
   }
   return 0;
 }
@@ -624,11 +608,13 @@ static boolean __attribute__((unused)) print_lisp_fun_name(char* pc)
 #include <libunwind.h>
 #include "genesis/thread-instance.h"
 #include "genesis/mutex.h"
-static int backtrace_completion_pipe[2] = {-1,-1};
+static __attribute__((unused))int backtrace_completion_pipe[2] = {-1,-1};
 void libunwind_backtrace(struct thread *th, os_context_t *context)
 {
     char procname[100];
     fprintf(stderr, "Lisp thread @ %p, tid %d", th, (int)th->os_kernel_tid);
+#ifdef LISP_FEATURE_SB_THREAD
+    // the TLS area is not used if #-sb-thread. And if so, it must be "main thred"
     struct thread_instance* lispthread = (void*)native_pointer(th->lisp_thread);
     if (lispthread->name != NIL) {
         fprintf(stderr, " (\"");
@@ -653,15 +639,22 @@ void libunwind_backtrace(struct thread *th, os_context_t *context)
         }
         putc('\n', stderr);
     }
+#endif
     unw_cursor_t cursor;
     // "unw_init_local() is thread-safe as well as safe to use from a signal handler."
     // "unw_get_proc_name() is thread-safe. If cursor cp is in the local address-space,
     //  this routine is also safe to use from a signal handler."
-    unw_init_local(&cursor, context);
+    if (context) {
+        unw_init_local(&cursor, context);
+    } else {
+        unw_context_t here;
+        unw_getcontext(&here);
+        unw_init_local(&cursor, &here);
+    }
     do {
         uword_t offset;
         char *pc;
-        unw_get_reg(&cursor, UNW_X86_64_RIP, (uword_t*)&pc);
+        unw_get_reg(&cursor, UNW_TDEP_IP, (uword_t*)&pc);
         if (print_lisp_fun_name(pc)) {
             // printed
         } else if (!unw_get_proc_name(&cursor, procname, sizeof procname, &offset)) {
@@ -675,6 +668,7 @@ void backtrace_lisp_threads(int __attribute__((unused)) signal,
                                    siginfo_t __attribute__((unused)) *info,
                                    os_context_t *context)
 {
+#ifdef LISP_FEATURE_SB_THREAD
     if (backtrace_completion_pipe[1] >= 0) {
         libunwind_backtrace(current_thread, context);
         write(backtrace_completion_pipe[1], context /* any random byte */, 1);
@@ -704,5 +698,8 @@ void backtrace_lisp_threads(int __attribute__((unused)) signal,
         close(backtrace_completion_pipe[0]);
         backtrace_completion_pipe[0] = backtrace_completion_pipe[1] = -1;
     }
+#else
+    libunwind_backtrace(arch_os_get_current_thread(), context);
+#endif
 }
 #endif
