@@ -46,7 +46,7 @@ extern void *gc_general_alloc(sword_t nbytes,int page_type_flag);
 #define note_transported_object(old, new) /* do nothing */
 
 static inline lispobj
-gc_general_copy_object(lispobj object, long nwords, int page_type_flag)
+gc_general_copy_object(lispobj object, size_t nwords, int page_type_flag)
 {
     CHECK_COPY_PRECONDITIONS(object, nwords);
 
@@ -234,7 +234,7 @@ static inline void add_to_weak_pointer_chain(struct weak_pointer *wp) {
      * In cheneygc, chaining is performed in 'trans_weak_pointer'
      * which works just as well, since an object is transported
      * at most once per GC cycle */
-    wp->next = (struct weak_pointer *)LOW_WORD(weak_pointer_chain);
+    wp->next = weak_pointer_chain;
     weak_pointer_chain = wp;
 }
 
@@ -249,8 +249,9 @@ static inline boolean layout_bitmap_logbitp(int index, lispobj bitmap)
 }
 
 /* Keep in sync with 'target-hash-table.lisp' */
+#define hashtable_kind(ht) ((ht->flags >> (4+N_FIXNUM_TAG_BITS)) & 3)
 #define hashtable_weakp(ht) (ht->flags & (8<<N_FIXNUM_TAG_BITS))
-#define hashtable_weakness(ht) (ht->flags >> (4+N_FIXNUM_TAG_BITS))
+#define hashtable_weakness(ht) (ht->flags >> (6+N_FIXNUM_TAG_BITS))
 
 #if defined(LISP_FEATURE_GENCGC)
 
