@@ -33,10 +33,10 @@
  *   the value from interrupt_low_level_handlers[..], instead of the
  *   ordinary interrupt_handle_now(..) or interrupt_handle_later(..).
  *
- * o the SIGTRAP (Linux/Alpha) which Lisp code uses to handle breakpoints,
+ * o the SIGTRAP which Lisp code may uses to handle breakpoints,
  *   pseudo-atomic sections, and some classes of error (e.g. "function
  *   not defined").  This never goes anywhere near the Lisp handlers at all.
- *   See runtime/alpha-arch.c and code/signal.lisp
+ *   See src/code/signal.lisp
  *
  * - WHN 20000728, dan 20010128 */
 
@@ -112,7 +112,7 @@ boolean internal_errors_enabled = 0;
 static
 void (*interrupt_low_level_handlers[NSIG]) (int, siginfo_t*, os_context_t*);
 #endif
-union interrupt_handler interrupt_handlers[NSIG];
+lispobj lisp_sig_handlers[NSIG];
 
 /* Under Linux on some architectures, we appear to have to restore the
  * FPU control word from the context, as after the signal is delivered
@@ -178,49 +178,51 @@ static void sigmask_logandc(sigset_t *dest, const sigset_t *source)
  * work for SIGSEGV and similar. It is good enough for timers, and
  * maybe all deferrables. */
 
-#ifndef LISP_FEATURE_WIN32
-static void
-add_handled_signals(sigset_t *sigset)
-{
-    int i;
-    for(i = 1; i < NSIG; i++) {
-        if (!(ARE_SAME_HANDLER(interrupt_low_level_handlers[i], SIG_DFL)) ||
-            !(ARE_SAME_HANDLER(interrupt_handlers[i].c, SIG_DFL))) {
-            sigaddset(sigset, i);
-        }
-    }
-}
+#if defined LISP_FEATURE_DARWIN && defined LISP_FEATURE_SB_THREAD
+pthread_key_t sigwait_bug_mitigation;
+int sigwait_bug_mitigation_count;
 #endif
 
-static boolean
-maybe_resignal_to_lisp_thread(int signal, os_context_t *context)
-{
-#ifndef LISP_FEATURE_WIN32
-    if (!lisp_thread_p(context)) {
-        if (!(sigismember(&deferrable_sigset,signal))) {
-            corruption_warning_and_maybe_lose
-#ifdef LISP_FEATURE_SB_THREAD
-                ("Received signal %d in non-lisp thread %lu, resignalling to a lisp thread.",
-                 signal, pthread_self());
+#ifdef LISP_FEATURE_WIN32
+#define resignal_to_lisp_thread(dummy1,dummy2) {}
 #else
-            ("Received signal %d in non-lisp thread, resignalling to a lisp thread.",
-             signal);
+static void
+resignal_to_lisp_thread(int signal, os_context_t *context)
+{
+#if defined LISP_FEATURE_DARWIN && defined LISP_FEATURE_SB_THREAD
+    if (signal == SIG_STOP_FOR_GC && pthread_getspecific(sigwait_bug_mitigation)) {
+        // fprintf(stderr, "ignored a STOP_FOR_GC signal\n");
+        __sync_fetch_and_add(&sigwait_bug_mitigation_count, 1);
+        pthread_setspecific(sigwait_bug_mitigation, 0);
+        return;
+    }
 #endif
+    if (!sigismember(&deferrable_sigset,signal)) {
+        corruption_warning_and_maybe_lose
+#ifdef LISP_FEATURE_SB_THREAD
+            ("Received signal %d @ %lx in non-lisp"THREAD_ID_LABEL", resignaling to a lisp thread.",
+             signal, *os_context_pc_addr(context), THREAD_ID_VALUE);
+#else
+            ("Received signal %d in non-lisp thread, resignaling to a lisp thread.", signal);
+#endif
+    }
+    sigset_t sigset;
+    sigemptyset(&sigset);
+    int i;
+    for(i = 1; i < NSIG; i++) {
+        if (!ARE_SAME_HANDLER(interrupt_low_level_handlers[i], SIG_DFL)
+            || lisp_sig_handlers[i] != NIL) {
+            sigaddset(&sigset, i);
         }
-        sigset_t sigset;
-        sigemptyset(&sigset);
-        add_handled_signals(&sigset);
-        thread_sigmask(SIG_BLOCK, &sigset, 0);
-        // This arranges for every handled signal to be blocked on return from this
-        // handler invocation, presumably because that avoids further detours through
-        // this thread's handler for signals that we don't want it to handle.
-        sigmask_logior(os_context_sigmask_addr(context), &sigset);
-        kill(getpid(), signal);
-        return 1;
-    } else
-#endif
-        return 0;
+    }
+    thread_sigmask(SIG_BLOCK, &sigset, 0);
+    // This arranges for every handled signal to be blocked on return from this
+    // handler invocation, presumably because that avoids further detours through
+    // this thread's handler for signals that we don't want it to handle.
+    sigmask_logior(os_context_sigmask_addr(context), &sigset);
+    kill(getpid(), signal);
 }
+#endif
 
 #if INSTALL_SIG_MEMORY_FAULT_HANDLER && defined(THREAD_SANITIZER)
 /* Under TSAN, any delivered signal blocks all other signals regardless of the
@@ -261,16 +263,20 @@ maybe_resignal_to_lisp_thread(int signal, os_context_t *context)
  * kernel properly, so we fix it up ourselves in the
  * arch_os_get_context(..) function. -- CSR, 2002-07-23
  */
+#ifdef LISP_FEATURE_WIN32
+# define should_handle_in_this_thread(c) (1)
+#else
+# define should_handle_in_this_thread(c) lisp_thread_p(c)
+#endif
 #define SAVE_ERRNO(signal,context,void_context)                 \
     {                                                           \
         int _saved_errno = errno;                               \
         UNBLOCK_SIGSEGV();                                      \
         RESTORE_FP_CONTROL_WORD(context,void_context);          \
-        if (!maybe_resignal_to_lisp_thread(signal, context))    \
-        {
+        if (should_handle_in_this_thread(context)) {
 
 #define RESTORE_ERRNO                                           \
-        }                                                       \
+        } else resignal_to_lisp_thread(signal,void_context);    \
         errno = _saved_errno;                                   \
     }
 
@@ -353,7 +359,6 @@ sigaddset_deferrable(sigset_t *s)
     sigaddset(s, SIGINT);
     sigaddset(s, SIGTERM);
     sigaddset(s, SIGQUIT);
-    sigaddset(s, SIGPIPE);
     sigaddset(s, SIGALRM);
     sigaddset(s, SIGURG);
     sigaddset(s, SIGTSTP);
@@ -363,10 +368,10 @@ sigaddset_deferrable(sigset_t *s)
 #else
     sigaddset(s, SIGPOLL);
 #endif
-#ifndef LISP_FEATURE_HPUX
+#ifndef LISP_FEATURE_LIBUNWIND_BACKTRACE
     sigaddset(s, SIGXCPU);
-    sigaddset(s, SIGXFSZ);
 #endif
+    sigaddset(s, SIGXFSZ);
     sigaddset(s, SIGVTALRM);
     sigaddset(s, SIGPROF);
     sigaddset(s, SIGWINCH);
@@ -385,6 +390,11 @@ sigaddset_blockable(sigset_t *sigset)
 {
     sigaddset_deferrable(sigset);
     sigaddset_gc(sigset);
+    // SIGPIPE is *NOT* an asynchronous signal. In normal usage you receive this signal
+    // synchronously in response to a system call. As such we do not place it in the
+    // deferrable set, but it _is_ blockable. If you're doing something wherein SIGPIPE
+    // interrupts a pseudo-atomic section, then you're doing something wrong for sure.
+    sigaddset(sigset, SIGPIPE);
 }
 
 /* initialized in interrupt_init */
@@ -434,8 +444,7 @@ check_blockables_unblocked_or_lose(sigset_t *sigset)
 #endif
 }
 
-void
-check_blockables_blocked_or_lose(sigset_t *sigset)
+static void assert_blockables_blocked()
 {
 #if !defined(LISP_FEATURE_WIN32)
     /* On Windows, there are no actual signals, but since the win32 port
@@ -456,7 +465,7 @@ check_blockables_blocked_or_lose(sigset_t *sigset)
      * So we merely skip this assertion.
      *   -- DFL, trying to expand on a comment by AK.
      */
-    if (!blockables_blocked_p(sigset))
+    if (!blockables_blocked_p(0))
         lose("blockables unblocked");
 #endif
 }
@@ -578,7 +587,7 @@ maybe_save_gc_mask_and_block_deferrables(sigset_t *sigset)
 {
 #ifndef LISP_FEATURE_WIN32
     struct thread *thread = arch_os_get_current_thread();
-    struct interrupt_data *data = thread->interrupt_data;
+    struct interrupt_data *data = &thread_interrupt_data(thread);
     sigset_t oldset;
     /* Obviously, this function is called when signals may not be
      * blocked. Let's make sure we are not interrupted. */
@@ -635,7 +644,7 @@ check_interrupt_context_or_lose(os_context_t *context)
 {
 #if !defined(LISP_FEATURE_WIN32) || defined(LISP_FEATURE_SB_THREAD)
     struct thread *thread = arch_os_get_current_thread();
-    struct interrupt_data *data = thread->interrupt_data;
+    struct interrupt_data *data = &thread_interrupt_data(thread);
     int interrupt_deferred_p = (data->pending_handler != 0);
     int interrupt_pending = (read_TLS(INTERRUPT_PENDING,thread) != NIL);
     sigset_t *sigset = os_context_sigmask_addr(context);
@@ -775,15 +784,11 @@ build_fake_control_stack_frames(struct thread __attribute__((unused)) *th,
 
 /* Stores the context for gc to scavange and builds fake stack
  * frames. */
-void
-fake_foreign_function_call(os_context_t *context)
+void fake_foreign_function_call_noassert(os_context_t *context)
 {
     int context_index;
     struct thread *thread=arch_os_get_current_thread();
 
-    /* context_index incrementing must not be interrupted */
-    check_blockables_blocked_or_lose(0);
-
 #ifdef reg_ALLOC
 #ifdef LISP_FEATURE_SB_THREAD
     thread->pseudo_atomic_bits =
@@ -794,17 +799,11 @@ fake_foreign_function_call(os_context_t *context)
             (*os_context_register_addr(context, reg_ALLOC));
 /*     fprintf(stderr,"dynamic_space_free_pointer: %p\n", */
 /*             dynamic_space_free_pointer); */
-#if defined(LISP_FEATURE_ALPHA) || defined(LISP_FEATURE_MIPS)
+#if defined(LISP_FEATURE_MIPS)
     if ((sword_t)dynamic_space_free_pointer & 1) {
         lose("dead in fake_foreign_function_call, context = %x", context);
     }
 #endif
-/* why doesnt PPC and SPARC do something like this: */
-#if defined(LISP_FEATURE_HPPA)
-    if ((sword_t)dynamic_space_free_pointer & 4) {
-        lose("dead in fake_foreign_function_call, context = %x, d_s_f_p = %x", context, dynamic_space_free_pointer);
-    }
-#endif
 #endif
 #ifdef reg_BSP
     set_binding_stack_pointer(thread,
@@ -825,10 +824,8 @@ fake_foreign_function_call(os_context_t *context)
     context_index =
         fixnum_value(read_TLS(FREE_INTERRUPT_CONTEXT_INDEX,thread));
 
-    if (context_index >= (MAX_INTERRUPTS-THREAD_HEADER_SLOTS)) {
-        lose("maximum interrupt nesting depth (%d) exceeded",
-             MAX_INTERRUPTS-THREAD_HEADER_SLOTS);
-    }
+    if (context_index >= MAX_INTERRUPTS)
+        lose("maximum interrupt nesting depth (%d) exceeded", MAX_INTERRUPTS);
 
     bind_variable(FREE_INTERRUPT_CONTEXT_INDEX,
                   make_fixnum(context_index + 1),thread);
@@ -842,6 +839,12 @@ fake_foreign_function_call(os_context_t *context)
     foreign_function_call_active_p(thread) = 1;
 #endif
 }
+void fake_foreign_function_call(os_context_t *context)
+{
+    /* context_index incrementing must not be interrupted */
+    assert_blockables_blocked();
+    fake_foreign_function_call_noassert(context);
+}
 
 /* blocks all blockable signals.  If you are calling from a signal handler,
  * the usual signal mask will be restored from the context when the handler
@@ -944,8 +947,7 @@ interrupt_internal_error(os_context_t *context, boolean continuable)
 boolean
 interrupt_handler_pending_p(void)
 {
-    struct thread *thread = arch_os_get_current_thread();
-    struct interrupt_data *data = thread->interrupt_data;
+    struct interrupt_data *data = &thread_interrupt_data(arch_os_get_current_thread());
     return (data->pending_handler != 0);
 }
 
@@ -971,7 +973,7 @@ interrupt_handle_pending(os_context_t *context)
      * pending asynchronous tasks. */
 
     struct thread *thread = arch_os_get_current_thread();
-    struct interrupt_data *data = thread->interrupt_data;
+    struct interrupt_data *data = &thread_interrupt_data(thread);
 
     if (arch_pseudo_atomic_atomic(context)) {
         lose("Handling pending interrupt in pseudo atomic.");
@@ -979,7 +981,7 @@ interrupt_handle_pending(os_context_t *context)
 
     FSHOW_SIGNAL((stderr, "/entering interrupt_handle_pending\n"));
 
-    check_blockables_blocked_or_lose(0);
+    assert_blockables_blocked();
 #ifndef LISP_FEATURE_SB_SAFEPOINT
     /*
      * (On safepoint builds, there is no gc_blocked_deferrables nor
@@ -1077,7 +1079,7 @@ interrupt_handle_pending(os_context_t *context)
             lose("Trapping to run pending handler while GC in progress.");
         }
 
-        check_blockables_blocked_or_lose(0);
+        assert_blockables_blocked();
 
         /* No GC shall be lost. If SUB_GC triggers another GC then
          * that should be handled on the spot. */
@@ -1139,88 +1141,55 @@ void
 interrupt_handle_now(int signal, siginfo_t *info, os_context_t *context)
 {
     boolean were_in_lisp;
-    union interrupt_handler handler;
+    lispobj handler = lisp_sig_handlers[signal];
 
-    check_blockables_blocked_or_lose(0);
+    if (!functionp(handler)) return;
+
+    assert_blockables_blocked();
 
 #if !defined(LISP_FEATURE_WIN32) || defined(LISP_FEATURE_SB_THREAD)
     if (sigismember(&deferrable_sigset,signal))
         check_interrupts_enabled_or_lose(context);
 #endif
 
-    handler = interrupt_handlers[signal];
-
-    if (ARE_SAME_HANDLER(handler.c, SIG_IGN)) {
-        return;
-    }
-
     were_in_lisp = !foreign_function_call_active_p(arch_os_get_current_thread());
     if (were_in_lisp)
     {
-        fake_foreign_function_call(context);
+        // Use the variant of fake_ffc that doesn't do another pthread_sigmask syscall,
+        // as we've just asserted that signals are blocked.
+        fake_foreign_function_call_noassert(context);
     }
 
-    FSHOW_SIGNAL((stderr,
-                  "/entering interrupt_handle_now(%d, info, context)\n",
-                  signal));
-
-    if (ARE_SAME_HANDLER(handler.c, SIG_DFL)) {
-
-        /* This can happen if someone tries to ignore or default one
-         * of the signals we need for runtime support, and the runtime
-         * support decides to pass on it. */
-        lose("no handler for signal %d in interrupt_handle_now(..)", signal);
-
-        // BUG: if a C function pointer can be misaligned such that it
-        // looks to satisfy functionp() then we do the wrong thing.
-    } else if (functionp(handler.lisp)) {
         /* Once we've decided what to do about contexts in a
          * return-elsewhere world (the original context will no longer
          * be available; should we copy it or was nobody using it anyway?)
          * then we should convert this to return-elsewhere */
 
-#ifndef LISP_FEATURE_SB_SAFEPOINT
+#if !defined(LISP_FEATURE_SB_SAFEPOINT) && defined(LISP_FEATURE_C_STACK_IS_CONTROL_STACK)
         /* Leave deferrable signals blocked, the handler itself will
          * allow signals again when it sees fit. */
-#ifdef LISP_FEATURE_C_STACK_IS_CONTROL_STACK
         /* handler.lisp will hide from the GC, will be enabled in the handler itself.
          * Not a problem for the conservative GC. */
         unblock_gc_signals();
 #endif
-#else
+
         WITH_GC_AT_SAFEPOINTS_ONLY()
-#endif
         { // the block is needed for WITH_GC_AT_SAFEPOINTS_ONLY() to work
             DX_ALLOC_SAP(context_sap, context);
             DX_ALLOC_SAP(info_sap, info);
 
             FSHOW_SIGNAL((stderr,"/calling Lisp-level handler\n"));
 
-            funcall3(handler.lisp,
+            funcall3(handler,
                      make_fixnum(signal),
                      info_sap,
                      context_sap);
         }
-    } else {
-        /* This cannot happen in sane circumstances. */
-
-        FSHOW_SIGNAL((stderr,"/calling C-level handler\n"));
-
-#if !defined(LISP_FEATURE_WIN32) || defined(LISP_FEATURE_SB_THREAD)
-        /* Allow signals again. */
-        thread_sigmask(SIG_SETMASK, os_context_sigmask_addr(context), 0);
-        (*handler.c)(signal, info, context);
-#endif
-    }
 
     if (were_in_lisp)
     {
         undo_fake_foreign_function_call(context); /* block signals again */
     }
-
-    FSHOW_SIGNAL((stderr,
-                  "/returning from interrupt_handle_now(%d, info, context)\n",
-                  signal));
 }
 
 /* This is called at the end of a critical section if the indications
@@ -1244,20 +1213,20 @@ run_deferred_handler(struct interrupt_data *data, os_context_t *context)
 }
 
 #ifndef LISP_FEATURE_WIN32
-boolean
-maybe_defer_handler(void *handler, struct interrupt_data *data,
-                    int signal, siginfo_t *info, os_context_t *context)
+static boolean
+can_handle_now(void *handler, struct interrupt_data *data,
+               int signal, siginfo_t *info, os_context_t *context)
 {
     struct thread *thread=arch_os_get_current_thread();
 
-    check_blockables_blocked_or_lose(0);
+    assert_blockables_blocked();
 
     if (read_TLS(INTERRUPT_PENDING,thread) != NIL)
         lose("interrupt already pending");
-    if (thread->interrupt_data->pending_handler)
+    if (thread_interrupt_data(thread).pending_handler)
         lose("there is a pending handler already (PA)");
     if (data->gc_blocked_deferrables)
-        lose("maybe_defer_handler: gc_blocked_deferrables true");
+        lose("can_handle_now: gc_blocked_deferrables true");
 
     /* If interrupts are disabled then INTERRUPT_PENDING is set and
      * not PSEDUO_ATOMIC_INTERRUPTED. This is important for a pseudo
@@ -1270,33 +1239,33 @@ maybe_defer_handler(void *handler, struct interrupt_data *data,
     if ((read_TLS(INTERRUPTS_ENABLED,thread) == NIL) ||
         in_leaving_without_gcing_race_p(thread)) {
         FSHOW_SIGNAL((stderr,
-                      "/maybe_defer_handler(%p,%d): deferred (RACE=%d)\n",
+                      "/can_handle_now(%p,%d): deferred (RACE=%d)\n",
                       handler,signal,
                       in_leaving_without_gcing_race_p(thread)));
         store_signal_data_for_later(data,handler,signal,info,context);
         write_TLS(INTERRUPT_PENDING, T,thread);
         check_interrupt_context_or_lose(context);
-        return 1;
+        return 0;
     }
     /* a slightly confusing test. arch_pseudo_atomic_atomic() doesn't
      * actually use its argument for anything on x86, so this branch
      * may succeed even when context is null (gencgc alloc()) */
     if (arch_pseudo_atomic_atomic(context)) {
         FSHOW_SIGNAL((stderr,
-                      "/maybe_defer_handler(%p,%d): deferred(PA)\n",
+                      "/can_handle_now(%p,%d): deferred(PA)\n",
                       handler,signal));
         store_signal_data_for_later(data,handler,signal,info,context);
         arch_set_pseudo_atomic_interrupted(context);
         check_interrupt_context_or_lose(context);
-        return 1;
+        return 0;
     }
 
     check_interrupt_context_or_lose(context);
 
     FSHOW_SIGNAL((stderr,
-                  "/maybe_defer_handler(%p,%d): not deferred\n",
+                  "/can_handle_now(%p,%d): not deferred\n",
                   handler,signal));
-    return 0;
+    return 1;
 }
 
 static void
@@ -1334,8 +1303,8 @@ maybe_now_maybe_later(int signal, siginfo_t *info, void *void_context)
 {
     SAVE_ERRNO(signal,context,void_context);
     struct thread *thread = arch_os_get_current_thread();
-    struct interrupt_data *data = thread->interrupt_data;
-    if(!maybe_defer_handler(interrupt_handle_now,data,signal,info,context))
+    struct interrupt_data *data = &thread_interrupt_data(thread);
+    if (can_handle_now(interrupt_handle_now, data, signal, info, context))
         interrupt_handle_now(signal, info, context);
     RESTORE_ERRNO;
 }
@@ -1345,7 +1314,7 @@ low_level_interrupt_handle_now(int signal, siginfo_t *info,
                                os_context_t *context)
 {
     /* No FP control fixage needed, caller has done that. */
-    check_blockables_blocked_or_lose(0);
+    assert_blockables_blocked();
     check_interrupts_enabled_or_lose(context);
     (*interrupt_low_level_handlers[signal])(signal, info, context);
 }
@@ -1355,10 +1324,9 @@ low_level_maybe_now_maybe_later(int signal, siginfo_t *info, void *void_context)
 {
     SAVE_ERRNO(signal,context,void_context);
     struct thread *thread = arch_os_get_current_thread();
-    struct interrupt_data *data = thread->interrupt_data;
+    struct interrupt_data *data = &thread_interrupt_data(thread);
 
-    if(!maybe_defer_handler(low_level_interrupt_handle_now,data,
-                            signal,info,context))
+    if (can_handle_now(low_level_interrupt_handle_now, data, signal, info, context))
         low_level_interrupt_handle_now(signal, info, context);
     RESTORE_ERRNO;
 }
@@ -1414,21 +1382,24 @@ sig_stop_for_gc_handler(int __attribute__((unused)) signal,
      * pseudo_atomic_interrupted but without a pending interrupt or
      * GC. GC_BLOCKED_DEFERRABLES is also left at 1. So let's tidy it
      * up. */
-    if (thread->interrupt_data->gc_blocked_deferrables) {
+    if (thread_interrupt_data(thread).gc_blocked_deferrables) {
         FSHOW_SIGNAL((stderr,"cleaning up after gc_blocked_deferrables\n"));
         clear_pseudo_atomic_interrupted(thread);
-        sigcopyset(os_context_sigmask_addr(context),
-                   &thread->interrupt_data->pending_mask);
-        thread->interrupt_data->gc_blocked_deferrables = 0;
+        struct interrupt_data *interrupt_data = &thread_interrupt_data(thread);
+        sigcopyset(os_context_sigmask_addr(context), &interrupt_data->pending_mask);
+        interrupt_data->gc_blocked_deferrables = 0;
     }
 
-    if(thread_state(thread)!=STATE_RUNNING) {
-        // macOS warns if OBJ_FMTX is used to format a 'sword_t'
-        // which fixnum_value() returns.
-        lose("sig_stop_for_gc_handler: wrong thread state: %"OBJ_FMTX,
-             (lispobj)fixnum_value(thread->state));
-    }
+    /* No need to use an atomic memory load here - this thead "owns" its state
+     * for now, and nobody else touches it, the sole exception being that GC
+     * sets it to RUNNING. The loads inside thread_wait_until_not()
+     * are slightly more interesting from that perspective */
+    if (thread->state != STATE_RUNNING)
+        lose("stop_for_gc: bad thread state: %x", (int)thread->state);
 
+    /* We say that the thread is "stopped" as of now, but the blocking operation
+     * occurs below at thread_wait_until_not(STATE_STOPPED). Note that sem_post()
+     * is expressly permitted in signal handlers, and set_thread_state uses it */
     set_thread_state(thread, STATE_STOPPED, 0);
     FSHOW_SIGNAL((stderr,"suspended\n"));
 
@@ -1438,13 +1409,19 @@ sig_stop_for_gc_handler(int __attribute__((unused)) signal,
      * actually a must. */
     scrub_control_stack();
 
-    wait_for_thread_state_change(thread, STATE_STOPPED);
+    /* Now we wait on a semaphore, which, to be pedantic, is not specified as async-safe.
+     * Normally the way to implement a "suspend" operation is to issue any blocking
+     * syscall such as sigsuspend() or select(). Apparently every OS + C runtime that
+     * we wish to support has no problem with sem_wait() here in the signal handler. */
+    int my_state = thread_wait_until_not(STATE_STOPPED, thread);
     FSHOW_SIGNAL((stderr,"resumed\n"));
 
-    if(thread_state(thread)!=STATE_RUNNING) {
-        lose("sig_stop_for_gc_handler: wrong thread state on wakeup: %"OBJ_FMTX,
-             (lispobj)fixnum_value(thread_state(thread)));
-    }
+    /* The state can't go from STOPPED to DEAD because it's this thread is reading
+     * its own state, hence it must be running.
+     * (If we tried to observe a different thread, it could appear to change from
+     * STOPPED to DEAD, skipping RUNNING, because if you blink you might miss it) */
+    if (my_state != STATE_RUNNING)
+        lose("stop_for_gc: bad state on wakeup: %x", my_state);
 
     if (was_in_lisp) {
         undo_fake_foreign_function_call(context);
@@ -1539,11 +1516,11 @@ arrange_return_to_c_function(os_context_t *context,
      */
 
 #ifndef LISP_FEATURE_DARWIN
-    u32 *sp=(u32 *)*os_context_register_addr(context,reg_ESP);
+    uint32_t *sp=(uint32_t *)*os_context_register_addr(context,reg_ESP);
 #endif
 
 #if defined(LISP_FEATURE_DARWIN)
-    u32 *register_save_area = (u32 *)os_allocate(0x40);
+    uint32_t *register_save_area = (uint32_t *)os_allocate(0x40);
 
     FSHOW_SIGNAL((stderr, "/arrange_return_to_lisp_function: preparing to go to function %x, sp: %x\n", function,
                   *os_context_register_addr(context,reg_ESP)));
@@ -1575,7 +1552,7 @@ arrange_return_to_c_function(os_context_t *context,
 #else
 
     /* return address for call_into_lisp: */
-    *(sp-15) = (u32)post_signal_tramp;
+    *(sp-15) = (uint32_t)post_signal_tramp;
     *(sp-14) = function;        /* args for call_into_lisp : function*/
     *(sp-13) = 0;               /*                           arg array */
     *(sp-12) = 0;               /*                           no. args */
@@ -1598,10 +1575,10 @@ arrange_return_to_c_function(os_context_t *context,
 #endif
 
 #elif defined(LISP_FEATURE_X86_64)
-    u64 *sp=(u64 *)*os_context_register_addr(context,reg_RSP);
+    uword_t *sp=(uword_t *)*os_context_register_addr(context,reg_RSP);
 
     /* return address for call_into_lisp: */
-    *(sp-18) = (u64)post_signal_tramp;
+    *(sp-18) = (uint64_t)post_signal_tramp;
 
     *(sp-17)=*os_context_register_addr(context,reg_R15);
     *(sp-16)=*os_context_register_addr(context,reg_R14);
@@ -1842,94 +1819,11 @@ extern void restore_sbcl_signals () {
     for (signal = 0; signal < NSIG; signal++) {
         interrupt_handler_t handler = interrupt_low_level_handlers[signal];
         if (handler) {
-            undoably_install_low_level_interrupt_handler(signal, handler);
+            ll_install_handler(signal, handler);
         }
     }
 }
 
-#if defined(LISP_FEATURE_SB_SAFEPOINT_STRICTLY) && !defined(LISP_FEATURE_WIN32)
-
-static void *
-signal_thread_trampoline(void *pthread_arg)
-{
-    intptr_t signo = (intptr_t) pthread_arg;
-    os_context_t fake_context;
-    siginfo_t fake_info;
-#if defined LISP_FEATURE_PPC || defined LISP_FEATURE_PPC64
-    mcontext_t uc_regs;
-#endif
-
-    memset(&fake_info, 0, sizeof(fake_info));
-    memset(&fake_context, 0, sizeof(fake_context));
-#if defined LISP_FEATURE_PPC || defined LISP_FEATURE_PPC64
-    memset(&uc_regs, 0, sizeof(uc_regs));
-    fake_context.uc_mcontext.uc_regs = &uc_regs;
-#endif
-
-    *os_context_pc_addr(&fake_context) = (intptr_t) &signal_thread_trampoline;
-#ifdef ARCH_HAS_STACK_POINTER /* aka x86(-64) */
-    *os_context_sp_addr(&fake_context) = (intptr_t) __builtin_frame_address(0);
-#endif
-
-    signal_handler_callback(interrupt_handlers[signo].lisp,
-                            signo, &fake_info, &fake_context);
-    return 0;
-}
-
-static void
-sigprof_handler_trampoline(int signal, siginfo_t *info, void *void_context)
-{
-    SAVE_ERRNO(signal,context,void_context);
-    struct thread *self = arch_os_get_current_thread();
-
-    /* alloc() is not re-entrant and still uses pseudo atomic (even though
-     * inline allocation does not).  In this case, give up. */
-    if (get_pseudo_atomic_atomic(self))
-        goto cleanup;
-
-    struct alloc_region tmp = self->alloc_region;
-    self->alloc_region = self->sprof_alloc_region;
-    self->sprof_alloc_region = tmp;
-
-    interrupt_handle_now_handler(signal, info, void_context);
-
-    /* And we're back.  We know that the SIGPROF handler never unwinds
-     * non-locally, and can simply swap things back: */
-
-    tmp = self->alloc_region;
-    self->alloc_region = self->sprof_alloc_region;
-    self->sprof_alloc_region = tmp;
-
-cleanup:
-    ; /* Dear C compiler, it's OK to have a label here. */
-    RESTORE_ERRNO;
-}
-
-static void
-spawn_signal_thread_handler(int signal, siginfo_t *info, void *void_context)
-{
-    SAVE_ERRNO(signal,context,void_context);
-
-    pthread_attr_t attr;
-    pthread_t th;
-
-    if (pthread_attr_init(&attr))
-        goto lost;
-    if (pthread_attr_setstacksize(&attr, thread_control_stack_size))
-        goto lost;
-    if (pthread_create(&th, &attr, &signal_thread_trampoline, (void*)(intptr_t) signal))
-        goto lost;
-    if (pthread_attr_destroy(&attr))
-        goto lost;
-
-    RESTORE_ERRNO;
-    return;
-
-lost:
-    lose("spawn_signal_thread_handler");
-}
-#endif
-
 static void
 low_level_handle_now_handler(int signal, siginfo_t *info, void *void_context)
 {
@@ -1939,8 +1833,7 @@ low_level_handle_now_handler(int signal, siginfo_t *info, void *void_context)
 }
 
 void
-undoably_install_low_level_interrupt_handler (int signal,
-                                              interrupt_handler_t handler)
+ll_install_handler (int signal, interrupt_handler_t handler)
 {
     struct sigaction sa;
 
@@ -1960,7 +1853,7 @@ undoably_install_low_level_interrupt_handler (int signal,
      * but we don't actually want to defer it.  And if we put it only
      * into blockable_sigset, we'd have to special-case it around thread
      * creation at least. */
-    if (signal == SIGPIPE)
+    if (signal == SIGURG)
         sa.sa_sigaction = low_level_handle_now_handler;
 #endif
 
@@ -1969,7 +1862,7 @@ undoably_install_low_level_interrupt_handler (int signal,
 #if defined(LISP_FEATURE_C_STACK_IS_CONTROL_STACK)
     if(signal==SIG_MEMORY_FAULT) {
         sa.sa_flags |= SA_ONSTACK;
-# ifdef LISP_FEATURE_SB_SAFEPOINT
+# if defined LISP_FEATURE_SB_SAFEPOINT && !defined LISP_FEATURE_DARWIN
         sigaddset(&sa.sa_mask, SIGRTMIN);
         sigaddset(&sa.sa_mask, SIGRTMIN+1);
 # endif
@@ -1983,76 +1876,51 @@ undoably_install_low_level_interrupt_handler (int signal,
 #endif
 
 /* This is called from Lisp. */
-uword_t
-install_handler(int signal, void handler(int, siginfo_t*, os_context_t*),
-                lispobj ohandler, // (SIMPLE-VECTOR 1) as a tagged pointer
+void
+install_handler(int signal, lispobj handler,
                 int __attribute__((unused)) synchronous)
 {
 #ifndef LISP_FEATURE_WIN32
     struct sigaction sa;
-    sigset_t old;
 
-    FSHOW((stderr, "/entering POSIX install_handler(%d, ..)\n", signal));
-
-    block_blockable_signals(&old);
-
-    FSHOW((stderr, "/interrupt_low_level_handlers[signal]=%p\n",
-           interrupt_low_level_handlers[signal]));
     if (interrupt_low_level_handlers[signal]==0) {
-        if (handler == 0)
-            sa.sa_sigaction = (void (*)(int, siginfo_t*, void*))SIG_DFL;
-        else if ((lispobj)handler == 1)
-            sa.sa_sigaction = (void (*)(int, siginfo_t*, void*))SIG_IGN;
+        // Our "abstract" values for SIG_DFL and SIG_IGN are 0 and 1
+        // respectively which are probably the real values from signal.h
+        // but this way way don't need to put them in grovel-headers.c
+        if (handler== 0 || handler==1) {
+            memset(&sa, 0, sizeof sa);
+            sa.sa_handler = handler ? SIG_IGN : SIG_DFL;
+            // assign the OS level action before clearing the lisp function.
+            // (If a signal were to be delivered to the C trampoline when the lisp
+            // function is NIL, we'd get the effect of :IGNORE regardless
+            // of what the default action should be)
+            sigaction(signal, &sa, NULL);
+            lisp_sig_handlers[signal] = NIL;
+            return;
+        }
 #ifdef LISP_FEATURE_SB_SAFEPOINT_STRICTLY
-        else if (signal == SIGPROF)
+        if (signal == SIGPROF)
             sa.sa_sigaction = sigprof_handler_trampoline;
         else if (!synchronous)
             sa.sa_sigaction = spawn_signal_thread_handler;
+        else
 #endif
-        else if (sigismember(&deferrable_sigset, signal))
+        if (sigismember(&deferrable_sigset, signal))
             sa.sa_sigaction = maybe_now_maybe_later;
         else
             sa.sa_sigaction = interrupt_handle_now_handler;
 
         sa.sa_mask = blockable_sigset;
         sa.sa_flags = SA_SIGINFO | SA_RESTART | OS_SA_NODEFER;
+        // ensure the C handler sees a lisp function before doing sigaction()
+        lisp_sig_handlers[signal] = handler;
         sigaction(signal, &sa, NULL);
+    } else {
+        // When there's a low-level handler, we must leave it alone.
+        // Give it the lisp function to call if it decides to forward a signal.
+        // SIG_IGN and SIG_DFL don't always do what you think in such case.
+        lisp_sig_handlers[signal] = functionp(handler) ? handler : NIL;
     }
-
-    // We can GC-safely read interrupt_low_level_handlers[signal] and assign
-    // into ohandler despite that the C stack and registers are not roots on
-    // precise GC. That is, if GC were to occur as depicted:
-    //   obj = old_handler
-    //   -- GC signal occurs here --
-    //   result = obj
-    // then the register or stack word holding 'obj' could be obsolete.
-    // But that won't happen- the STOP_FOR_GC signal is currently blocked
-    // due to the block_blockable_signals() at the top of this function.
-    // However, there is an unlikely other reason for a wrong result -
-    // if the C function address does not have at least N_FIXNUM_TAG_BITS
-    // of alignment, then we say that it was NIL instead of a fixnum.
-    lispobj obj = interrupt_handlers[signal].lisp;
-    uword_t result = -1;
-    if ((void*)obj == (void*)SIG_DFL)
-        result = 0;
-    else if ((void*)obj == (void*)SIG_IGN)
-        result = 1;
-    else if (ohandler) // only store into 'ohandler' when it was supplied
-        VECTOR(ohandler)->data[0] = (functionp(obj)||fixnump(obj)) ? obj : NIL;
-
-    interrupt_handlers[signal].c = handler;
-
-    thread_sigmask(SIG_SETMASK, &old, 0);
-
-    FSHOW((stderr, "/leaving POSIX install_handler(%d, ..)\n", signal));
-
-    // After the thread's signal mask is restored to the mask in 'old',
-    // it is not possible to GC-safely refer to a Lisp function from C code,
-    // which is to say, naively utilizing "return obj;" would be wrong.
-    return result;
-#else
-    /* Probably-wrong Win32 hack */
-    return 0;
 #endif
 }
 
@@ -2073,7 +1941,7 @@ void
 interrupt_init(void)
 {
 #if !defined(LISP_FEATURE_WIN32) || defined(LISP_FEATURE_SB_THREAD)
-    int i;
+    int __attribute__((unused)) i;
     SHOW("entering interrupt_init()");
     sigemptyset(&deferrable_sigset);
     sigemptyset(&blockable_sigset);
@@ -2083,18 +1951,18 @@ interrupt_init(void)
     sigaddset_gc(&gc_sigset);
 #endif
 
+#ifdef LISP_FEATURE_LIBUNWIND_BACKTRACE
+    // Use this only if you know what you're doing
+    void backtrace_lisp_threads(int, siginfo_t*, os_context_t*);
+    ll_install_handler(SIGXCPU, backtrace_lisp_threads);
+#endif
+
 #ifndef LISP_FEATURE_WIN32
     /* Set up high level handler information. */
     for (i = 0; i < NSIG; i++) {
-        interrupt_handlers[i].c =
-            /* (The cast here blasts away the distinction between
-             * SA_SIGACTION-style three-argument handlers and
-             * signal(..)-style one-argument handlers, which is OK
-             * because it works to call the 1-argument form where the
-             * 3-argument form is expected.) */
-            (void (*)(int, siginfo_t*, os_context_t*))SIG_DFL;
+        lisp_sig_handlers[i] = NIL;
     }
-    undoably_install_low_level_interrupt_handler(SIGABRT, sigabrt_handler);
+    ll_install_handler(SIGABRT, sigabrt_handler);
 #endif
     SHOW("returning from interrupt_init()");
 }
@@ -2119,15 +1987,15 @@ lisp_memory_fault_error(os_context_t *context, os_vm_address_t addr)
     unsigned int offset = code ? pc - (char*)code : 0;
     if (offset)
         corruption_warning_and_maybe_lose(
-            "Memory fault at %p (pc=%p [code %p+0x%X ID 0x%x], fp=%p, sp=%p) tid %#lx",
+            "Memory fault at %p (pc=%p [code %p+0x%X ID 0x%x], fp=%p, sp=%p)" THREAD_ID_LABEL,
             addr, pc, code, offset, code_serialno(code),
             os_context_frame_pointer(context),
-            *os_context_sp_addr(context), thread_self()); // = 0 if -sb-thread
+            *os_context_sp_addr(context), THREAD_ID_VALUE);
     else
         corruption_warning_and_maybe_lose(
-            "Memory fault at %p (pc=%p, fp=%p, sp=%p) tid %#lx",
+            "Memory fault at %p (pc=%p, fp=%p, sp=%p)" THREAD_ID_LABEL,
             addr, pc, os_context_frame_pointer(context),
-            *os_context_sp_addr(context), thread_self()); // = 0 if -sb-thread
+            *os_context_sp_addr(context), THREAD_ID_VALUE);
 #else
     corruption_warning_and_maybe_lose("Memory fault at %p (pc=%p)",
                                       addr, *os_context_pc_addr(context));
