@@ -35,8 +35,6 @@
 #include "genesis/static-symbols.h"
 #include "genesis/fdefn.h"
 
-#include <sys/socket.h>
-#include <sys/utsname.h>
 #include <errno.h>
 
 #include <sys/types.h>
@@ -64,11 +62,98 @@ int personality (unsigned long);
 #include <sys/personality.h>
 #endif
 
-#if defined(LISP_FEATURE_SB_THREAD) && defined(LISP_FEATURE_SB_FUTEX) && !defined(LISP_FEATURE_SB_PTHREAD_FUTEX)
+#ifdef LISP_FEATURE_SB_FUTEX
 #include <sys/syscall.h>
 #include <unistd.h>
 #include <errno.h>
 
+#ifdef MUTEX_EVENTRECORDING
+#include "genesis/mutex.h"
+#define MAXEVENTS 200
+static struct {
+    struct thread* th;
+    struct timespec ts;
+    char *label;
+    char *mutex_name;
+    sword_t timeout;
+} events[MAXEVENTS];
+static int record_mutex_events;
+static int eventcount;
+
+void lisp_mutex_event(char *string) {
+    if (record_mutex_events) {
+        int id = __sync_fetch_and_add(&eventcount, 1);
+        if (id >= MAXEVENTS) lose("event buffer overflow");
+        clock_gettime(CLOCK_REALTIME, &events[id].ts);
+        events[id].th = current_thread;
+        events[id].label = string;
+        events[id].mutex_name = 0;
+        events[id].timeout = -1;
+    }
+}
+void lisp_mutex_event1(char *string, char *string2) {
+    if (record_mutex_events) {
+        int id = __sync_fetch_and_add(&eventcount, 1);
+        if (id >= MAXEVENTS) lose("event buffer overflow");
+        clock_gettime(CLOCK_REALTIME, &events[id].ts);
+        events[id].th = current_thread;
+        events[id].label = string;
+        events[id].mutex_name = string2;
+        events[id].timeout = -1;
+    }
+}
+void lisp_mutex_event2(char *string, char *string2, uword_t usec) {
+    if (record_mutex_events) {
+        int id = __sync_fetch_and_add(&eventcount, 1);
+        if (id >= MAXEVENTS) lose("event buffer overflow");
+        clock_gettime(CLOCK_REALTIME, &events[id].ts);
+        events[id].th = current_thread;
+        events[id].label = string;
+        events[id].mutex_name = string2;
+        events[id].timeout = usec;
+    }
+}
+void lisp_mutex_start_eventrecording() {
+    eventcount = 0;
+    record_mutex_events = 1;
+}
+void lisp_mutex_done_eventrecording() {
+    record_mutex_events = 0;
+    int i;
+    fprintf(stderr, "event log:\n");
+    struct timespec basetime = events[0].ts;
+    for(i=0; i<eventcount;++i) {
+        struct thread *th = events[i].th;
+        struct thread_instance *ti = (void*)native_pointer(th->lisp_thread);
+        struct timespec rel_time = events[i].ts;
+        rel_time.tv_sec -= basetime.tv_sec;
+        rel_time.tv_nsec -= basetime.tv_nsec;
+        if (rel_time.tv_nsec<0) rel_time.tv_nsec += 1000 * 1000 * 1000, rel_time.tv_sec--;
+        lispobj threadname = ti->name;
+        if (events[i].timeout >= 0) // must also have mutex_name in this case
+            fprintf(stderr, "[%d.%09ld] %s: %s '%s' timeout %ld\n",
+                    (int)rel_time.tv_sec, rel_time.tv_nsec,
+                    (char*)VECTOR(threadname)->data,
+                    events[i].label, events[i].mutex_name, events[i].timeout);
+        else if (events[i].mutex_name)
+            fprintf(stderr, "[%d.%09ld] %s: %s '%s'\n",
+                    (int)rel_time.tv_sec, rel_time.tv_nsec,
+                    (char*)VECTOR(threadname)->data,
+                    events[i].label, events[i].mutex_name);
+        else
+            fprintf(stderr, "[%d.%09ld] %s: %s\n",
+                    (int)rel_time.tv_sec, rel_time.tv_nsec,
+                    (char*)VECTOR(threadname)->data,
+                    events[i].label);
+    }
+    fprintf(stderr, "-----\n");
+}
+#else
+#define lisp_mutex_event(x)
+#define lisp_mutex_event1(x,y)
+#define lisp_mutex_event2(x,y,z)
+#endif
+
 /* values taken from the kernel's linux/futex.h.  This header file
    doesn't exist in userspace, which is our excuse for not grovelling
    them automatically */
@@ -138,14 +223,21 @@ futex_wait(int *lock_word, int oldval, long sec, unsigned long usec)
   struct timespec timeout;
   int t;
 
+#ifdef MUTEX_EVENTRECORDING
+    struct mutex* m = (void*)((char*)lock_word - offsetof(struct mutex,state));
+    char *name = m->name != NIL ? (char*)VECTOR(m->name)->data : "(unnamed)";
+#endif
   if (sec<0) {
+      lisp_mutex_event1("start futex wait", name);
       t = sys_futex(lock_word, futex_wait_op(), oldval, 0);
   }
   else {
       timeout.tv_sec = sec;
       timeout.tv_nsec = usec * 1000;
+      lisp_mutex_event2("start futex timedwait", name, usec);
       t = sys_futex(lock_word, futex_wait_op(), oldval, &timeout);
   }
+  lisp_mutex_event1("back from sys_futex", name);
   if (t==0)
       return 0;
   else if (errno==ETIMEDOUT)
@@ -160,6 +252,11 @@ futex_wait(int *lock_word, int oldval, long sec, unsigned long usec)
 int
 futex_wake(int *lock_word, int n)
 {
+#ifdef MUTEX_EVENTRECORDING
+    struct mutex* m = (void*)((char*)lock_word - offsetof(struct mutex,state));
+    char *name = m->name != NIL ? (char*)VECTOR(m->name)->data : "(unnamed)";
+    lisp_mutex_event1("waking futex", name);
+#endif
     return sys_futex(lock_word, futex_wake_op(),n,0);
 }
 #endif
@@ -168,7 +265,7 @@ futex_wake(int *lock_word, int n)
 void os_init(char __attribute__((unused)) *argv[],
              char __attribute__((unused)) *envp[])
 {
-#if defined(LISP_FEATURE_SB_THREAD) && defined(LISP_FEATURE_SB_FUTEX) && !defined(LISP_FEATURE_SB_PTHREAD_FUTEX)
+#ifdef LISP_FEATURE_SB_FUTEX
     futex_init();
 #endif
 
@@ -288,22 +385,6 @@ sigsegv_handler(int signal, siginfo_t *info, os_context_t *context)
 {
     os_vm_address_t addr = arch_get_bad_addr(signal, info, context);
 
-#ifdef LISP_FEATURE_ALPHA
-    /* Alpha stuff: This is the end of a pseudo-atomic section during
-       which a signal was received.  We must deal with the pending
-       interrupt (see also interrupt.c, ../code/interrupt.lisp)
-
-       (how we got here: when interrupting, we set bit 63 in reg_ALLOC.
-       At the end of the atomic section we tried to write to reg_ALLOC,
-       got a SIGSEGV (there's nothing mapped there) so ended up here. */
-    if (addr != NULL &&
-        *os_context_register_addr(context, reg_ALLOC) & (1L<<63)) {
-        *os_context_register_addr(context, reg_ALLOC) -= (1L<<63);
-        interrupt_handle_pending(context);
-        return;
-    }
-#endif
-
 #ifdef LISP_FEATURE_SB_SAFEPOINT
     if (!handle_safepoint_violation(context, addr))
 #endif
@@ -321,8 +402,7 @@ void
 os_install_interrupt_handlers(void)
 {
     if (INSTALL_SIG_MEMORY_FAULT_HANDLER) {
-    undoably_install_low_level_interrupt_handler(SIG_MEMORY_FAULT,
-                                                 sigsegv_handler);
+    ll_install_handler(SIG_MEMORY_FAULT, sigsegv_handler);
     }
 
     /* OAOOM c.f. sunos-os.c.
@@ -330,26 +410,12 @@ os_install_interrupt_handlers(void)
 #ifdef LISP_FEATURE_SB_THREAD
 # ifdef LISP_FEATURE_SB_SAFEPOINT
 #  ifdef LISP_FEATURE_SB_THRUPTION
-    undoably_install_low_level_interrupt_handler(SIGPIPE, thruption_handler);
+    ll_install_handler(SIGURG, thruption_handler);
 #  endif
 # else
-    undoably_install_low_level_interrupt_handler(SIG_STOP_FOR_GC,
-                                                 sig_stop_for_gc_handler);
+    ll_install_handler(SIG_STOP_FOR_GC, sig_stop_for_gc_handler);
 # endif
 #endif
-
-#ifdef LISP_FEATURE_LIBUNWIND_BACKTRACE
-    // Use this only if you know what you're doing
-    void backtrace_lisp_threads(int, siginfo_t*, os_context_t*);
-    fprintf(stderr, "SIGXCPU (%d), will backtrace in all threads\n", SIGXCPU);
-    sigdelset(&deferrable_sigset, SIGXCPU);
-    sigdelset(&blockable_sigset, SIGXCPU);
-    undoably_install_low_level_interrupt_handler(SIGXCPU, backtrace_lisp_threads);
-    sigset_t ss;
-    sigemptyset(&ss);
-    sigaddset(&ss, SIGXCPU);
-    pthread_sigmask(SIG_UNBLOCK, &ss, 0);
-#endif
 }
 
 char *os_get_runtime_executable_path()
