@@ -32,13 +32,21 @@
 # include <dlfcn.h>
 #endif
 
+/*
+ * historically, this used sysconf to select the runtime page size
+ * per recent changes on other arches and discussion on sbcl-devel,
+ * however, this is not necessary -- the VM page size need not match
+ * the OS page size (and the default backend page size has been
+ * ramped up accordingly for efficiency reasons).
+*/
+os_vm_size_t os_vm_page_size = BACKEND_PAGE_BYTES;
+
 /* Expose to Lisp the value of the preprocessor define. Don't touch! */
 int install_sig_memory_fault_handler = INSTALL_SIG_MEMORY_FAULT_HANDLER;
 
 /* Except for os_zero, these routines are only called by Lisp code.
  * These routines may also be replaced by os-dependent versions
- * instead. See hpux-os.c for some useful restrictions on actual
- * usage. */
+ * instead. */
 
 #ifdef LISP_FEATURE_CHENEYGC
 void
@@ -92,8 +100,8 @@ os_get_errno(void)
     return errno;
 }
 
-
-#if defined(LISP_FEATURE_SB_THREAD) && (!defined(CANNOT_USE_POSIX_SEM_T) || defined(LISP_FEATURE_WIN32))
+#if defined LISP_FEATURE_SB_THREAD && !defined LISP_FEATURE_SB_SAFEPOINT \
+  && !defined CANNOT_USE_POSIX_SEM_T
 
 void
 os_sem_init(os_sem_t *sem, unsigned int value)
@@ -130,7 +138,7 @@ os_sem_destroy(os_sem_t *sem)
 
 #endif
 
-/* When :SB-DYNAMIC-CORE is enabled, the special category of /static/ foreign
+/* When :LINKAGE-TABLE is enabled, the special category of /static/ foreign
  * symbols disappears. Foreign fixups are resolved to linkage table locations
  * during genesis, and for each of them a record is added to
  * REQUIRED_FOREIGN_SYMBOLS vector, of the form "name" for a function reference,
@@ -141,7 +149,7 @@ os_sem_destroy(os_sem_t *sem)
  * table entry for each element of REQUIRED_FOREIGN_SYMBOLS.
  */
 
-#if defined(LISP_FEATURE_SB_DYNAMIC_CORE) && !defined(LISP_FEATURE_WIN32)
+#if defined(LISP_FEATURE_LINKAGE_TABLE) && !defined(LISP_FEATURE_WIN32)
 void *
 os_dlsym_default(char *name)
 {
@@ -153,9 +161,13 @@ os_dlsym_default(char *name)
 int lisp_linkage_table_n_prelinked;
 void os_link_runtime()
 {
-#ifdef LISP_FEATURE_SB_DYNAMIC_CORE
-    char *link_target = (char*)(intptr_t)LINKAGE_TABLE_SPACE_START;
-    void *validated_end = link_target;
+    // There is a potentially better technique we could use which would simplify
+    // this function, rendering REQUIRED_FOREIGN_SYMBOLS unnecessary, namely:
+    // all we need are two prefilled entries: one for dlsym() itself, and one
+    // for the allocation region overflow handler ("alloc" or "alloc_tramp").
+    // Lisp can fill in the linkage table as the very first action on startup.
+#ifdef LISP_FEATURE_LINKAGE_TABLE
+    int entry_index = 0;
     lispobj symbol_name;
     char *namechars;
     boolean datap;
@@ -175,41 +187,19 @@ void os_link_runtime()
         namechars = (void*)(intptr_t)(VECTOR(symbol_name)->data);
         result = os_dlsym_default(namechars);
 
-        if (link_target == validated_end) {
-            validated_end = (char*)validated_end + os_vm_page_size;
-#ifdef LISP_FEATURE_WIN32
-            os_validate_recommit(link_target,os_vm_page_size);
-#endif
-        }
         if (result) {
-            arch_write_linkage_table_entry(link_target, result, datap);
+            arch_write_linkage_table_entry(entry_index, result, datap);
         } else { // startup might or might not work. ymmv
-            printf("Missing required foreign symbol '%s'\n", namechars);
+            fprintf(stderr, "Missing required foreign symbol '%s'\n", namechars);
         }
 
-        link_target += LINKAGE_TABLE_ENTRY_SIZE;
+        ++entry_index;
     }
-#ifdef LISP_FEATURE_ARM
-    lispobj* static_space = (lispobj*)STATIC_SPACE_START;
-    extern uword_t alloc_tramp(uword_t);
-    static_space[6] = (lispobj)&alloc_tramp; // for LR rN, [NULL, #k] ; BLX rN
-#endif
-#ifdef CALL_INTO_C
-    extern long call_into_c();
-    SYMBOL(CALL_INTO_C)->value = (lispobj)call_into_c;
-#endif
-#endif /* LISP_FEATURE_SB_DYNAMIC_CORE */
+#endif /* LISP_FEATURE_LINKAGE_TABLE */
 }
 
 void os_unlink_runtime()
 {
-#ifdef CALL_INTO_C
-    SYMBOL(CALL_INTO_C)->value = UNBOUND_MARKER_WIDETAG;
-#endif
-#ifdef LISP_FEATURE_ARM
-    lispobj* static_space = (lispobj*)STATIC_SPACE_START;
-    static_space[6] = 0;
-#endif
 }
 
 boolean
@@ -236,17 +226,9 @@ gc_managed_heap_space_p(lispobj addr)
 
 /* Remap a part of an already existing memory mapping from a file,
  * and/or create a new mapping as need be */
-void* load_core_bytes(int fd, int offset, os_vm_address_t addr, os_vm_size_t len)
+void* load_core_bytes(int fd, os_vm_offset_t offset, os_vm_address_t addr, os_vm_size_t len)
 {
     int fail = 0;
-#ifdef LISP_FEATURE_HPUX
-    // Revision afcfb8b5da said that mmap() didn't work on HPUX, changing to use read() instead.
-    // Strangely it also read 4K at a time into a buffer and used memcpy to transfer the buffer.
-    // I don't see why, and given the lack of explanation, I've simplified to 1 read.
-    fail = lseek(fd, offset, SEEK_SET) == (off_t)-1 || read(fd, addr, len) != (ssize_t)len;
-    // This looks bogus but harmlesss, so I'm leaving it.
-    os_flush_icache(addr, len);
-#else
     os_vm_address_t actual;
     actual = mmap(addr, len,
                   // If mapping to a random address, then the assumption is
@@ -264,11 +246,11 @@ void* load_core_bytes(int fd, int offset, os_vm_address_t addr, os_vm_size_t len
     } else if (addr && (addr != actual)) {
         fail = 1;
     }
-#endif
     if (fail)
-        lose("load_core_bytes(%d,%x,%lx,%x) failed", fd, offset, (long)addr, (int)len);
+        lose("load_core_bytes(%d,%zx,%p,%zx) failed", fd, offset, addr, len);
     return (void*)actual;
 }
+#endif
 
 boolean
 gc_managed_addr_p(lispobj addr)
@@ -288,4 +270,4 @@ gc_managed_addr_p(lispobj addr)
     return 0;
 }
 
-#endif
+
