@@ -152,16 +152,11 @@ vodxprint_fun(const char *fmt, va_list args)
 #endif
     int original_errno = errno;
 
-    QSHOW_BLOCK;
-
     char buf[1024];
     int n = 0;
 
 #ifdef LISP_FEATURE_SB_THREAD
-    struct thread *arch_os_get_current_thread(void);
-    struct thread *self = arch_os_get_current_thread();
-    void *pth = self ? (void *) self->os_thread : 0;
-    snprintf(buf, sizeof(buf), "[%p/%p] ", self, pth);
+    snprintf(buf, sizeof(buf), "["THREAD_ID_LABEL"] ", THREAD_ID_VALUE);
     n = strlen(buf);
 #endif
 
@@ -184,8 +179,6 @@ vodxprint_fun(const char *fmt, va_list args)
     fflush(stderr);
 #endif
 
-    QSHOW_UNBLOCK;
-
 #ifdef LISP_FEATURE_WIN32
     SetLastError(lastError);
 #endif
@@ -216,9 +209,6 @@ fshow_fun(void __attribute__((__unused__)) *ignored,
 #ifdef LISP_FEATURE_GENCGC
 #include "gencgc-alloc-region.h" /* genesis/thread.h needs this */
 #endif
-#if defined(LISP_FEATURE_WIN32)
-# include "win32-thread-private-events.h" /* genesis/thread.h needs this */
-#endif
 #include "genesis/static-symbols.h"
 #include "genesis/primitive-objects.h"
 #include "genesis/static-symbols.h"
@@ -293,12 +283,7 @@ static void print_unknown(lispobj obj)
   printf("unknown object: %p", (void *)obj);
 }
 
-/* Except for Alpha, we define sword_t as intptr_t, and 32-bit Darwin
- * defines intptr_t as long, so the printf conversion is "ld", not "d".
- * Alpha (32-on-64) defines sword_t as s32, so we need just "d". */
-#ifdef LISP_FEATURE_ALPHA
-# define OBJ_FMTd "d"
-#elif defined(PRIdPTR)
+#ifdef PRIdPTR
 # define OBJ_FMTd PRIdPTR
 #else
 # error "Your inttypes.h is lame"
@@ -591,13 +576,8 @@ lispobj symbol_function(lispobj* symbol)
 
 static void print_fun_or_otherptr(lispobj obj)
 {
-#ifndef LISP_FEATURE_ALPHA
     lispobj *ptr;
     unsigned long header;
-#else
-    u32 *ptr;
-    u32 header;
-#endif
     int count, type, index;
     char buffer[16];
 
@@ -782,11 +762,7 @@ static void print_fun_or_otherptr(lispobj obj)
 
     case SAP_WIDETAG:
         NEWLINE_OR_RETURN;
-#ifndef LISP_FEATURE_ALPHA
-        printf("0x%08lx", (unsigned long) *ptr);
-#else
-        printf("0x%016lx", *(lispobj*)(ptr+1));
-#endif
+        printf("%p", (void*)*ptr);
         break;
 
     case WEAK_POINTER_WIDETAG:
