@@ -29,12 +29,16 @@
 #include "interr.h"
 #include "gc.h"
 #include "gc-internal.h"
+#include "gc-private.h"
 #include "thread.h"
+#include "genesis/gc-tables.h"
 #include "genesis/primitive-objects.h"
 #include "genesis/static-symbols.h"
 #include "genesis/layout.h"
+#include "genesis/defstruct-description.h"
 #include "genesis/hash-table.h"
-#include "gencgc.h"
+#include "code.h"
+#include "getallocptr.h"
 
 /* We don't ever do purification with GENCGC as of 1.0.5.*. There was
  * a lot of hairy and fragile ifdeffage in here to support purify on
@@ -53,20 +57,20 @@ static lispobj *dynamic_space_purify_pointer;
 
 static lispobj *read_only_end, *static_end;
 
+/* These are private to purify, not to be confused with the external symbols
+ * named 'read_only_space_free_pointer', respectively 'static_space' */
 static lispobj *read_only_free, *static_free;
 
 static lispobj *pscav(lispobj *addr, long nwords, boolean constant);
 
 #define LATERBLOCKSIZE 1020
-#define LATERMAXCOUNT 10
 
 static struct
 later {
     struct later *next;
-    union {
-        lispobj *ptr;
-        long count;
-    } u[LATERBLOCKSIZE];
+    // slightly denser packing vs a struct of a lispobj* and an int
+    lispobj *ptr[LATERBLOCKSIZE];
+    int count[LATERBLOCKSIZE];
 } *later_blocks = NULL;
 static long later_count = 0;
 
@@ -95,13 +99,13 @@ newspace_alloc(long nwords, int constantp)
     gc_assert((nwords & 1) == 0);
     if(constantp) {
         if(read_only_free + nwords >= (lispobj *)READ_ONLY_SPACE_END) {
-            lose("Ran out of read-only space while purifying!\n");
+            lose("Ran out of read-only space while purifying!");
         }
         ret=read_only_free;
         read_only_free+=nwords;
     } else {
         if(static_free + nwords >= (lispobj *)STATIC_SPACE_END) {
-            lose("Ran out of static space while purifying!\n");
+            lose("Ran out of static space while purifying!");
         }
         ret=static_free;
         static_free+=nwords;
@@ -110,33 +114,22 @@ newspace_alloc(long nwords, int constantp)
 }
 
 
+/* Enqueue <where,count> into later_blocks */
 static void
 pscav_later(lispobj *where, long count)
 {
     struct later *new;
 
-    if (count > LATERMAXCOUNT) {
-        while (count > LATERMAXCOUNT) {
-            pscav_later(where, LATERMAXCOUNT);
-            count -= LATERMAXCOUNT;
-            where += LATERMAXCOUNT;
-        }
+    if (later_blocks == NULL || later_count == LATERBLOCKSIZE) {
+        new  = (struct later *)malloc(sizeof(struct later));
+        new->next = later_blocks;
+        later_blocks = new;
+        later_count = 0;
     }
-    else {
-        if (later_blocks == NULL || later_count == LATERBLOCKSIZE ||
-            (later_count == LATERBLOCKSIZE-1 && count > 1)) {
-            new  = (struct later *)malloc(sizeof(struct later));
-            new->next = later_blocks;
-            if (later_blocks && later_count < LATERBLOCKSIZE)
-                later_blocks->u[later_count].ptr = NULL;
-            later_blocks = new;
-            later_count = 0;
-        }
 
-        if (count != 1)
-            later_blocks->u[later_count++].count = count;
-        later_blocks->u[later_count++].ptr = where;
-    }
+    later_blocks->count[later_count] = count;
+    later_blocks->ptr[later_count] = where;
+    ++later_count;
 }
 
 static lispobj
@@ -144,11 +137,11 @@ ptrans_boxed(lispobj thing, lispobj header, boolean constant)
 {
     /* Allocate it */
     lispobj *old = native_pointer(thing);
-    long nwords = sizetab[widetag_of(header)](old);
+    long nwords = sizetab[header_widetag(header)](old);
     lispobj *new = newspace_alloc(nwords,constant);
 
     /* Copy it. */
-    bcopy(old, new, nwords * sizeof(lispobj));
+    memcpy(new, old, nwords * sizeof(lispobj));
 
     /* Deposit forwarding pointer. */
     lispobj result = make_lispobj(new, lowtag_of(thing));
@@ -166,19 +159,17 @@ ptrans_boxed(lispobj thing, lispobj header, boolean constant)
 static lispobj
 ptrans_instance(lispobj thing, lispobj header, boolean /* ignored */ constant)
 {
-    struct layout *layout =
-      (struct layout *) native_pointer(((struct instance *)native_pointer(thing))->slots[0]);
-    lispobj pure = layout->pure;
-
-    switch (pure) {
-    case T:
-        return (ptrans_boxed(thing, header, 1));
-    case NIL:
-        return (ptrans_boxed(thing, header, 0));
-    default:
-        gc_abort();
-        return NIL; /* dummy value: return something ... */
+    constant = 0;
+    lispobj info = LAYOUT(instance_layout(native_pointer(thing)))->info;
+    if (info != NIL) {
+        lispobj pure = ((struct defstruct_description*)native_pointer(info))->pure;
+        if (pure != NIL && pure != T) {
+            gc_abort();
+            return NIL; /* dummy value: return something ... */
+        }
+        constant = (pure == T);
     }
+    return ptrans_boxed(thing, header, constant);
 }
 
 static lispobj
@@ -186,11 +177,11 @@ ptrans_fdefn(lispobj thing, lispobj header)
 {
     /* Allocate it */
     lispobj *old = native_pointer(thing);
-    long nwords = sizetab[widetag_of(header)](old);
+    long nwords = sizetab[header_widetag(header)](old);
     lispobj *new = newspace_alloc(nwords, 0);    /* inconstant */
 
     /* Copy it. */
-    bcopy(old, new, nwords * sizeof(lispobj));
+    memcpy(new, old, nwords * sizeof(lispobj));
 
     /* Deposit forwarding pointer. */
     lispobj result = make_lispobj(new, lowtag_of(thing));
@@ -211,11 +202,11 @@ ptrans_unboxed(lispobj thing, lispobj header)
 {
     /* Allocate it */
     lispobj *old = native_pointer(thing);
-    long nwords = sizetab[widetag_of(header)](old);
+    long nwords = sizetab[header_widetag(header)](old);
     lispobj *new = newspace_alloc(nwords, 1);     /* always constant */
 
     /* copy it. */
-    bcopy(old, new, nwords * sizeof(lispobj));
+    memcpy(new, old, nwords * sizeof(lispobj));
 
     /* Deposit forwarding pointer. */
     lispobj result = make_lispobj(new, lowtag_of(thing));
@@ -228,10 +219,10 @@ static lispobj
 ptrans_vector(lispobj thing, boolean boxed, boolean constant)
 {
     struct vector *vector = VECTOR(thing);
-    long nwords = sizetab[widetag_of(vector->header)]((lispobj*)vector);
+    long nwords = sizetab[header_widetag(vector->header)]((lispobj*)vector);
 
     lispobj *new = newspace_alloc(nwords, (constant || !boxed));
-    bcopy(vector, new, nwords * sizeof(lispobj));
+    memcpy(new, vector, nwords * sizeof(lispobj));
 
     lispobj result = make_lispobj(new, lowtag_of(thing));
     vector->header = result;
@@ -246,41 +237,45 @@ static lispobj
 ptrans_code(lispobj thing)
 {
     struct code *code = (struct code *)native_pointer(thing);
-    long nwords = code_header_words(code->header)
-                + code_instruction_words(code->code_size);
+    long nwords = code_total_nwords(code);
 
     struct code *new = (struct code *)newspace_alloc(nwords,1); /* constant */
 
-    bcopy(code, new, nwords * sizeof(lispobj));
+    memcpy(new, code, nwords * sizeof(lispobj));
 
     lispobj result = make_lispobj(new, OTHER_POINTER_LOWTAG);
-
-    /* Put in forwarding pointers for all the functions. */
     uword_t displacement = result - thing;
+
+#if defined LISP_FEATURE_PPC || defined LISP_FEATURE_PPC64
+    // Fixup absolute jump tables. These aren't recorded in code->fixups
+    // because we don't need to denote an arbitrary set of places in the code.
+    // The count alone suffices. A GC immediately after creating the code
+    // could cause us to observe some 0 words here. Those should be ignored.
+    lispobj* jump_table = code_jumptable_start(new);
+    int count = jumptable_count(jump_table);
+    int i;
+    for (i = 1; i < count; ++i)
+        if (jump_table[i]) jump_table[i] += displacement;
+#endif
+    /* Put in forwarding pointers for all the functions. */
     for_each_simple_fun(i, newfunc, new, 1, {
-        lispobj* old = (lispobj*)LOW_WORD((char*)newfunc - displacement);
+        lispobj* old = (lispobj*)((char*)newfunc - displacement);
         *old = make_lispobj(newfunc, FUN_POINTER_LOWTAG);
+        pscav(&newfunc->self, 1, 1); // and fix the self-pointer now
     });
 
+    int n_funs = code_n_funs(code);
     /* Stick in a forwarding pointer for the code object. */
     /* This smashes the header, so do it only after reading n_funs */
     *(lispobj *)code = result;
 
-    /* Arrange to scavenge the debug info later. */
-    pscav_later(&new->debug_info, 1);
-
-    /* Scavenge the constants. */
-    pscav(new->constants,
-          code_header_words(new->header) - (offsetof(struct code, constants) >> WORD_SHIFT),
-          1);
-
-    /* Scavenge all the functions. */
-    for_each_simple_fun(i, func, new, 1, {
-        gc_assert(!dynamic_pointer_p((lispobj)func));
-        pscav(&func->self, 1, 1);
-        pscav_later(&func->name, 4);
-    })
-
+    int n_later_words = 1 + n_funs * CODE_SLOTS_PER_SIMPLE_FUN;
+    /* Scavenge the constants excluding the ones that will be done later. */
+    lispobj* from = &new->debug_info + n_later_words;
+    lispobj* end = (lispobj*)new + code_header_words(new);
+    pscav(from, end - from, 1);
+    /* Arrange to scavenge the debug info and simple-fun metadata later. */
+    pscav_later(&new->debug_info, n_later_words);
     return result;
 }
 
@@ -293,15 +288,14 @@ ptrans_func(lispobj thing, lispobj header)
      * Otherwise we have to do something strange, 'cause it is buried
      * inside a code object. */
 
-    if (widetag_of(header) == SIMPLE_FUN_WIDETAG) {
+    if (header_widetag(header) == SIMPLE_FUN_WIDETAG) {
 
         /* We can only end up here if the code object has not been
          * scavenged, because if it had been scavenged, forwarding pointers
          * would have been left behind for all the entry points. */
 
         struct simple_fun *function = (struct simple_fun *)native_pointer(thing);
-        lispobj code = make_lispobj(native_pointer(thing) - HeaderValue(function->header),
-                                    OTHER_POINTER_LOWTAG);
+        lispobj code = fun_code_tagged((lispobj*)function);
 
         /* This will cause the function's header to be replaced with a
          * forwarding pointer. */
@@ -313,16 +307,16 @@ ptrans_func(lispobj thing, lispobj header)
     } else {
         /* It's some kind of closure-like thing. */
         lispobj *old = native_pointer(thing);
-        long nwords = sizetab[widetag_of(header)](old);
+        long nwords = sizetab[header_widetag(header)](old);
 
         /* Allocate the new one.  FINs *must* not go in read_only
          * space.  Closures can; they never change */
 
         lispobj *new = newspace_alloc
-            (nwords,(widetag_of(header)!=FUNCALLABLE_INSTANCE_WIDETAG));
+            (nwords,(header_widetag(header)!=FUNCALLABLE_INSTANCE_WIDETAG));
 
         /* Copy it. */
-        bcopy(old, new, nwords * sizeof(lispobj));
+        memcpy(new, old, nwords * sizeof(lispobj));
 
         /* Deposit forwarding pointer. */
         lispobj result = make_lispobj(new, lowtag_of(thing));
@@ -350,7 +344,7 @@ ptrans_returnpc(lispobj thing, lispobj header)
     return new + (thing - code);
 }
 
-#define WORDS_PER_CONS CEILING(sizeof(struct cons) / sizeof(lispobj), 2)
+#define WORDS_PER_CONS ALIGN_UP(sizeof(struct cons) / sizeof(lispobj), 2)
 
 static lispobj
 ptrans_list(lispobj thing, boolean constant)
@@ -375,7 +369,7 @@ ptrans_list(lispobj thing, boolean constant)
 
         /* And count this cell. */
         length++;
-    } while (lowtag_of(thing) == LIST_POINTER_LOWTAG &&
+    } while (listp(thing) &&
              dynamic_pointer_p(thing) &&
              !(forwarding_pointer_p(*native_pointer(thing))));
 
@@ -388,7 +382,8 @@ ptrans_list(lispobj thing, boolean constant)
 static lispobj
 ptrans_otherptr(lispobj thing, lispobj header, boolean constant)
 {
-    switch (widetag_of(header)) {
+    int widetag = header_widetag(header);
+    switch (widetag) {
         /* FIXME: this needs a reindent */
       case BIGNUM_WIDETAG:
       case SINGLE_FLOAT_WIDETAG:
@@ -427,9 +422,6 @@ ptrans_otherptr(lispobj thing, lispobj header, boolean constant)
       case SYMBOL_WIDETAG:
         return ptrans_boxed(thing, header, 0);
 
-#include "genesis/specialized-vectors.inc"
-        return ptrans_vector(thing, 0, constant);
-
       case SIMPLE_VECTOR_WIDETAG:
         return ptrans_vector(thing, 1, constant);
 
@@ -443,7 +435,10 @@ ptrans_otherptr(lispobj thing, lispobj header, boolean constant)
         return ptrans_fdefn(thing, header);
 
       default:
-        fprintf(stderr, "Invalid widetag: %d\n", widetag_of(header));
+        if (other_immediate_lowtag_p(widetag) &&
+            specialized_vector_widetag_p(widetag))
+            return ptrans_vector(thing, 0, constant);
+        fprintf(stderr, "Invalid widetag: %d\n", header_widetag(header));
         /* Should only come across other pointers to the above stuff. */
         gc_abort();
         return NIL;
@@ -471,6 +466,7 @@ pscav(lispobj *addr, long nwords, boolean constant)
 
     while (nwords > 0) {
         thing = *addr;
+        int widetag = header_widetag(thing);
         if (is_lisp_pointer(thing)) {
             /* It's a pointer. Is it something we might have to move? */
             if (dynamic_pointer_p(thing)) {
@@ -509,14 +505,14 @@ pscav(lispobj *addr, long nwords, boolean constant)
             count = 1;
         }
 #if N_WORD_BITS == 64
-        else if (widetag_of(thing) == SINGLE_FLOAT_WIDETAG) {
+        else if (widetag == SINGLE_FLOAT_WIDETAG) {
             count = 1;
         }
 #endif
         else if (thing & FIXNUM_TAG_MASK) {
             /* It's an other immediate. Maybe the header for an unboxed */
             /* object. */
-            switch (widetag_of(thing)) {
+            switch (widetag) {
               case BIGNUM_WIDETAG:
               case SINGLE_FLOAT_WIDETAG:
               case DOUBLE_FLOAT_WIDETAG:
@@ -525,22 +521,19 @@ pscav(lispobj *addr, long nwords, boolean constant)
 #endif
               case SAP_WIDETAG:
                 /* It's an unboxed simple object. */
-                count = CEILING(HeaderValue(thing)+1, 2);
+                count = ALIGN_UP(HeaderValue(thing)+1, 2);
                 break;
 
               case SIMPLE_VECTOR_WIDETAG:
-                  if (HeaderValue(thing) == subtype_VectorValidHashing) {
-                    struct hash_table *hash_table =
-                        (struct hash_table *)native_pointer(addr[2]);
-                    hash_table->needs_rehash_p = T;
-                  }
+                // addr[0] : header
+                //     [1] : vector length
+                //     [2] : element[0] = high-water mark
+                //     [3] : element[1] = rehash bit
+                if (is_vector_subtype(thing, VectorAddrHashing))
+                    addr[3] = make_fixnum(1); // just flag it for rehash
                 count = 2;
                 break;
 
-#include "genesis/specialized-vectors.inc"
-                count = sizetab[widetag_of(thing)](addr);
-                break;
-
               case CODE_HEADER_WIDETAG:
                 gc_abort(); /* no code headers in static space */
                 break;
@@ -567,19 +560,18 @@ pscav(lispobj *addr, long nwords, boolean constant)
 
               case INSTANCE_WIDETAG:
                 {
-                    struct layout *layout
-                        = (struct layout *)native_pointer(instance_layout(addr));
+                    lispobj lbitmap = LAYOUT(instance_layout(addr))->bitmap;
                     lispobj* slots = addr + 1;
                     long nslots = instance_length(*addr) | 1;
                     int index;
-                    if (fixnump(layout->bitmap)) {
-                      sword_t bitmap = (sword_t)layout->bitmap >> N_FIXNUM_TAG_BITS;
+                    if (fixnump(lbitmap)) {
+                      sword_t bitmap = fixnum_value(lbitmap);
                       for (index = 0; index < nslots ; index++, bitmap >>= 1)
                         if (bitmap & 1)
                           pscav(slots + index, 1, constant);
                     } else {
                       struct bignum * bitmap;
-                      bitmap = (struct bignum*)native_pointer(layout->bitmap);
+                      bitmap = (struct bignum*)native_pointer(lbitmap);
                       for (index = 0; index < nslots ; index++)
                         if (positive_bignum_logbitp(index, bitmap))
                           pscav(slots + index, 1, constant);
@@ -589,7 +581,11 @@ pscav(lispobj *addr, long nwords, boolean constant)
                 break;
 
               default:
-                count = 1;
+                if (other_immediate_lowtag_p(widetag) &&
+                    specialized_vector_widetag_p(widetag))
+                    count = sizetab[header_widetag(thing)](addr);
+                else
+                    count = 1;
                 break;
             }
         }
@@ -605,12 +601,30 @@ pscav(lispobj *addr, long nwords, boolean constant)
     return addr;
 }
 
+static void
+verify_range(lispobj *base, lispobj *end)
+{
+    lispobj* where = base;
+    while (where < end) {
+        if (widetag_of(where) == CODE_HEADER_WIDETAG) {
+            int n_boxed_words = code_header_words((struct code*)where);
+            int i;
+            for (i = 0; i < n_boxed_words; ++i) {
+                lispobj word = where[i];
+                if (is_lisp_pointer(word) && dynamic_pointer_p(word))
+                    lose("purify failed, obj=%p, where=%p, val=%"OBJ_FMTX,
+                         where, where+i, word);
+            }
+        }
+        where += OBJECT_SIZE(*where, where);
+    }
+}
+
 int
 purify(lispobj static_roots, lispobj read_only_roots)
 {
     lispobj *clean;
     long count, i;
-    struct later *laters, *next;
     struct thread *thread;
 
     if(all_threads->next) {
@@ -636,12 +650,10 @@ purify(lispobj static_roots, lispobj read_only_roots)
         return 0;
     }
 
-    dynamic_space_purify_pointer = dynamic_space_free_pointer;
+    dynamic_space_purify_pointer = get_alloc_pointer();
 
-    read_only_end = read_only_free =
-        (lispobj *)SymbolValue(READ_ONLY_SPACE_FREE_POINTER,0);
-    static_end = static_free =
-        (lispobj *)SymbolValue(STATIC_SPACE_FREE_POINTER,0);
+    read_only_end = read_only_free = read_only_space_free_pointer;
+    static_end = static_free = static_space_free_pointer;
 
 #ifdef PRINTNOISE
     printf(" roots");
@@ -655,9 +667,7 @@ purify(lispobj static_roots, lispobj read_only_roots)
     printf(" handlers");
     fflush(stdout);
 #endif
-    pscav((lispobj *) interrupt_handlers,
-          sizeof(interrupt_handlers) / sizeof(lispobj),
-          0);
+    pscav(lisp_sig_handlers, NSIG, 0);
 
 #ifdef PRINTNOISE
     printf(" stack");
@@ -689,8 +699,7 @@ purify(lispobj static_roots, lispobj read_only_roots)
     if (SymbolValue(SCAVENGE_READ_ONLY_SPACE) != UNBOUND_MARKER_WIDETAG
         && SymbolValue(SCAVENGE_READ_ONLY_SPACE) != NIL) {
       unsigned  read_only_space_size =
-          (lispobj *)SymbolValue(READ_ONLY_SPACE_FREE_POINTER) -
-          (lispobj *)READ_ONLY_SPACE_START;
+          read_only_space_free_pointer - (lispobj *)READ_ONLY_SPACE_START;
       fprintf(stderr,
               "scavenging read only space: %d bytes\n",
               read_only_space_size * sizeof(lispobj));
@@ -706,22 +715,14 @@ purify(lispobj static_roots, lispobj read_only_roots)
     do {
         while (clean != static_free)
             clean = pscav(clean, static_free - clean, 0);
-        laters = later_blocks;
+        struct later* laters = later_blocks;
         count = later_count;
         later_blocks = NULL;
         later_count = 0;
         while (laters != NULL) {
-            for (i = 0; i < count; i++) {
-                if (laters->u[i].count == 0) {
-                    ;
-                } else if (laters->u[i].count <= LATERMAXCOUNT) {
-                    pscav(laters->u[i+1].ptr, laters->u[i].count, 1);
-                    i++;
-                } else {
-                    pscav(laters->u[i].ptr, 1, 1);
-                }
-            }
-            next = laters->next;
+            for (i = 0; i < count; i++)
+                pscav(laters->ptr[i], laters->count[i], 1);
+            struct later* next = laters->next;
             free(laters);
             laters = next;
             count = LATERBLOCKSIZE;
@@ -732,9 +733,6 @@ purify(lispobj static_roots, lispobj read_only_roots)
     printf(" cleanup");
     fflush(stdout);
 #endif
-#ifdef LISP_FEATURE_HPUX
-    clear_auto_gc_trigger(); /* restore mmap as it was given by os */
-#endif
 
     os_zero((os_vm_address_t) current_dynamic_space, dynamic_space_size);
 
@@ -746,10 +744,10 @@ purify(lispobj static_roots, lispobj read_only_roots)
 
     /* It helps to update the heap free pointers so that free_heap can
      * verify after it's done. */
-    SetSymbolValue(READ_ONLY_SPACE_FREE_POINTER, (lispobj)read_only_free,0);
-    SetSymbolValue(STATIC_SPACE_FREE_POINTER, (lispobj)static_free,0);
+    read_only_space_free_pointer = read_only_free;
+    static_space_free_pointer = static_free;
 
-    dynamic_space_free_pointer = current_dynamic_space;
+    set_alloc_pointer((lispobj)current_dynamic_space);
     set_auto_gc_trigger(bytes_consed_between_gcs);
 
     /* Blast away instruction cache */
@@ -757,6 +755,7 @@ purify(lispobj static_roots, lispobj read_only_roots)
     os_flush_icache((os_vm_address_t)STATIC_SPACE_START, STATIC_SPACE_SIZE);
 
 #ifdef PRINTNOISE
+    verify_range((lispobj*)READ_ONLY_SPACE_START, read_only_space_free_pointer);
     printf(" done]\n");
     fflush(stdout);
 #endif
@@ -764,7 +763,8 @@ purify(lispobj static_roots, lispobj read_only_roots)
 }
 #else /* LISP_FEATURE_GENCGC */
 int
-purify(lispobj static_roots, lispobj read_only_roots)
+purify(lispobj __attribute__((unused)) static_roots,
+       lispobj __attribute__((unused)) read_only_roots)
 {
     lose("purify called for GENCGC. This should not happen.");
 }
