@@ -83,6 +83,7 @@ struct scan_state {
     // A hashmap from object to list of objects pointing to it
     struct hopscotch_table* inverted_heap;
     struct scratchpad scratchpad;
+    lispobj ignored_objects;
     int keep_leaves;
 };
 
@@ -98,8 +99,8 @@ static const char* classify_obj(lispobj ptr)
     switch(lowtag_of(ptr)) {
     case INSTANCE_POINTER_LOWTAG:
         name = instance_classoid_name(native_pointer(ptr));
-        if (widetag_of(name) == SIMPLE_BASE_STRING_WIDETAG)
-          return (char*)(name + 2);
+        if (widetag_of(name) == SIMPLE_BASE_STRING_WIDETAG) return (char*)(name + 2);
+        break;
     case LIST_POINTER_LOWTAG:
         return "cons";
     case FUN_POINTER_LOWTAG:
@@ -700,6 +701,16 @@ static boolean record_ptr(lispobj* source, lispobj target,
     if (count_only) COUNT_POINTER(x) \
     else if (is_lisp_pointer(x) && relevant_ptr_p(x)) record_ptr(where,x,ss); }
 
+static boolean ignorep(lispobj* base_ptr,
+                       lispobj ignored_objects)
+{
+    int i;
+    for (i = fixnum_value(VECTOR(ignored_objects)->length)-1; i >= 0; --i)
+      if (native_pointer(VECTOR(ignored_objects)->data[i]) == base_ptr)
+          return 1;
+    return 0;
+}
+
 static uword_t build_refs(lispobj* where, lispobj* end,
                           struct scan_state* ss)
 {
@@ -710,6 +721,10 @@ static uword_t build_refs(lispobj* where, lispobj* end,
 
     boolean count_only = !ss->record_ptrs;
     for ( ; where < end ; where += nwords ) {
+        if (ss->ignored_objects && ignorep(where, ss->ignored_objects)) {
+            nwords = OBJECT_SIZE(*where, where);
+            continue;
+        }
         ++n_objects;
         lispobj word = *where;
         if (!is_header(word)) {
@@ -843,10 +858,12 @@ static void scan_spaces(struct scan_state* ss)
 #define HASH_FUNCTION HOPSCOTCH_HASH_FUN_MIX
 
 static void* compute_heap_inverse(boolean keep_leaves,
+                                  lispobj ignored_objects,
                                   struct scratchpad* scratchpad)
 {
     struct scan_state ss;
     memset(&ss, 0, sizeof ss);
+    ss.ignored_objects = ignored_objects;
     ss.keep_leaves = keep_leaves;
     if (heap_trace_verbose) fprintf(stderr, "Pass 1: Counting heap objects...\n");
     scan_spaces(&ss);
@@ -868,7 +885,7 @@ static void* compute_heap_inverse(boolean keep_leaves,
     // Add one pointer due to inability to use the first
     // two words of the scratchpad.
     uword_t scratchpad_min_size = (1 + ss.n_pointers) * 2 * sizeof (uint32_t);
-    int pagesize = getpagesize();
+    int pagesize = os_reported_page_size;
     uword_t scratchpad_size = ALIGN_UP(scratchpad_min_size, pagesize);
     ss.scratchpad.base = os_allocate(scratchpad_size);
     gc_assert(ss.scratchpad.base);
@@ -931,6 +948,7 @@ static boolean finding_leaf_p(lispobj weak_pointers)
 static int trace_paths(void (*context_scanner)(),
                        lispobj weak_pointers, // list of inputs
                        lispobj paths, // vector of outputs
+                       lispobj ignore, // vector of ignored objects
                        int n_pins, lispobj* pins,
                        int criterion)
 {
@@ -950,7 +968,7 @@ static int trace_paths(void (*context_scanner)(),
                   ((i%8)==7||i==n_pins-1)?"\n":"");
     }
     inverted_heap = compute_heap_inverse(finding_leaf_p(weak_pointers),
-                                         &scratchpad);
+                                         ignore, &scratchpad);
     hopscotch_create(&visited, HASH_FUNCTION, 0, 32, 0);
     hopscotch_create(&targets, HASH_FUNCTION, 0, 32, 0);
     i = 0;
@@ -993,9 +1011,10 @@ int gc_prove_liveness(void(*context_scanner)(),
                       int criterion)
 {
     int n_watched = 0, n_live = 0, n_bad = 0, n_imm = 0, n_paths = 0;
-    lispobj input = CONS(objects)->car,
-            output = CONS(objects)->cdr,
-            paths = CONS(output)->cdr;
+    lispobj input  = VECTOR(objects)->data[0],
+            ignore = VECTOR(objects)->data[1],
+            output = VECTOR(objects)->data[2],
+            paths  = CONS(output)->cdr;
     lispobj list;
     for (list = input ; list != NIL && listp(list) ; list = CONS(list)->cdr) {
         ++n_watched;
@@ -1040,7 +1059,7 @@ int gc_prove_liveness(void(*context_scanner)(),
             pins[i] = compute_lispobj((lispobj*)pins[i]);
         }
     }
-    n_paths = trace_paths(context_scanner, input, paths,
+    n_paths = trace_paths(context_scanner, input, paths, ignore,
                           n_pins, (lispobj*)pins, criterion);
     CONS(output)->car = make_fixnum(n_paths);
     return n_paths;
@@ -1055,20 +1074,3 @@ int prove_liveness(lispobj objects, int criterion)
     extern int gc_n_stack_pins;
     return gc_prove_liveness(0, objects, gc_n_stack_pins, pinned_objects.keys, criterion);
 }
-
-#ifdef LISP_FEATURE_SB_THREAD
-/// Given a pthread identifier, return the lisp thread's string name.
-/// Caveat emptor: This doesn't acquire the all_threads_lock.
-/// Ironically perhaps, 'traceroot' no longer uses this, but it's nice to have.
-struct vector* lisp_thread_name(os_thread_t os_thread)
-{
-    struct thread* th;
-    for_each_thread (th) {
-        if (pthread_equal(os_thread, th->os_thread)) {
-            struct thread_instance *lispthread = (void*)native_pointer(th->lisp_thread);
-            return VECTOR(lispthread->name);
-        }
-    }
-    return 0;
-}
-#endif
