@@ -60,11 +60,6 @@
 #include "thread.h"
 #include "align.h"
 
-#ifndef LISP_FEATURE_SB_THREAD
-/* dummy definition to reduce ifdef clutter */
-#define WITH_GC_AT_SAFEPOINTS_ONLY() if (0) ; else
-#endif
-
 #include "gc.h"
 #include "gencgc-internal.h"
 #include <wincrypt.h>
@@ -481,7 +476,7 @@ void unmap_gc_page()
  * Of course, this name-parsing trick lacks conceptual clarity; we're
  * going to get rid of it eventually. */
 
-u32 os_get_build_time_shared_libraries(u32 excl_maximum,
+uint32_t os_get_build_time_shared_libraries(uint32_t excl_maximum,
                                        void* opt_root,
                                        void** opt_store_handles,
                                        const char *opt_store_names[])
@@ -497,7 +492,7 @@ u32 os_get_build_time_shared_libraries(u32 excl_maximum,
 
     void* check_duplicates[excl_maximum];
 
-    if ((*(u32*)base_magic_location)!=0x4550) {
+    if ((*(uint32_t*)base_magic_location)!=0x4550) {
         /* We don't need this DLL thingie _that_ much. If the world
          * has changed to a degree where PE magic isn't found, let's
          * silently return `no libraries detected'. */
@@ -516,7 +511,7 @@ u32 os_get_build_time_shared_libraries(u32 excl_maximum,
             &image_optional_header->DataDirectory[IMAGE_DIRECTORY_ENTRY_IMPORT];
         IMAGE_IMPORT_DESCRIPTOR* image_import_descriptor =
             base + image_import_direntry->VirtualAddress;
-        u32 nlibrary, j;
+        uint32_t nlibrary, j;
 
         for (nlibrary=0u; nlibrary < excl_maximum
                           && image_import_descriptor->FirstThunk;
@@ -584,7 +579,7 @@ u32 os_get_build_time_shared_libraries(u32 excl_maximum,
     }
 }
 
-static u32 buildTimeImageCount = 0;
+static uint32_t buildTimeImageCount = 0;
 static void* buildTimeImages[16];
 
 /* Resolve symbols against the executable and its build-time dependencies */
@@ -645,8 +640,44 @@ void* os_dlsym_default(char* name)
    started before _any_ TLS slot is allocated by libraries, and
    some C compiler vendors rely on this fact. */
 
+extern CRITICAL_SECTION code_allocator_lock, alloc_profiler_lock;
+static CRITICAL_SECTION interrupt_io_lock;
+
+struct {
+    console_char buffer[MAX_CONSOLE_TCHARS];
+    DWORD head, tail;
+    CRITICAL_SECTION lock;
+    CONDITION_VARIABLE cond_has_data;
+    CONDITION_VARIABLE cond_has_client;
+    HANDLE thread;
+    boolean initialized;
+    HANDLE handle;
+    boolean in_progress;
+} ttyinput;
+
+#ifdef LISP_FEATURE_64_BIT
+// 32-bit uses a hardwired value (63) as this index because codegen
+// usese %FS:[a_constant] to get CURRENT-THREAD-SAP, etc.
+DWORD sbcl_thread_tls_index;
+#endif
+
 int os_preinit(char *argv[], char *envp[])
 {
+#ifdef LISP_FEATURE_SB_FUTEX
+    if (GetModuleHandle("API-MS-Win-Core-Synch-l1-2-0") == NULL) {
+        // calling lose() here loses. Such irony.
+        fprintf(stderr, "This binary was compiled for Windows 8 or later but you appear to be"
+" using 7 or earlier.\nRecompiling SBCL from source on the older release will"
+" probably work. See also\n"
+                "https://support.microsoft.com/en-us/help/4057281/windows-7-support-ended-on-january-14-2020\n");
+        fflush(stderr);
+        exit(1);
+    }
+#endif
+    InitializeCriticalSection(&code_allocator_lock);
+    InitializeCriticalSection(&alloc_profiler_lock);
+    InitializeCriticalSection(&interrupt_io_lock);
+    InitializeCriticalSection(&ttyinput.lock);
 #ifdef LISP_FEATURE_X86
     DWORD slots[TLS_MINIMUM_AVAILABLE];
     DWORD key;
@@ -672,61 +703,12 @@ int os_preinit(char *argv[], char *envp[])
              "(last TlsAlloc() returned %u)",key);
     }
 #else
-    (void) argv; /* unused */
-    (void) envp; /* unused */
+    OUR_TLS_INDEX = TlsAlloc();
 #endif
     return 0;
 }
 #endif  /* LISP_FEATURE_SB_THREAD */
 
-
-#ifdef LISP_FEATURE_X86_64
-/* Windows has 32-bit 'longs', so printf...%lX (and other %l patterns) doesn't
- * work well with address-sized values, like it's done all over the place in
- * SBCL. And msvcrt uses I64, not LL, for printing long longs.
- *
- * I've already had enough search/replace with longs/words/intptr_t for today,
- * so I prefer to solve this problem with a format string translator. */
-
-/* There is (will be) defines for printf and friends. */
-
-static int translating_vfprintf(FILE*stream, const char *fmt, va_list args)
-{
-    char translated[1024];
-    unsigned i=0, delta = 0;
-
-    while (fmt[i-delta] && i<sizeof(translated)-1) {
-        if((fmt[i-delta]=='%')&&
-           (fmt[i-delta+1]=='l')) {
-            translated[i++]='%';
-            translated[i++]='I';
-            translated[i++]='6';
-            translated[i++]='4';
-            delta += 2;
-        } else {
-            translated[i]=fmt[i-delta];
-            ++i;
-        }
-    }
-    translated[i++]=0;
-    return vfprintf(stream,translated,args);
-}
-
-int printf(const char*fmt,...)
-{
-    va_list args;
-    va_start(args,fmt);
-    return translating_vfprintf(stdout,fmt,args);
-}
-int fprintf(FILE*stream,const char*fmt,...)
-{
-    va_list args;
-    va_start(args,fmt);
-    return translating_vfprintf(stream,fmt,args);
-}
-
-#endif
-
 int os_number_of_processors = 1;
 
 BOOL (*ptr_CancelIoEx)(HANDLE /*handle*/, LPOVERLAPPED /*overlapped*/);
@@ -781,81 +763,16 @@ void os_init(char __attribute__((__unused__)) *argv[],
     runtime_module_handle = (HMODULE)win32_get_module_handle_by_address(&runtime_module_handle);
 }
 
-static inline boolean local_thread_stack_address_p(os_vm_address_t address)
-{
-    return this_thread &&
-        (((((u64)address >= (u64)this_thread->os_address) &&
-           ((u64)address < ((u64)this_thread)-THREAD_CSP_PAGE_SIZE))||
-          (((u64)address >= (u64)this_thread->control_stack_start)&&
-           ((u64)address < (u64)this_thread->control_stack_end))));
-}
-
-/*
- * So we have three fun scenarios here.
- *
- * First, we could be being called to reserve the memory areas
- * during initialization (prior to loading the core file).
- *
- * Second, we could be being called by the GC to commit a page
- * that has just been decommitted (for easy zero-fill).
- *
- * Third, we could be being called by create_thread_struct()
- * in order to create the sundry and various stacks.
- *
- * The third case is easy to pick out because it passes an
- * addr of 0.
- *
- * The second case is easy to pick out because it will be for
- * a range of memory that is MEM_RESERVE rather than MEM_FREE.
- *
- * The second case is also an easy implement, because we leave
- * the memory as reserved (since we do lazy commits).
- */
-
 os_vm_address_t
 os_validate(int attributes, os_vm_address_t addr, os_vm_size_t len)
 {
-    MEMORY_BASIC_INFORMATION mem_info;
-
     if (!addr) {
-        /* the simple case first */
         int protection = attributes & IS_GUARD_PAGE ? PAGE_NOACCESS : PAGE_EXECUTE_READWRITE;
         return
             AVERLAX(VirtualAlloc(addr, len, MEM_RESERVE|MEM_COMMIT, protection));
     }
 
-    if (!AVERLAX(VirtualQuery(addr, &mem_info, sizeof mem_info)))
-        return 0;
-
-    if ((mem_info.State == MEM_RESERVE) && (mem_info.RegionSize >= len)) {
-        /* It would be correct to return here. However, support for Wine
-         * is beneficial, and Wine has a strange behavior in this
-         * department. It reports all memory below KERNEL32.DLL as
-         * reserved, but disallows MEM_COMMIT.
-         *
-         * Let's work around it: reserve the region we need for a second
-         * time. The second reservation is documented to fail on normal NT
-         * family, but it will succeed on Wine if this region is
-         * actually free.
-         */
-        VirtualAlloc(addr, len, MEM_RESERVE, PAGE_EXECUTE_READWRITE);
-        /* If it is wine, the second call has succeeded, and now the region
-         * is really reserved. */
-        return addr;
-    }
-
-    DWORD mode;
-    if (mem_info.State == MEM_RESERVE) {
-        fprintf(stderr, "validation of reserved space too short.\n");
-        fflush(stderr);
-        /* Oddly, we do not treat this assertion as fatal; hence also the
-         * provision for MEM_RESERVE in the following code, I suppose: */
-        mode = MEM_COMMIT;
-    } else {
-        mode = MEM_RESERVE;
-    }
-
-    os_vm_address_t actual = VirtualAlloc(addr, len, mode, PAGE_EXECUTE_READWRITE);
+    os_vm_address_t actual = VirtualAlloc(addr, len, MEM_RESERVE|MEM_COMMIT, PAGE_EXECUTE_READWRITE);
 
     if (!actual) {
         if (!(attributes & MOVABLE)) {
@@ -866,83 +783,50 @@ os_validate(int attributes, os_vm_address_t addr, os_vm_size_t len)
             return 0;
         }
 
-        return AVERLAX(VirtualAlloc(NULL, len, mode, PAGE_EXECUTE_READWRITE));
+        return AVERLAX(VirtualAlloc(NULL, len, MEM_RESERVE|MEM_COMMIT, PAGE_EXECUTE_READWRITE));
     }
 
     return actual;
 }
 
-/*
- * For os_invalidate(), we merely decommit the memory rather than
- * freeing the address space. This loses when freeing per-thread
- * data and related memory since it leaks address space.
- *
- * So far the original comment (author unknown).  It used to continue as
- * follows:
- *
- *   It's not too lossy, however, since the two scenarios I'm aware of
- *   are fd-stream buffers, which are pooled rather than torched, and
- *   thread information, which I hope to pool (since windows creates
- *   threads at its own whim, and we probably want to be able to have
- *   them callback without funky magic on the part of the user, and
- *   full-on thread allocation is fairly heavyweight).
- *
- * But: As it turns out, we are no longer content with decommitting
- * without freeing, and have now grown a second function
- * os_invalidate_free(), sort of a really_os_invalidate().
- *
- * As discussed on #lisp, this is not a satisfactory solution, and probably
- * ought to be rectified in the following way:
- *
- *  - Any cases currently going through the non-freeing version of
- *    os_invalidate() are ultimately meant for zero-filling applications.
- *    Replace those use cases with an os_revalidate_bzero() or similarly
- *    named function, which explicitly takes care of that aspect of
- *    the semantics.
- *
- *  - The remaining uses of os_invalidate should actually free, and once
- *    the above is implemented, we can rename os_invalidate_free back to
- *    just os_invalidate().
- *
- * So far the new plan, as yet unimplemented. -- DFL
- */
+/* Used to allocate the dynamic space, as it may be very large. Dynamically comitted by os_commit_memory in handle_access_violation */
+os_vm_address_t
+os_validate_nocommit(int attributes, os_vm_address_t addr, os_vm_size_t len)
+{
+    os_vm_address_t actual = VirtualAlloc(addr, len, MEM_RESERVE, PAGE_EXECUTE_READWRITE);
+
+    if (!actual) {
+        if (!(attributes & MOVABLE)) {
+            fprintf(stderr,
+                    "VirtualAlloc: wanted %lu bytes at %p, actually mapped at %p\n",
+                    (unsigned long) len, addr, actual);
+            fflush(stderr);
+            return 0;
+        }
+        return AVERLAX(VirtualAlloc(NULL, len, MEM_RESERVE, PAGE_EXECUTE_READWRITE));
+    }
+
+    return actual;
+}
+
+void* os_commit_memory(os_vm_address_t addr, os_vm_size_t len)
+{
+    return
+        AVERLAX(VirtualAlloc(addr, len, MEM_COMMIT, PAGE_EXECUTE_READWRITE));
+}
+
+
+void os_revalidate_bzero(os_vm_address_t addr,  os_vm_size_t len) {
+    AVERLAX(VirtualFree(addr, len, MEM_DECOMMIT));
+    AVERLAX(VirtualAlloc(addr, len, MEM_COMMIT, PAGE_EXECUTE_READWRITE));
+}
 
 void
 os_invalidate(os_vm_address_t addr, os_vm_size_t len)
-{
-    AVERLAX(VirtualFree(addr, len, MEM_DECOMMIT));
-}
-
-void
-os_invalidate_free(os_vm_address_t addr,
-                   os_vm_size_t __attribute__((__unused__)) len)
 {
     AVERLAX(VirtualFree(addr, 0, MEM_RELEASE));
 }
 
-void
-os_invalidate_free_by_any_address(os_vm_address_t addr,
-                                  os_vm_size_t __attribute__((__unused__)) len)
-{
-    MEMORY_BASIC_INFORMATION minfo;
-    AVERLAX(VirtualQuery(addr, &minfo, sizeof minfo));
-    AVERLAX(minfo.AllocationBase);
-    AVERLAX(VirtualFree(minfo.AllocationBase, 0, MEM_RELEASE));
-}
-
-/* os_validate doesn't commit, i.e. doesn't actually "validate" in the
- * sense that we could start using the space afterwards.  Usually it's
- * load_core_bytes or Lisp code that will run into that, in which case we recommit
- * elsewhere in this file.  For cases where C wants to write into newly
- * os_validate()d memory, it needs to commit it explicitly first:
- */
-os_vm_address_t
-os_validate_recommit(os_vm_address_t addr, os_vm_size_t len)
-{
-    return
-        AVERLAX(VirtualAlloc(addr, len, MEM_COMMIT, PAGE_EXECUTE_READWRITE));
-}
-
 /*
  * load_core_bytes() is called to load a chunk of the core file into memory.
  *
@@ -954,18 +838,23 @@ os_validate_recommit(os_vm_address_t addr, os_vm_size_t len)
  * thing to maintain).
  */
 
-void* load_core_bytes(int fd, int offset, os_vm_address_t addr, os_vm_size_t len)
+void* load_core_bytes(int fd, os_vm_offset_t offset, os_vm_address_t addr, os_vm_size_t len)
 {
-    os_vm_size_t count;
-
-    AVER(VirtualAlloc(addr, len, MEM_COMMIT, PAGE_EXECUTE_READWRITE)||
-         VirtualAlloc(addr, len, MEM_RESERVE|MEM_COMMIT,
-                      PAGE_EXECUTE_READWRITE));
-
+    os_commit_memory(addr, len);
+#ifdef LISP_FEATURE_64_BIT
+    CRT_AVER_NONNEGATIVE(_lseeki64(fd, offset, SEEK_SET));
+#else
     CRT_AVER_NONNEGATIVE(lseek(fd, offset, SEEK_SET));
+#endif
+    size_t count;
 
-    count = read(fd, addr, len);
-    CRT_AVER( count == len );
+    while (len) {
+        unsigned to_read = len > INT_MAX ? INT_MAX : len;
+        count = read(fd, addr, to_read);
+        addr += count;
+        len -= count;
+        CRT_AVER(count == to_read);
+    }
     return (void*)0;
 }
 static DWORD os_protect_modes[8] = {
@@ -993,75 +882,6 @@ os_protect(os_vm_address_t address, os_vm_size_t length, os_vm_prot_t prot)
              address,length,prot,new_prot,old_prot);
 }
 
-/* FIXME: Now that FOO_END, rather than FOO_SIZE, is the fundamental
- * description of a space, we could probably punt this and just do
- * (FOO_START <= x && x < FOO_END) everywhere it's called. */
-static boolean
-in_range_p(lispobj a, lispobj sbeg, size_t slen)
-{
-    char* beg = (char*)((uword_t)sbeg);
-    char* end = (char*)((uword_t)sbeg) + slen;
-    char* adr = (char*)a;
-    return (adr >= beg && adr < end);
-}
-
-boolean
-is_linkage_table_addr(os_vm_address_t addr)
-{
-    return in_range_p((lispobj)addr, LINKAGE_TABLE_SPACE_START, LINKAGE_TABLE_SPACE_SIZE);
-}
-
-static boolean is_some_thread_local_addr(os_vm_address_t addr);
-
-boolean
-gc_managed_addr_p(lispobj addr)
-{
-    if(gc_managed_heap_space_p(addr) ||
-       is_some_thread_local_addr((os_vm_address_t)addr))
-        return 1;
-    return 0;
-}
-
-/* test if an address is within thread-local space */
-static boolean
-is_thread_local_addr(struct thread* th, os_vm_address_t addr)
-{
-    /* Assuming that this is correct, it would warrant further comment,
-     * I think.  Based on what our call site is doing, we have been
-     * tasked to check for the address of a lisp object; not merely any
-     * foreign address within the thread's area.  Indeed, this used to
-     * be a check for control and binding stack only, rather than the
-     * full thread "struct".  So shouldn't the THREAD_STRUCT_SIZE rather
-     * be (thread_control_stack_size+BINDING_STACK_SIZE) instead?  That
-     * would also do away with the LISP_FEATURE_SB_THREAD case.  Or does
-     * it simply not matter?  --DFL */
-    ptrdiff_t diff = ((char*)th->os_address)-(char*)addr;
-    return diff > (ptrdiff_t)0 && diff < (ptrdiff_t)THREAD_STRUCT_SIZE
-#ifdef LISP_FEATURE_SB_THREAD
-        && addr != (os_vm_address_t) th->csp_around_foreign_call
-#endif
-        ;
-}
-
-static boolean
-is_some_thread_local_addr(os_vm_address_t addr)
-{
-    boolean result = 0;
-#ifdef LISP_FEATURE_SB_THREAD
-    struct thread *th;
-    pthread_mutex_lock(&all_threads_lock);
-    for_each_thread(th) {
-        if(is_thread_local_addr(th,addr)) {
-            result = 1;
-            break;
-        }
-    }
-    pthread_mutex_unlock(&all_threads_lock);
-#endif
-    return result;
-}
-
-
 /* A tiny bit of interrupt.c state we want our paws on. */
 extern boolean internal_errors_enabled;
 
@@ -1196,14 +1016,6 @@ handle_access_violation(os_context_t *ctx,
              exception_record->ExceptionInformation[0]);
 #endif
 
-    /* Stack: This case takes care of our various stack exhaustion
-     * protect pages (with the notable exception of the control stack!). */
-    if (self && local_thread_stack_address_p(fault_address)) {
-        if (handle_guard_page_triggered(ctx, fault_address))
-            return 0; /* gc safety? */
-        goto try_recommit;
-    }
-
     /* Safepoint pages */
 #ifdef LISP_FEATURE_SB_THREAD
     if (fault_address == (void *) GC_SAFEPOINT_TRAP_ADDR) {
@@ -1211,72 +1023,38 @@ handle_access_violation(os_context_t *ctx,
         return 0;
     }
 
-    if ((((u64)fault_address) == ((u64)self->csp_around_foreign_call))){
+    if (1+(lispobj*)fault_address == (lispobj*)self) {
         thread_in_safety_transition(ctx);
         return 0;
     }
 #endif
 
     /* dynamic space */
-    page_index_t index = find_page_index(fault_address);
-    if (index != -1) {
-        /*
-         * Now, if the page is supposedly write-protected and this
-         * is a write, tell the gc that it's been hit.
-         */
-        if (page_table[index].write_protected) {
-            gencgc_handle_wp_violation(fault_address);
-        } else {
-            AVER(VirtualAlloc(PTR_ALIGN_DOWN(fault_address,os_vm_page_size),
-                              os_vm_page_size,
-                              MEM_COMMIT, PAGE_EXECUTE_READWRITE));
-        }
-        return 0;
-    } else {
-#ifdef LISP_FEATURE_IMMOBILE_SPACE
-        extern int immobile_space_handle_wp_violation(void*);
-        if (immobile_space_handle_wp_violation(fault_address)) {
+    if (DYNAMIC_SPACE_START <= (lispobj)fault_address &&
+        (lispobj)fault_address < (DYNAMIC_SPACE_START + dynamic_space_size)) {
+        MEMORY_BASIC_INFORMATION mem_info;
+        if (AVERLAX(VirtualQuery(fault_address, &mem_info, sizeof mem_info)) &&
+            mem_info.State == MEM_RESERVE) {
+            os_commit_memory(PTR_ALIGN_DOWN(fault_address, os_vm_page_size),
+                             os_vm_page_size);
             return 0;
         }
-#endif
+    }
+    if (gencgc_handle_wp_violation(fault_address)) {
+        return 0;
     }
 
-    if (fault_address == undefined_alien_address)
-        return -1;
-
-    /* linkage table or a "valid_lisp_addr" outside of dynamic space (?) */
-    if (is_linkage_table_addr(fault_address)
-        || gc_managed_addr_p((lispobj)fault_address))
-        goto try_recommit;
+#ifdef LISP_FEATURE_IMMOBILE_SPACE
+    extern int immobile_space_handle_wp_violation(void*);
+    if (immobile_space_handle_wp_violation(fault_address)) {
+        return 0;
+    }
+#endif
+    if (handle_guard_page_triggered(ctx, fault_address)) {
+        return 0;
+    }
 
     return -1;
-
-try_recommit:
-    /* First use of a new page, lets get some memory for it. */
-
-#if defined(LISP_FEATURE_X86)
-    AVER(VirtualAlloc(PTR_ALIGN_DOWN(fault_address,os_vm_page_size),
-                      os_vm_page_size,
-                      MEM_COMMIT, PAGE_EXECUTE_READWRITE)
-         ||(fprintf(stderr,"Unable to recommit addr %p eip 0x%08lx\n",
-                    fault_address, win32_context->Eip) &&
-            (c_level_backtrace("BT",5),
-             fake_foreign_function_call(ctx),
-             lose("Lispy backtrace"),
-             0)));
-#else
-    AVER(VirtualAlloc(PTR_ALIGN_DOWN(fault_address,os_vm_page_size),
-                      os_vm_page_size,
-                      MEM_COMMIT, PAGE_EXECUTE_READWRITE)
-         ||(fprintf(stderr,"Unable to recommit addr %p eip %p\n",
-                    fault_address, (void*)win32_context->Rip) &&
-            (c_level_backtrace("BT",5),
-             fake_foreign_function_call(ctx),
-             lose("Lispy backtrace"),
-             0)));
-#endif
-
-    return 0;
 }
 
 static void
@@ -1393,14 +1171,12 @@ handle_exception(EXCEPTION_RECORD *exception_record,
 
     os_context_t context, *ctx = &context;
     context.win32_context = win32_context;
-#if defined(LISP_FEATURE_SB_THREAD)
-    context.sigmask = self ? self->os_thread->blocked_signal_set : 0;
-#endif
+    context.sigmask = self ? thread_extra_data(self)->blocked_signal_set : 0;
 
     os_context_register_t oldbp = 0;
     if (self) {
-        oldbp = self ? self->carried_base_pointer : 0;
-        self->carried_base_pointer
+        oldbp = self ? thread_extra_data(self)->carried_base_pointer : 0;
+        thread_extra_data(self)->carried_base_pointer
             = (os_context_register_t) voidreg(win32_context, bp);
     }
 
@@ -1447,7 +1223,7 @@ handle_exception(EXCEPTION_RECORD *exception_record,
         signal_internal_error_or_lose(ctx, exception_record, fault_address);
 
     if (self)
-        self->carried_base_pointer = oldbp;
+        thread_extra_data(self)->carried_base_pointer = oldbp;
 
     errno = lastErrno;
     SetLastError(lastError);
@@ -1466,7 +1242,7 @@ veh(EXCEPTION_POINTERS *ep)
     EXCEPTION_DISPOSITION disp;
 
     RESTORING_ERRNO() {
-        if (!pthread_self())
+        if (!arch_os_get_current_thread())
             return EXCEPTION_CONTINUE_SEARCH;
     }
 
@@ -1489,7 +1265,7 @@ os_context_register_t
 carry_frame_pointer(os_context_register_t default_value)
 {
     struct thread* self = arch_os_get_current_thread();
-    os_context_register_t bp = self->carried_base_pointer;
+    os_context_register_t bp = thread_extra_data(self)->carried_base_pointer;
     return bp ? bp : default_value;
 }
 
@@ -1558,19 +1334,32 @@ char *dirname(char *path)
 int
 socket_input_available(HANDLE socket)
 {
-    unsigned long count = 0, count_size = 0;
+    int count = 0;
     int wsaErrno = GetLastError();
-    int err = WSAIoctl((SOCKET)socket, FIONREAD, NULL, 0,
-                       &count, sizeof(count), &count_size, NULL, NULL);
+    TIMEVAL timeout = {0, 0};
+    fd_set readfds, errfds;
 
-    int ret;
+    FD_ZERO(&readfds);
+    FD_ZERO(&errfds);
+    FD_SET(socket, &readfds);
+    FD_SET(socket, &errfds);
 
-    if (err == 0) {
-        ret = (count > 0) ? 1 : 2;
-    } else
-        ret = 0;
+    count = select(0, &readfds, NULL, &errfds, &timeout);
     SetLastError(wsaErrno);
-    return ret;
+
+    if (count == SOCKET_ERROR)
+        /* This is an error in select() itself, not with the socket. */
+        return 0;
+    else if (count == 0)
+        return 2;
+    /* True if there is data, or the socket has been closed (i.e. a read
+     * would retrieve an EOF error) */
+    else if (FD_ISSET(socket, &readfds))
+        return 1;
+    else
+        /* This is an error in the socket. The count was positive, and
+         * it wasn't in the read set, so it must be in the error set. */
+        return 0;
 }
 
 #ifdef LISP_FEATURE_SB_THREAD
@@ -1586,17 +1375,15 @@ boolean io_begin_interruptible(HANDLE handle)
     if (!ptr_CancelIoEx)
         return 1;
 
-    if (!__sync_bool_compare_and_swap(&this_thread->synchronous_io_handle_and_flag,
+    if (!__sync_bool_compare_and_swap(&thread_extra_data(this_thread)->synchronous_io_handle_and_flag,
                                       0, handle)) {
-        ResetEvent(this_thread->private_events.events[0]);
-        this_thread->synchronous_io_handle_and_flag = 0;
+        ResetEvent(thread_private_events(this_thread,0));
+        thread_extra_data(this_thread)->synchronous_io_handle_and_flag = 0;
         return 0;
     }
     return 1;
 }
 
-static pthread_mutex_t interrupt_io_lock = PTHREAD_MUTEX_INITIALIZER;
-
 /* Unmark current thread as (probably) doing synchronous I/O; if an
  * I/O cancellation was requested, postpone it until next
  * io_begin_interruptible */
@@ -1605,10 +1392,10 @@ io_end_interruptible(HANDLE handle)
 {
     if (!ptr_CancelIoEx)
         return;
-    pthread_mutex_lock(&interrupt_io_lock);
-    __sync_bool_compare_and_swap(&this_thread->synchronous_io_handle_and_flag,
+    thread_mutex_lock(&interrupt_io_lock);
+    __sync_bool_compare_and_swap(&thread_extra_data(this_thread)->synchronous_io_handle_and_flag,
                                  handle, 0);
-    pthread_mutex_unlock(&interrupt_io_lock);
+    thread_mutex_unlock(&interrupt_io_lock);
 }
 #define WITH_INTERRUPTIBLE_IO(handle)      \
     if (!io_begin_interruptible(handle)) { \
@@ -1686,30 +1473,17 @@ int console_handle_p(HANDLE handle)
  * entered.
  */
 
-struct {
-    console_char buffer[MAX_CONSOLE_TCHARS];
-    DWORD head, tail;
-    pthread_mutex_t lock;
-    pthread_cond_t cond_has_data;
-    pthread_cond_t cond_has_client;
-    pthread_t thread;
-    boolean initialized;
-    HANDLE handle;
-    boolean in_progress;
-} ttyinput = {.lock = PTHREAD_MUTEX_INITIALIZER};
-
-static void*
-tty_read_line_server()
+static __stdcall unsigned int tty_read_line_server(LPVOID arg)
 {
-    pthread_mutex_lock(&ttyinput.lock);
+    thread_mutex_lock(&ttyinput.lock);
     while (ttyinput.handle) {
         DWORD nchars;
         BOOL ok;
 
         while (!ttyinput.in_progress)
-            pthread_cond_wait(&ttyinput.cond_has_client,&ttyinput.lock);
+          SleepConditionVariableCS(&ttyinput.cond_has_client,&ttyinput.lock,INFINITE);
 
-        pthread_mutex_unlock(&ttyinput.lock);
+        thread_mutex_unlock(&ttyinput.lock);
 #ifdef LISP_FEATURE_SB_UNICODE
         ok = ReadConsoleW(ttyinput.handle,
                           &ttyinput.buffer[ttyinput.tail],
@@ -1722,16 +1496,16 @@ tty_read_line_server()
                          &nchars,NULL);
 #endif
 
-        pthread_mutex_lock(&ttyinput.lock);
+        thread_mutex_lock(&ttyinput.lock);
 
         if (ok) {
             ttyinput.tail += nchars;
-            pthread_cond_broadcast(&ttyinput.cond_has_data);
+            WakeAllConditionVariable(&ttyinput.cond_has_data);
         }
         ttyinput.in_progress = 0;
     }
-    pthread_mutex_unlock(&ttyinput.lock);
-    return NULL;
+    thread_mutex_unlock(&ttyinput.lock);
+    return 0;
 }
 
 static boolean
@@ -1743,9 +1517,13 @@ tty_maybe_initialize_unlocked(HANDLE handle)
                              0,FALSE,DUPLICATE_SAME_ACCESS)) {
             return 0;
         }
-        pthread_cond_init(&ttyinput.cond_has_data,NULL);
-        pthread_cond_init(&ttyinput.cond_has_client,NULL);
-        pthread_create(&ttyinput.thread,NULL,tty_read_line_server,NULL);
+        InitializeConditionVariable(&ttyinput.cond_has_data);
+        InitializeConditionVariable(&ttyinput.cond_has_client);
+        ttyinput.thread =
+          (HANDLE)_beginthreadex(NULL,
+                                 0, // stack size = default
+                                 tty_read_line_server, 0,
+                                 0, 0);
         ttyinput.initialized = 1;
     }
     return 1;
@@ -1757,7 +1535,7 @@ win32_tty_listen(HANDLE handle)
     boolean result = 0;
     INPUT_RECORD ir;
     DWORD nevents;
-    pthread_mutex_lock(&ttyinput.lock);
+    thread_mutex_lock(&ttyinput.lock);
     if (!tty_maybe_initialize_unlocked(handle))
         result = 0;
 
@@ -1769,11 +1547,11 @@ win32_tty_listen(HANDLE handle)
         } else {
             if (PeekConsoleInput(ttyinput.handle,&ir,1,&nevents) && nevents) {
                 ttyinput.in_progress = 1;
-                pthread_cond_broadcast(&ttyinput.cond_has_client);
+                WakeAllConditionVariable(&ttyinput.cond_has_client);
             }
         }
     }
-    pthread_mutex_unlock(&ttyinput.lock);
+    thread_mutex_unlock(&ttyinput.lock);
     return result;
 }
 
@@ -1789,7 +1567,7 @@ static int win32_read_console(HANDLE handle, void* buf, int count)
 
     count = nchars*sizeof(console_char);
 
-    pthread_mutex_lock(&ttyinput.lock);
+    thread_mutex_lock(&ttyinput.lock);
 
     if (!tty_maybe_initialize_unlocked(handle)) {
         result = -1;
@@ -1809,9 +1587,9 @@ static int win32_read_console(HANDLE handle, void* buf, int count)
                     /* We are to wait */
                     ttyinput.in_progress=1;
                     /* wake console reader */
-                    pthread_cond_broadcast(&ttyinput.cond_has_client);
+                    WakeAllConditionVariable(&ttyinput.cond_has_client);
                 }
-                pthread_cond_wait(&ttyinput.cond_has_data, &ttyinput.lock);
+                SleepConditionVariableCS(&ttyinput.cond_has_data, &ttyinput.lock, INFINITE);
                 io_end_interruptible(ttyinput.handle);
             }
         }
@@ -1846,7 +1624,7 @@ static int win32_read_console(HANDLE handle, void* buf, int count)
         }
     }
 unlock:
-    pthread_mutex_unlock(&ttyinput.lock);
+    thread_mutex_unlock(&ttyinput.lock);
     return result;
 }
 
@@ -1856,28 +1634,33 @@ win32_maybe_interrupt_io(void* thread)
     struct thread *th = thread;
     boolean done = 0;
 
+#ifdef LISP_FEATURE_SB_FUTEX
+    if (thread_extra_data(th)->waiting_on_address)
+        WakeByAddressAll(thread_extra_data(th)->waiting_on_address);
+#endif
+
     if (ptr_CancelIoEx) {
-        pthread_mutex_lock(&interrupt_io_lock);
+        thread_mutex_lock(&interrupt_io_lock);
         HANDLE h = (HANDLE)
             InterlockedExchangePointer((volatile LPVOID *)
-                                       &th->synchronous_io_handle_and_flag,
+                                       &thread_extra_data(th)->synchronous_io_handle_and_flag,
                                        (LPVOID)INVALID_HANDLE_VALUE);
 
         if (h && (h!=INVALID_HANDLE_VALUE)) {
             if (console_handle_p(h)) {
-                pthread_mutex_lock(&ttyinput.lock);
-                pthread_cond_broadcast(&ttyinput.cond_has_data);
-                pthread_mutex_unlock(&ttyinput.lock);
+                thread_mutex_lock(&ttyinput.lock);
+                WakeAllConditionVariable(&ttyinput.cond_has_data);
+                thread_mutex_unlock(&ttyinput.lock);
                 done = 1;
                 goto unlock;
             }
             if (ptr_CancelSynchronousIo) {
-                done = !!ptr_CancelSynchronousIo(th->os_thread->handle);
+                done = !!ptr_CancelSynchronousIo((HANDLE)th->os_thread);
             }
             done |= !!ptr_CancelIoEx(h,NULL);
         }
     unlock:
-        pthread_mutex_unlock(&interrupt_io_lock);
+        thread_mutex_unlock(&interrupt_io_lock);
     }
     return done;
 }
@@ -1934,7 +1717,7 @@ win32_unix_write(HANDLE handle, void * buf, int count)
         return win32_write_console(handle,buf,count);
     }
 
-    overlapped.hEvent = self->private_events.events[0];
+    overlapped.hEvent = thread_private_events(self,0);
     seekable = SetFilePointerEx(handle,
                                 zero_large_offset,
                                 &file_position,
@@ -1965,7 +1748,7 @@ win32_unix_write(HANDLE handle, void * buf, int count)
             errno = errorCode;
             return -1;
         } else {
-            if(WaitForMultipleObjects(2,self->private_events.events,
+            if(WaitForMultipleObjects(2,thread_extra_data(self)->private_events,
                                       FALSE,INFINITE) != WAIT_OBJECT_0) {
                 CancelIo(handle);
                 waitInGOR = TRUE;
@@ -2009,7 +1792,7 @@ win32_unix_read(HANDLE handle, void * buf, int count)
         return win32_read_console(handle, buf, count);
     }
 
-    overlapped.hEvent = self->private_events.events[0];
+    overlapped.hEvent = thread_private_events(self,0);
     /* If it has a position, we won't try overlapped */
     seekable = SetFilePointerEx(handle,
                                 zero_large_offset,
@@ -2051,7 +1834,7 @@ win32_unix_read(HANDLE handle, void * buf, int count)
             return -1;
         } else {
             int ret;
-            if( (ret = WaitForMultipleObjects(2,self->private_events.events,
+            if( (ret = WaitForMultipleObjects(2,thread_extra_data(self)->private_events,
                                               FALSE,INFINITE)) != WAIT_OBJECT_0) {
                 CancelIo(handle);
                 waitInGOR = TRUE;
@@ -2129,7 +1912,7 @@ win32_wait_object_or_signal(HANDLE waitFor)
 {
 #ifdef LISP_FEATURE_SB_THREAD
     struct thread *self = arch_os_get_current_thread();
-    HANDLE handles[] = {waitFor, self->private_events.events[1]};
+    HANDLE handles[] = {waitFor, thread_private_events(self,1)};
     return
         WaitForMultipleObjects(2,handles, FALSE, INFINITE);
 #else
@@ -2142,7 +1925,7 @@ win32_wait_for_multiple_objects_or_signal(HANDLE *handles, DWORD count)
 {
 #ifdef LISP_FEATURE_SB_THREAD
     struct thread *self = arch_os_get_current_thread();
-    handles[count] = self->private_events.events[1];
+    handles[count] = thread_private_events(self,1);
     return
         WaitForMultipleObjects(count + 1, handles, FALSE, INFINITE);
 #else
@@ -2196,4 +1979,121 @@ os_cancel_wtimer(HANDLE handle)
 }
 #endif
 
+#ifdef LISP_FEATURE_SB_FUTEX
+int
+futex_wait(int *lock_word, int oldval, long sec, unsigned long usec)
+{
+  DWORD timeout = sec < 0 ? INFINITE : (sec * 1000) + (usec / 1000);
+  struct thread* th = arch_os_get_current_thread();
+  thread_extra_data(th)->waiting_on_address = lock_word;
+  int result = WaitOnAddress(lock_word, &oldval, 4, timeout);
+  thread_extra_data(th)->waiting_on_address = 0;
+  if (result)
+      return 0;
+  if (GetLastError() == ERROR_TIMEOUT)
+      return 1;
+  return 0;
+}
+
+int
+futex_wake(PVOID lock_word, int n)
+{
+    if (n == 1)
+        WakeByAddressSingle(lock_word);
+    else
+        WakeByAddressAll(lock_word);
+    return 0;
+}
+#endif
+
+int _sbcl_pthread_sigmask(int how, const sigset_t *set, sigset_t *oldset)
+{
+  struct extra_thread_data* self = thread_extra_data(arch_os_get_current_thread());
+  if (oldset)
+    *oldset = self->blocked_signal_set;
+  if (set) {
+    switch (how) {
+      case SIG_BLOCK:
+        self->blocked_signal_set |= *set;
+        break;
+      case SIG_UNBLOCK:
+        self->blocked_signal_set &= ~(*set);
+        break;
+      case SIG_SETMASK:
+        self->blocked_signal_set = *set;
+        break;
+    }
+  }
+  return 0;
+}
+
+int sb_pthr_kill(struct thread* thread, int signum)
+{
+    __sync_fetch_and_or(&thread_extra_data(thread)->pending_signal_set, 1<<signum);
+    return 0;
+}
+
+int sigpending(sigset_t *set)
+{
+    struct extra_thread_data* data = thread_extra_data(arch_os_get_current_thread());
+    *set = InterlockedCompareExchange((volatile LONG*)&data->pending_signal_set,
+                                      0, 0);
+    return 0;
+}
+
+/* Signals */
+struct sigaction signal_handlers[NSIG];
+
+/* Never called for now */
+int sigaction(int signum, const struct sigaction* act, struct sigaction* oldact)
+{
+  struct sigaction newact = *act;
+  if (oldact)
+    *oldact = signal_handlers[signum];
+  if (!(newact.sa_flags & SA_SIGINFO)) {
+      newact.sa_sigaction = (typeof(newact.sa_sigaction))newact.sa_handler;
+  }
+  signal_handlers[signum] = newact;
+  return 0;
+}
+
+int sched_yield()
+{
+  /* http://stackoverflow.com/questions/1383943/switchtothread-vs-sleep1
+     SwitchToThread(); was here. Unsure what's better for us, just trying.. */
+
+  if(!SwitchToThread())
+      Sleep(0);
+  return 0;
+}
+
+int sigemptyset(sigset_t *set)
+{
+  *set = 0;
+  return 0;
+}
+
+int sigfillset(sigset_t *set)
+{
+  *set = 0xfffffffful;
+  return 0;
+}
+
+int sigaddset(sigset_t *set, int signum)
+{
+  *set |= 1 << signum;
+  return 0;
+}
+
+int sigdelset(sigset_t *set, int signum)
+{
+  *set &= ~(1 << signum);
+  return 0;
+}
+
+int sigismember(const sigset_t *set, int signum)
+{
+  return (*set & (1 << signum)) != 0;
+}
+
 /* EOF */
