@@ -40,7 +40,6 @@
 #define UD2_INST 0x0b0f
 #define BREAKPOINT_WIDTH 1
 
-unsigned int cpuid_fn1_ecx;
 int avx_supported = 0, avx2_supported = 0;
 
 static void cpuid(unsigned info, unsigned subinfo,
@@ -77,6 +76,7 @@ static void xgetbv(unsigned *eax, unsigned *edx)
 void tune_asm_routines_for_microarch(void)
 {
     unsigned int eax, ebx, ecx, edx;
+    unsigned int cpuid_fn1_ecx = 0;
 
     cpuid(0, 0, &eax, &ebx, &ecx, &edx);
     if (eax >= 1) { // see if we can execute basic id function 1
@@ -94,7 +94,13 @@ void tune_asm_routines_for_microarch(void)
             }
         }
     }
-    SetSymbolValue(CPUID_FN1_ECX, (lispobj)make_fixnum(cpuid_fn1_ecx), 0);
+    int our_cpu_feature_bits = 0;
+    // avx2_supported gets copied into bit 1 of *CPU-FEATURE-BITS*
+    if (avx2_supported) our_cpu_feature_bits |= 1;
+    // POPCNT = ECX bit 23, which gets copied into bit 2 in *CPU-FEATURE-BITS*
+    if (cpuid_fn1_ecx & (1<<23)) our_cpu_feature_bits |= 2;
+    SetSymbolValue(CPU_FEATURE_BITS, make_fixnum(our_cpu_feature_bits), 0);
+
     // I don't know if this works on Windows
 #ifndef _MSC_VER
     cpuid(0, 0, &eax, &ebx, &ecx, &edx);
@@ -113,6 +119,7 @@ void tune_asm_routines_for_microarch(void)
 void untune_asm_routines_for_microarch(void)
 {
     asm_routine_poke(VECTOR_FILL_T, 0x12, 0xEB); // Change JL to JMP
+    SetSymbolValue(CPU_FEATURE_BITS, 0, 0);
 }
 
 #ifndef _WIN64
@@ -478,12 +485,12 @@ arch_install_interrupt_handlers()
      * CL way, I hope there will at least be a comment to explain
      * why.. -- WHN 2001-06-07 */
 #if !defined(LISP_FEATURE_MACH_EXCEPTION_HANDLER) && !defined(LISP_FEATURE_WIN32)
-    undoably_install_low_level_interrupt_handler(SIGILL , sigill_handler);
-    undoably_install_low_level_interrupt_handler(SIGTRAP, sigtrap_handler);
+    ll_install_handler(SIGILL , sigill_handler);
+    ll_install_handler(SIGTRAP, sigtrap_handler);
 #endif
 
 #if defined(X86_64_SIGFPE_FIXUP) && !defined(LISP_FEATURE_WIN32)
-    undoably_install_low_level_interrupt_handler(SIGFPE, sigfpe_handler);
+    ll_install_handler(SIGFPE, sigfpe_handler);
 #endif
 
     SHOW("returning from arch_install_interrupt_handlers()");
@@ -604,8 +611,12 @@ lispobj fdefn_callee_lispobj(struct fdefn* fdefn) {
 extern unsigned int alloc_profile_n_counters;
 extern unsigned int max_alloc_point_counters;
 #ifdef LISP_FEATURE_SB_THREAD
+#ifdef LISP_FEATURE_WIN32
+extern CRITICAL_SECTION alloc_profiler_lock;
+#else
 extern pthread_mutex_t alloc_profiler_lock;
 #endif
+#endif
 
 static unsigned int claim_index(int qty)
 {
@@ -694,9 +705,10 @@ allocation_tracker_counted(uword_t* sp)
         unsigned int index = claim_index(1);
         if (index == 0)
             index = 2; // reserved overflow counter for fixed-size alloc
+        uword_t disp = index * 8;
         // rewrite call into: LOCK INC QWORD PTR, [R11+n] ; opcode = 0xFF / 0
         uword_t new_inst =
-          0xF0 | (0x49 << 8) | (0xFF << 16) | (0x83L << 24) | ((index*8L) << 32);
+          0xF0 | (0x49 << 8) | (0xFF << 16) | (0x83L << 24) | (disp << 32);
         // Ensure atomicity of the write. A plain store would probably do,
         // but since this is self-modifying code, the most stringent memory
         // order is prudent.
@@ -728,14 +740,15 @@ allocation_tracker_sized(uword_t* sp)
         }
         // rewrite call into:
         //  LOCK INC QWORD PTR, [R11+n] ; opcode = 0xFF / 0
+        uword_t disp = index * 8;
         uword_t new_inst1 =
-          0xF0 | (0x49 << 8) | (0xFF << 16) | (0x83L << 24) | ((index * 8L) << 32);
+          0xF0 | (0x49 << 8) | (0xFF << 16) | (0x83L << 24) | (disp << 32);
         //  LOCK ADD [R11+n], Rxx ; opcode = 0x01
         prefix = 0x49 | ((prefix & 1) << 2); // 'b' bit becomes 'r' bit
         modrm  = 0x83 | (modrm & (7<<3)); // copy 'reg' into new modrm byte
+        disp = (1 + index) * 8;
         uword_t new_inst2 =
-          0xF0 | (prefix << 8) | (0x01 << 16) | ((long)modrm << 24)
-            | (((1 + index) * 8L) << 32);
+          0xF0 | (prefix << 8) | (0x01 << 16) | ((long)modrm << 24) | (disp << 32);
         // Overwrite the second instruction first, because as soon as the CALL
         // opcode is changed, fallthrough to the next instruction occurs.
         if (!__sync_bool_compare_and_swap(pc+1, word_after_pc, new_inst2) ||
