@@ -20,7 +20,7 @@
   (let ((segment (sb-assem:make-segment)))
     (sb-assem:assemble (segment 'nil)
         (apply #'sb-assem:inst* (car inst) (cdr inst)))
-    (let* ((buf (sb-assem::segment-buffer segment))
+    (let* ((buf (sb-assem:segment-buffer segment))
            (string
              (with-output-to-string (stream)
                (with-pinned-objects (buf)
