@@ -448,3 +448,41 @@
       (assert (typep (second c4) 'sb-kernel:simple-character-string)))
     (assert (or (equal l1 l2)
                 (equal l1 (reverse l2))))))
+
+(defun cons-on-list-p (cons list)
+  (assert (consp cons))
+  (loop for cdr on list thereis (eq cons cdr)))
+
+(defun constant-folding-cdr-test-fun ()
+  (let ((list '(a b c)))
+    (dolist (x (list list (cdr list) (cddr list)))
+      (assert (cons-on-list-p x list)))))
+
+(with-test (:name (:cdr-eq-detection :lp1583753))
+  (constant-folding-cdr-test-fun))
+
+(locally
+    (declare (optimize (debug 2) (safety 3)))
+  (defconstant +consy-constant+ (if (boundp '+consy-constant+) (symbol-value '+consy-constant+) '(message-id CAOrNaszM=eSufoqA4KFVNq4CUpjSJym-ktQnufQgj7a5g2sHmg@mail.gmail.com)))
+  (defun test-constant-identity (x)
+    (list '(message-id CAOrNaszM=eSufoqA4KFVNq4CUpjSJym-ktQnufQgj7a5g2sHmg@mail.gmail.com)
+          (eq x +consy-constant+))))
+
+(with-test (:name (defconstant :reference :identity))
+  (let ((z (eval `(test-constant-identity ,(intern "+CONSY-CONSTANT+")))))
+    (assert (equal (car z) +consy-constant+))
+    (assert (cadr z))))
+
+(macrolet ((expand ()
+             `(progn
+                ,@(loop for i from 0 below sb-vm:n-word-bits
+                        collect
+                        `(sb-int:defconstant-eqx ,(intern (format nil "MYSAP~d" i))
+                             ,(sb-sys:int-sap (ash 1 i))
+                           #'sb-sys:sap=)))))
+  (expand))
+(with-test (:name :dump-sap)
+  (dotimes (i sb-vm:n-word-bits)
+    (let ((s (intern (format nil "MYSAP~d" i))))
+      (assert (= (sb-sys:sap-int (symbol-value s))
+                 (ash 1 i))))))
