@@ -139,6 +139,31 @@
    (let ((thread (sb-thread:make-thread (lambda () (sb-ext:gc)))))
      (loop while (sb-thread:thread-alive-p thread)))))
 
+(defglobal *some-object-handles* nil)
+(defun make-some-objects ()
+  (declare (notinline format))
+  (let* ((string-one (format nil "~a~a~a" "pot" "ayt" "o"))
+         (string-two (concatenate 'string "two " string-one))
+         (afunction
+          (let (#+immobile-space (sb-c::*compile-to-memory-space* :dynamic))
+            (compile nil `(sb-int:named-lambda ,string-two (x) (coerce x 'float))))))
+    (setq *some-object-handles*
+          (list (sb-kernel:get-lisp-obj-address afunction)
+                (sb-kernel:get-lisp-obj-address string-one)
+                (sb-kernel:get-lisp-obj-address string-two)))))
+#+gencgc
+(with-test (:name :pin-all-code-with-gc-enabled)
+  #+sb-thread (sb-thread:join-thread (sb-thread:make-thread #'make-some-objects))
+  #-sb-thread (progn (make-some-objects) (sb-sys:scrub-control-stack))
+  (sb-sys:with-code-pages-pinned (:dynamic) (gc))
+  ;; this should not fail to find FUN at its old address
+  (let ((fun (sb-kernel:make-lisp-obj (first *some-object-handles*))))
+    ;; this should fail to find a string at its old address
+    (assert (not (nth-value 1 (sb-kernel:make-lisp-obj (second *some-object-handles*) nil))))
+    ;; this should similarly fail- STRING-TWO was transitively reachable but movable
+    (assert (not (nth-value 1 (sb-kernel:make-lisp-obj (third *some-object-handles*) nil))))
+    (assert (string= (sb-kernel:%simple-fun-name fun) "two potayto"))))
+
 (with-test (:name :without-gcing)
   (let ((gc-happend nil))
     (push (lambda () (setq gc-happend t)) sb-ext:*after-gc-hooks*)
@@ -435,3 +460,38 @@
     (sb-sys:with-pinned-objects (rest)
       (sb-ext:gc :full t)
       (assert (and (equal list rest) t)))))
+
+(defun use-up-thread-region ()
+  ;; cons until the thread-local allocation buffer uses up a page
+  (loop
+   (let* ((c (cons 1 2))
+          (end (+ (sb-kernel:get-lisp-obj-address c)
+                  (- sb-vm:list-pointer-lowtag)
+                  (* 2 sb-vm:n-word-bytes))))
+     (when (zerop (logand end (1- sb-vm:gencgc-card-bytes)))
+       (return)))))
+(defglobal *go* nil)
+
+#+sb-thread
+(with-test (:name :c-call-save-p)
+  (let* ((fun (compile nil '(lambda (a b c d e f g h i j k l m)
+                             (declare (optimize (sb-c::alien-funcall-saves-fp-and-pc 0)))
+                             (setq *go* t)
+                             #+win32
+                             (alien-funcall (extern-alien "Sleep" (function void int))  300)
+                             #-win32
+                             (alien-funcall (extern-alien "sb_nanosleep" (function void int int)) 0 300000000)
+                             (values a b c d e f g h i j k l m))))
+         (thr (sb-thread:make-thread (lambda ()
+                                       (let ((args #1=(list (LIST 'A) (LIST 'B) (LIST 'C)
+                                                            (LIST 'D) (LIST 'E) (LIST 'F) (LIST 'G)
+                                                            (LIST 'H) (LIST 'I) (LIST 'J) (LIST 'K)
+                                                            (LIST 'L) (LIST 'M))))
+                                         (use-up-thread-region)
+                                         (apply fun
+                                                args))))))
+    (loop (sb-thread:barrier (:read))
+          (if *go* (return))
+          (sleep .1))
+    (gc)
+    (assert (equal (multiple-value-list (sb-thread:join-thread thr)) #1#))))
