@@ -5,6 +5,20 @@ echo ==== Writing logs to $logdir ====
 junkdir=${SBCL_PAREXEC_TMP:-/tmp}/junk
 mkdir -p $junkdir $logdir
 
+case `uname` in
+    CYGWIN* | WindowsNT | MINGW* | MSYS*)
+        if [ $# -ne 1 ]
+        then
+            echo $0: Need arg
+            exit 1
+        fi
+        echo ";; Using -j$1"
+        echo "LOGDIR=$logdir" >$logdir/Makefile
+        ../run-sbcl.sh --script genmakefile.lisp >>$logdir/Makefile
+        exec $GNUMAKE -k -j $1 -f $logdir/Makefile
+        ;;
+esac
+
 export TEST_DIRECTORY SBCL_HOME
 TEST_DIRECTORY=$junkdir SBCL_HOME=../obj/sbcl-home exec ../src/runtime/sbcl \
   --noinform --core ../output/sbcl.core \
