@@ -64,27 +64,19 @@
     (loop until returning)
     (assert (= saved-errno (sb-unix::get-errno)))))
 
-#+(or sb-safepoint sb-thruption)
-(sb-alien:define-alien-routine "kill_safely"
-  int
-  (sb-thread::os-thread unsigned)
-  (signal int))
-(defun current-thread-os-thread ()
-  #+sb-thread (sb-thread::thread-os-thread sb-thread:*current-thread*)
-  #-sb-thread 0)
-
 ;; It is desirable to support C-c on Windows, but SIGINT
 ;; is not the mechanism to use on this platform.
+;; This test used to call kill_safely() in the C runtime if using safepoints,
+;; and perhaps at some point kill_safely() interacted with the safepoint state
+;; for POSIX (i.e. not win32), but it doesn't, at least not now.
+;; The special case in kill_safely() for the current thread is pthread_kill()
+;; and not a thing more, unless on win32, which skips this test.
 #-win32
 (with-test (:name :handle-interactive-interrupt)
   (assert (eq :condition
               (handler-case
                   (progn
-                    ;; KILL-SAFELY has C code for it only if (OR :SB-SAFEPOINT :SB-THRUPTION),
-                    ;; otherwise we directly call a POSIX function.
-                    #+(or sb-safepoint sb-thruption) (kill-safely (current-thread-os-thread)
-                                                                  sb-unix:sigint)
-                    #-(or sb-safepoint sb-thruption) (sb-thread::raise sb-unix:sigint)
+                    (sb-thread::raise sb-unix:sigint)
                     #+sb-safepoint-strictly
                     ;; In this case, the signals handler gets invoked
                     ;; indirectly through an INTERRUPT-THREAD.  Give it
@@ -108,3 +100,16 @@
              (sb-ext:with-timeout 0.1 (sleep 1) t))))
      (sb-ext:timeout ()
        nil))))
+
+#+unix
+(with-test (:name :ignore-sigpipe)
+  (multiple-value-bind (read-side write-side) (sb-unix:unix-pipe)
+    (sb-unix:unix-close read-side)
+    (sb-sys:enable-interrupt sb-unix:sigpipe :ignore)
+    (let ((buffer "x"))
+      (sb-sys:with-pinned-objects (buffer)
+        (multiple-value-bind (nbytes errno)
+            (sb-unix:unix-write write-side buffer 0 1)
+          (assert (and (null nbytes)
+                       (= errno sb-unix:epipe))))))
+    (sb-unix:unix-close write-side)))
