@@ -23,6 +23,26 @@
             #P"/tmp/")))
 (require :sb-posix)
 
+;;; Believe it or not the x86-64-specific trap routine for UPDATE-OBJECT-LAYOUT-OR-INVALID
+;;; could fail to return the correct layout after calling from assembly code into lisp,
+;;; back to assembly code, back to the vop, and not a single regression test failed.
+(defclass astream (fundamental-output-stream) ())
+(defvar *str* (make-instance 'astream))
+(assert (streamp *str*))
+(defclass astream (fundamental-output-stream) (x y))
+(with-test (:name :update-stream-layout)
+  (assert (sb-kernel:layout-invalid (sb-kernel:%instance-layout *str*)))
+  (assert (streamp *str*))
+  (assert (/= 0 (sb-kernel:layout-clos-hash (sb-kernel:%instance-layout *str*))))
+  (defclass astream () (x y))
+  (assert (sb-kernel:layout-invalid (sb-kernel:%instance-layout *str*)))
+  (assert (= 0 (sb-kernel:layout-clos-hash (sb-kernel:%instance-layout *str*))))
+  (assert (not (streamp *str*)))
+  (assert (/= 0 (sb-kernel:layout-clos-hash (sb-kernel:%instance-layout *str*))))
+  (defclass astream (fundamental-output-stream) (x y))
+  (assert (sb-kernel:layout-invalid (sb-kernel:%instance-layout *str*)))
+  (assert (streamp *str*)))
+
 ;;; type errors for inappropriate stream arguments, fixed in
 ;;; sbcl-0.7.8.19
 (with-test (:name (make-two-way-stream type-error))
@@ -78,7 +98,7 @@
 
 ;;; improper buffering on (SIGNED-BYTE 8) streams (fixed by David Lichteblau):
 (with-test (:name (write-byte (unsigned-byte 8) read-byte (signed-byte 8)))
-  (let ((p "signed-byte-8-test.data"))
+  (let ((p (scratch-file-name)))
     (with-open-file (s p
                        :direction :output
                        :element-type '(unsigned-byte 8)
@@ -91,7 +111,7 @@
 ;;; :IF-EXISTS got :ERROR and NIL the wrong way round (reported by
 ;;; Milan Zamazal)
 (with-test (:name (open :if-exists :error))
-  (let* ((p "this-file-will-exist")
+  (let* ((p (scratch-file-name))
          (stream (open p :direction :output :if-exists :error)))
     (assert (null (with-open-file (s p :direction :output :if-exists nil) s)))
     (assert-error
@@ -111,7 +131,7 @@
 
 ;;; bidirectional streams getting confused about their position
 (with-test (:name (:direction :io))
-  (let ((p "bidirectional-stream-test"))
+  (let ((p (scratch-file-name)))
     (with-open-file (s p :direction :output :if-exists :supersede)
       (with-standard-io-syntax
         (format s "~S ~S ~S~%" 'these 'are 'symbols)))
@@ -127,7 +147,7 @@
 
 ;;; :DIRECTION :IO didn't work on non-existent pathnames
 (with-test (:name (with-open-file :direction :io :non-existent-pathname))
-  (let ((p "direction-io-test"))
+  (let ((p (scratch-file-name)))
     (ignore-errors (delete-file p))
     (with-open-file (s p :direction :io)
       (format s "1")
@@ -148,7 +168,7 @@
     (assert (= 0 (file-position s)))))
 
 (with-test (:name (file-position broadcast-stream 2))
-  (let ((p "broadcast-stream-test"))
+  (let ((p (scratch-file-name)))
     (ignore-errors (delete-file p))
     (with-open-file (f p :direction :output)
       (let ((s (make-broadcast-stream f)))
@@ -163,7 +183,7 @@
 ;;; CLOSING a non-new streams should not delete them, and superseded
 ;;; files should be restored.
 (with-test (:name :test-file-for-close-should-not-delete :fails-on :win32)
-  (let ((test "test-file-for-close-should-not-delete"))
+  (let ((test (scratch-file-name)))
     (macrolet ((test-mode (mode)
                           `(progn
                              (catch :close-test-exit
@@ -188,7 +208,8 @@
 ;;; test for read-write invariance of signed bytes, from Bruno Haible
 ;;; cmucl-imp 2004-09-06
 (defun bin-stream-test (&key (size (integer-length most-positive-fixnum))
-                        (type 'unsigned-byte) (file-name "stream-impure.tmp")
+                             (type 'unsigned-byte)
+                             (file-name (scratch-file-name))
                         (num-bytes 10)
                         (bytes (if (eq type 'signed-byte)
                                    (loop :repeat num-bytes :collect
@@ -231,7 +252,7 @@
 ;;; READ-N-BYTES, ANSI-STREAM-READ-BYTE, ANSI-STREAM-READ-BYTE.
 
 (with-test (:name (read-sequence type-error))
-  (let ((pathname "read-sequence.data"))
+  (let ((pathname (scratch-file-name)))
 
     ;; Create the binary data.
     (with-open-file (stream pathname
@@ -351,7 +372,7 @@
 ;;; (trace sb-impl::output-unsigned-byte-full-buffered sb-impl::output-signed-byte-full-buffered sb-impl::output-raw-bytes)
 
 (with-test (:name (write-sequence type-error))
-  (let ((pathname "write-sequence.data")
+  (let ((pathname (scratch-file-name))
         (generic-sequence (make-array 1 :initial-contents '(255)))
         (generic-character-sequence (make-array 1 :initial-element #\a))
         (generic-mixed-sequence (make-array 2 :initial-element #\a))
@@ -459,7 +480,7 @@
 ;;; to test on 64 bit platforms
 (with-test (:name (:write-char :long-lines :stream-ouput-column)
             :skipped-on :64-bit)
-  (let ((test "long-lines-write-test.tmp"))
+  (let ((test (scratch-file-name)))
     (unwind-protect
          (with-open-file (f test
                             :direction :output
@@ -485,16 +506,17 @@
 ;;; read-sequence misreported the amount read and lost position
 (with-test (:name (read-sequence :read-elements))
   (let ((string (make-array (* 3 sb-impl::+ansi-stream-in-buffer-length+)
-                            :element-type 'character)))
+                            :element-type 'character))
+        (file (scratch-file-name)))
     (dotimes (i (length string))
       (setf (char string i) (code-char (mod i char-code-limit))))
-    (with-open-file (f "read-sequence-character-test-data.tmp"
+    (with-open-file (f file
                        :if-exists :supersede
                        :direction :output
                        :external-format :utf-8)
       (write-sequence string f))
     (let ((copy
-            (with-open-file (f "read-sequence-character-test-data.tmp"
+            (with-open-file (f file
                                :if-does-not-exist :error
                                :direction :input
                                :external-format :utf-8)
@@ -506,7 +528,7 @@
                            (assert (<= (incf total n-read) (length string)))
                         while (and (= n-read 128))))))))
       (assert (equal copy string)))
-    (delete-file "read-sequence-character-test-data.tmp")))
+    (delete-file file)))
 
 ;;; ANSI-STREAM-OUTPUT-STREAM-P used to assume that a SYNONYM-STREAM's
 ;;; target was an ANSI stream, but it could be a user-defined stream,
@@ -532,7 +554,7 @@
 ;;; READ-LINE on ANSI-STREAM did not return T for the last line
 ;;; (reported by Yoshinori Tahara)
 (with-test (:name (read-line :last-line))
-  (let ((pathname "test-read-line-eol"))
+  (let ((pathname (scratch-file-name)))
     (with-open-file (out pathname :direction :output :if-exists :supersede)
       (format out "a~%b"))
     (let ((result (with-open-file (in pathname)
@@ -569,7 +591,7 @@
 ;;; READ-SEQUENCE into a byte buffer would lose when attempting to
 ;;; store the character in the vector.
 (with-test (:name (:bivalent stream unread-char read-byte))
-  (let ((pathname "bivalent-stream-unread-char-test.tmp"))
+  (let ((pathname (scratch-file-name)))
     (with-open-file (s pathname
                        :element-type :default
                        :direction :io :if-exists :rename)
@@ -580,7 +602,7 @@
     (delete-file pathname)))
 
 (with-test (:name (:bivalent stream unread-char read-sequence))
-  (let ((pathname "bivalent-stream-unread-char-test.tmp"))
+  (let ((pathname (scratch-file-name)))
     (with-open-file (s pathname
                        :element-type :default
                        :direction :io :if-exists :rename)
@@ -593,7 +615,7 @@
 
 (with-test (:name (:bivalent stream unread-char read-byte :utf8)
             :skipped-on (:not :sb-unicode))
-  (let ((pathname "bivalent-stream-unread-char-test.tmp"))
+  (let ((pathname (scratch-file-name)))
     (with-open-file (s pathname
                        :element-type :default
                        :direction :io :if-exists :rename
@@ -606,7 +628,7 @@
 
 (with-test (:name (:bivalent stream unread-char read-sequence :utf8)
             :skipped-on (:not :sb-unicode))
-  (let ((pathname "bivalent-stream-unread-char-test.tmp"))
+  (let ((pathname (scratch-file-name)))
     (with-open-file (s pathname
                        :element-type :default
                        :direction :io :if-exists :rename
@@ -619,7 +641,7 @@
     (delete-file pathname)))
 
 (with-test (:name (delete-file :on stream))
-  (with-open-file (f "delete-file-on-stream-test.tmp"
+  (with-open-file (f (scratch-file-name)
                      :direction :io)
     (delete-file f)
     #-win32
@@ -648,7 +670,8 @@
       (assert (< (- (get-universal-time) time) 2)))))
 
 #-win32
-(with-test (:name (open :interrupt) :skipped-on :win32)
+(with-test (:name (open :interrupt)
+                  :skipped-on (or :win32 (:and :darwin :sb-safepoint)))
   (let ((to 0))
     (with-scratch-file (fifo)
            ;; Make a FIFO
@@ -678,11 +701,12 @@
 (with-test (:name :overeager-character-buffering :skipped-on :win32)
   (let ((use-threads #+sb-thread t)
         (proc nil))
-    (sb-int:dohash ((format _) sb-impl::*external-formats*)
-      (declare (ignore _))
+    (sb-int:dovector (entry sb-impl::*external-formats*)
+      (unless entry (return))
       (with-scratch-file (fifo)
         (unwind-protect
-            (progn
+            (let ((format
+                    (car (sb-impl::ef-names (car (sb-int:ensure-list entry))))))
               (sb-posix:mkfifo fifo (logior sb-posix:s-iwusr sb-posix:s-irusr))
               ;; KLUDGE: because we have both ends in the same process, we would
               ;; need to use O_NONBLOCK, but this works too.
@@ -724,7 +748,7 @@
 
 (with-test (:name :bug-657183 :skipped-on (not :sb-unicode))
   #+sb-unicode
-  (let ((name (merge-pathnames "stream-impure.temp-test"))
+  (let ((name (scratch-file-name))
         (text '(#\GREEK_SMALL_LETTER_LAMDA
                 #\JAPANESE_BANK_SYMBOL
                 #\Space
@@ -755,7 +779,7 @@
       (ignore-errors (delete-file name)))))
 
 (with-test (:name :bug-561642)
-  (let ((p "bug-561642-test.tmp"))
+  (let ((p (scratch-file-name)))
     (unwind-protect
          (progn
            (with-open-file (f p
@@ -830,4 +854,12 @@
            (sb-impl::ansi-stream-read-string-from-frc-buffer string s 0 nil)))
       (assert (= endpos 0)))))
 
+(with-test (:name :named-pipe-wait-eof)
+  (let* ((process (run-program "cat" '() :search t
+                               :wait nil :input nil :output :stream))
+         (out (process-output process)))
+    (sb-sys:wait-until-fd-usable (sb-sys:fd-stream-fd out) :input)
+    (assert (null (read-byte (process-output process) nil nil)))
+    (process-close process)))
+
 ;;; success
