@@ -48,23 +48,34 @@
    (assert (not (thread-alive-p thread)))))
 
 ;;; Condition-wait should not be interruptible under WITHOUT-INTERRUPTS
+;;; BUT: Such a claim is without much merit. Even if a wait is not "interrupted",
+;;; the very definition of spurious wakeup is that return from the wait happens
+;;; for ANY reason - users of condition variables must ALWAYS anticipate needing
+;;; to loop over a condition-wait.
 
 (with-test (:name :without-interrupts+condition-wait
             :skipped-on (not :sb-thread)
             :broken-on :win32)
   (let* ((lock (make-mutex))
          (queue (make-waitqueue))
+         (actually-wakeup nil)
          (thread (make-thread (lambda ()
                                 (sb-sys:without-interrupts
                                   (with-mutex (lock)
-                                    (condition-wait queue lock)))))))
-    (sleep 1)
+                                    (loop
+                                     (condition-wait queue lock)
+                                     (if actually-wakeup (return)))))))))
+    (sleep .25)
     (assert (thread-alive-p thread))
+    ;; this is the supposed "interrupt that doesn't interrupt",
+    ;; but it _is_ permitted to wake the condition variable.
     (terminate-thread thread)
-    (sleep 1)
+    (sleep .5)
     (assert (thread-alive-p thread))
+    (setq actually-wakeup t)
+    (sb-thread:barrier (:write))
     (condition-notify queue)
-    (sleep 1)
+    (sleep .25)
     (assert (not (thread-alive-p thread)))))
 
 ;;; GRAB-MUTEX should not be interruptible under WITHOUT-INTERRUPTS
@@ -288,99 +299,6 @@
                        (sb-thread::symbol-value-in-thread-error-info e)))))
     (assert error-occurred)))
 
-(with-test (:name :deadlock-detection.1 :skipped-on (not :sb-thread))
-  (loop
-    repeat 1000
-    do (flet ((test (ma mb sa sb)
-                (lambda ()
-                  (handler-case
-                      (with-mutex (ma)
-                        (signal-semaphore sa)
-                        (wait-on-semaphore sb)
-                        (with-mutex (mb)
-                          :ok))
-                    (thread-deadlock (e)
-                      ;; (assert (plusp (length ...))) prevents
-                      ;; flushing.
-                      (assert (plusp (length (princ-to-string e))))
-                      :deadlock)))))
-         (let* ((m1 (make-mutex :name "M1"))
-                (m2 (make-mutex :name "M2"))
-                (s1 (make-semaphore :name "S1"))
-                (s2 (make-semaphore :name "S2"))
-                (t1 (make-thread (test m1 m2 s1 s2) :name "T1"))
-                (t2 (make-thread (test m2 m1 s2 s1) :name "T2")))
-           ;; One will deadlock, and the other will then complete normally.
-           (let ((res (list (join-thread t1)
-                            (join-thread t2))))
-             (assert (or (equal '(:deadlock :ok) res)
-                         (equal '(:ok :deadlock) res))))))))
-
-(with-test (:name :deadlock-detection.2 :skipped-on (not :sb-thread))
-  (let* ((m1 (make-mutex :name "M1"))
-         (m2 (make-mutex :name "M2"))
-         (s1 (make-semaphore :name "S1"))
-         (s2 (make-semaphore :name "S2"))
-         (t1 (make-thread
-              (lambda ()
-                (with-mutex (m1)
-                  (signal-semaphore s1)
-                  (wait-on-semaphore s2)
-                  (with-mutex (m2)
-                    :ok)))
-              :name "T1")))
-    (prog (err)
-     :retry
-       (handler-bind ((thread-deadlock
-                       (lambda (e)
-                         (unless err
-                           ;; Make sure we can print the condition
-                           ;; while it's active
-                           (let ((*print-circle* nil))
-                             (setf err (princ-to-string e)))
-                           (go :retry)))))
-         (when err
-           (sleep 1))
-         (assert (eq :ok (with-mutex (m2)
-                           (unless err
-                             (signal-semaphore s2)
-                             (wait-on-semaphore s1)
-                             (sleep 1))
-                           (with-mutex (m1)
-                             :ok)))))
-       (assert (stringp err)))
-    (assert (eq :ok (join-thread t1)))))
-
-(with-test (:name :deadlock-detection.3  :skipped-on (not :sb-thread))
-  (let* ((m1 (make-mutex :name "M1"))
-         (m2 (make-mutex :name "M2"))
-         (s1 (make-semaphore :name "S1"))
-         (s2 (make-semaphore :name "S2"))
-         (t1 (make-thread
-              (lambda ()
-                (with-mutex (m1)
-                  (signal-semaphore s1)
-                  (wait-on-semaphore s2)
-                  (with-mutex (m2)
-                    :ok)))
-              :name "T1")))
-    ;; Currently we don't consider it a deadlock
-    ;; if there is a timeout in the chain.
-    (assert (eq :deadline
-                (handler-case
-                    (with-mutex (m2)
-                      (signal-semaphore s2)
-                      (wait-on-semaphore s1)
-                      (sleep 1)
-                      (sb-sys:with-deadline (:seconds 0.1)
-                        (with-mutex (m1)
-                          :ok)))
-                  (sb-sys:deadline-timeout ()
-                    :deadline)
-                  (thread-deadlock ()
-                    :deadlock))))
-    (assert (eq :ok (join-thread t1)))))
-
 #+sb-thread
 (with-test (:name :pass-arguments-to-thread)
   (assert (= 3 (join-thread (make-thread #'+ :arguments '(1 2))))))
