@@ -25,8 +25,48 @@
                (specifier-type '(and (satisfies foo) fixnum))))
 (assert (type= (specifier-type '(member 1 2 3))
                (specifier-type '(member 2 3 1))))
-(assert (type= (specifier-type '(and (member 1.0 2 3) single-float))
-               (specifier-type '(member 1.0))))
+(assert (type= (specifier-type '(and (member $1.0 2 3) single-float))
+               (specifier-type '(member $1.0))))
+
+(assert (typep "hello" '(and array (not (array t)))))
+(assert (typep "hello" 'string))
+(assert (typep "hello" 'simple-string))
+(assert (typep "hello" 'unboxed-array))
+(assert (typep "hello" 'simple-unboxed-array))
+
+(assert (typep #*101 '(and array (not (array t)))))
+(assert (typep #*101 'bit-vector))
+(assert (typep #*101 'simple-bit-vector))
+(assert (typep #*101 'unboxed-array))
+(assert (typep #*101 'simple-unboxed-array))
+
+;;; When the host does not have (UNSIGNED-BYTE n), this makes an excellent test.
+;;; When the host *does* have it, this test is "suspicious" (in the sense that
+;;; it would not necessarily detect a bug in our portable array logic),
+;;; but is nonetheless valid, and especially since most other lisps don't
+;;; have (UNSIGNED-BYTE 2), it's a pretty reasonable thing to check.
+(dovector (x sb-vm:*specialized-array-element-type-properties*)
+  (let ((et (sb-vm:saetp-specifier x)))
+    ;; Test the numeric array specializations.
+    (unless (member et '(nil t base-char character))
+      (let ((a (make-array 11 :element-type et)))
+        (assert (type= (ctype-of a) (specifier-type `(simple-array ,et (11)))))
+        (assert (typep a '(and array (not (array t)))))
+        (assert (typep a `(simple-array ,et (11))))
+        (assert (typep a `(array ,et (11))))
+        (dolist (type-atom '(unboxed-array simple-unboxed-array))
+          (assert (typep a type-atom))
+          (assert (typep a `(,type-atom *)))
+          (assert (typep a `(,type-atom (*))))
+          (assert (typep a `(,type-atom (11)))))))))
+
+;;; Here it doesn't matter what we specify as element-type to MAKE-ARRAY
+;;; because it introspects as if it were SIMPLE-VECTOR due to
+;;; non-use of make-specialized-array.
+;;; (Note use of CL:MAKE-ARRAY. This file of tests causes symbol lookup
+;;; to default to using the SB-XC symbol otherwise)
+(assert (type= (ctype-of (cl:make-array 11 :element-type '(signed-byte 8)))
+               (specifier-type '(simple-vector 11))))
 
 (assert (typep #(1 2 3) 'simple-vector))
 (assert (typep #(1 2 3) 'vector))
@@ -81,12 +121,13 @@
                (type-union (specifier-type 'vector) (specifier-type 'list))))
 (assert (type= (specifier-type 'list)
                (type-union (specifier-type 'cons) (specifier-type 'list))))
-(assert (not (csubtypep (type-union (specifier-type 'list)
-                                    (specifier-type '(satisfies foo)))
-                        (specifier-type 'list))))
-(assert (csubtypep (specifier-type 'list)
-                   (type-union (specifier-type 'list)
-                               (specifier-type '(satisfies foo)))))
+(let ((sb-kernel::*xtypep-uncertainty-action* nil))
+  (assert (not (csubtypep (type-union (specifier-type 'list)
+                                      (specifier-type '(satisfies foo)))
+                          (specifier-type 'list))))
+  (assert (csubtypep (specifier-type 'list)
+                     (type-union (specifier-type 'list)
+                                 (specifier-type '(satisfies foo))))))
 
 ;;; Identities should be identities.
 (dolist (type-specifier '(nil
@@ -187,22 +228,14 @@
 (assert (type= (type-intersection (specifier-type 'number)
                                   (specifier-type 'integer))
                (specifier-type 'integer)))
-(assert (null (type-intersection2 (specifier-type 'symbol)
-                                  (specifier-type '(satisfies foo)))))
-(assert (intersection-type-p (specifier-type '(and symbol (satisfies foo)))))
-;; the target compiler returns NIL for this CTYPEP call involving KEYWORDP.
-;; therefore the cross-compiler should too.
-(assert (not (ctypep :x86 (specifier-type '(satisfies keywordp)))))
-(let* ((info (sb-int:info :function :info 'keywordp))
-       (attributes (sb-c::fun-info-attributes info)))
-  (unwind-protect
-       (progn
-         (setf (sb-c::fun-info-attributes info)
-               (logior attributes (sb-c::ir1-attributes sb-c::foldable)))
-         (assert (ctypep :x86 (specifier-type '(satisfies keywordp)))))
-    (setf (sb-c::fun-info-attributes info) attributes)))
-(assert (not (type= (specifier-type '(member :x86))
-                    (specifier-type '(and (member :x86) (satisfies keywordp))))))
+(let ((sb-kernel::*xtypep-uncertainty-action* nil))
+  (assert (null (type-intersection2 (specifier-type 'symbol)
+                                    (specifier-type '(satisfies foo)))))
+  (assert (intersection-type-p (specifier-type '(and symbol (satisfies foo))))))
+(assert (ctypep :x86 (specifier-type '(satisfies keywordp))))
+(assert (not (ctypep 'cons (specifier-type '(satisfies keywordp)))))
+(assert (type= (specifier-type '(member :x86))
+               (specifier-type '(and (member :x86) (satisfies keywordp)))))
 #+nil
 (let* ((type1 (specifier-type '(member :x86)))
        (type2 (specifier-type '(or keyword null)))
@@ -220,16 +253,16 @@
   (assert (type= isect (type-intersection type2 type1 type2)))
   (assert (type= isect (type-intersection type1 type1 type2 type1)))
   (assert (type= isect (type-intersection type1 type2 type1 type2))))
-(assert (csubtypep (specifier-type '(or (single-float -1.0 1.0)
-                                        (single-float 0.1)))
+(assert (csubtypep (specifier-type '(or (single-float $-1.0 $1.0)
+                                        (single-float $0.1)))
                    (specifier-type '(or (real -1 7)
-                                        (single-float 0.1)
-                                        (single-float -1.0 1.0)))))
+                                        (single-float $0.1)
+                                        (single-float $-1.0 $1.0)))))
 (assert (not (csubtypep (specifier-type '(or (real -1 7)
-                                             (single-float 0.1)
-                                             (single-float -1.0 1.0)))
-                        (specifier-type '(or (single-float -1.0 1.0)
-                                             (single-float 0.1))))))
+                                             (single-float $0.1)
+                                             (single-float $-1.0 $1.0)))
+                        (specifier-type '(or (single-float $-1.0 $1.0)
+                                             (single-float $0.1))))))
 
 (assert (typep #\, 'character))
 (assert (typep #\@ 'character))
@@ -311,11 +344,12 @@
   (assert (not yes))
   (assert win))
 ;; Used to run out of stack.
-(multiple-value-bind (yes win)
-    (handler-bind ((sb-kernel::cross-type-giving-up #'muffle-warning))
-      (subtypep 'null '(or unk0 unk1)))
-  (assert (not yes))
-  (assert (not win)))
+(let ((sb-kernel::*xtypep-uncertainty-action* nil))
+  (multiple-value-bind (yes win)
+      (handler-bind ((sb-kernel::cross-type-giving-up #'muffle-warning))
+        (subtypep 'null '(or unk0 unk1)))
+    (assert (not yes))
+    (assert (not win))))
 
 (multiple-value-bind (yes win)
     (subtypep '(and function instance) nil)
@@ -378,7 +412,20 @@
 
 #+x86-64
 (progn
-  (assert (= (sb-vm::immediate-constant-sc #c(0.0f0 0.0f0))
+  (assert (= (sb-vm::immediate-constant-sc (complex $0.0f0 $0.0f0))
              sb-vm::fp-complex-single-zero-sc-number))
-  (assert (= (sb-vm::immediate-constant-sc #c(0.0d0 0.0d0))
+  (assert (= (sb-vm::immediate-constant-sc (complex $0.0d0 $0.0d0))
              sb-vm::fp-complex-double-zero-sc-number)))
+
+;;; Unparse a union of (up to) 4 things depending on :sb-unicode as 2 things.
+(assert (equal (type-specifier (specifier-type '(or string null)))
+               '(or string null)))
+
+(multiple-value-bind (result exactp)
+    (sb-vm::primitive-type (specifier-type 'list))
+  (assert (and (eq result (sb-vm::primitive-type-or-lose 'list))
+               exactp)))
+(multiple-value-bind (result exactp)
+    (sb-vm::primitive-type (specifier-type 'cons))
+  (assert (and (eq result (sb-vm::primitive-type-or-lose 'list))
+               (not exactp))))
